#include <windows.h>
#include <time.h>
#include <stdio.h>
#include "data.h"
#include "evaluate.h"
#include "gui.h"

// --------------------------------------------------
// Constants for evaluation features
//
#define DOUBLED_V 15
#define ISLAND_V 10
#define ISOLATED_V 20
#define BACKWARD1_V 20

const int PASSED_RANK[9] = { 0, 0, 20, 25, 30, 44, 71, 104, 0 };
const int Q_Tropism[16] = { 12, 19, 16, 13,10, 9, 8, 7, 6, 5, 4, 2, 1, 0, 0, 0 }; 
const int R_Tropism[16] = { 12, 7, 2,  0, 0, 0, 0, 0 }; 
const int N_Tropism[16] = { 12, 10, 10, 12, 11, 9, 7, 6, 4, 2, 0, 0, 0, 0, 0, 0 }; 
const int B_Tropism[16] = { 11, 7, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }; 


#define CASTLE_LOST -29 // Penalty for losing castling rights ( without castling )
#define ROOK_HALF 15
#define ROOK_FULL 25 // Rook on open file, or behind a passed pawn
#define ROOKS_7 24
#define ROOKS_7K8 95
#define ROOK_CRAMPED -18 // Rook cannot move horizontally

#define BISHOP_CRAMPED -13
#define BISHOP_SEMICRAMPED -7
#define BISHOP_PAIR 15

#define KING_PAWNCOVER 11
#define KING_PAWNCOVERFRONT 16

#define SAFE_FROM_PAWN_SQ 10
		// Pieces can't
	/*	int i = 0;
		while (SPawns.Sqs [i] != 0 && i<8)
			{
			int nPiece = Board.Sq[SPawns.Sqs[i]];
			if (nPiece > BPAWN && nPiece <= BKNIGHT) eval-= SAFE_FROM_PAWN_SQ;
			i++;
			}
		i=8;
		while (SPawns.Sqs [i] != 0 && i<16)
			{
			int nPiece = Board.Sq[SPawns.Sqs[i]];
			if (nPiece > WPAWN && nPiece <= WKNIGHT) eval+= SAFE_FROM_PAWN_SQ;
			i++;
			}*/
	 // char Sqs[16];

extern g_nQueenSide;
// ==========================
// Pawn Hashing
// ==========================
// DATA
unsigned int WPHashSq [128];
unsigned int BPHashSq [128];
#define PAWNHASHTABLESIZE	((2 << 16) - 1)

PawnHashEntry* PawnTable = (PawnHashEntry*) malloc( sizeof (PawnHashEntry) * PAWNHASHTABLESIZE + 1);

// --------------------------------------------------
void Clear_PawnHash ( void )
{
ZeroMemory (PawnTable, sizeof (PawnHashEntry) * PAWNHASHTABLESIZE );
}

// --------------------------------------------------
//  Pawn LookUp
// --------------------------------------------------
int inline PawnHash_Lookup (PawnHashEntry &TEntry, const unsigned int &nCheckSum)
{
	int index = nCheckSum & PAWNHASHTABLESIZE;

	if (PawnTable [ index ].checksum == nCheckSum)
		{
		TEntry = PawnTable[ index ];
		return TRUE;
        }
	return FALSE;
}

// --------------------------------------------------
//  Pawn HashBoard
// --------------------------------------------------
unsigned int Pawn_HashBoard (SBoard &Board)
{
 unsigned int sum = 0;
 int i;

	 for (i = 0; i <= 8; i++)
		{
		if (Board.WPiece [i] != CAPTURED) sum += WPHashSq [ Board.WPiece [i] ];
		if (Board.BPiece [i] != CAPTURED) sum += BPHashSq [ Board.BPiece [i] ];
		}
 return sum;
}

//
// So it can display this term in the board flags display
//
int Liquidate ( SBoard &Board )
{
int eval = MatEval (Board, Board.nWP, Board.nBP, Board.MatVal);
return eval;
}


//
// Evaluate Rooks
//
int inline EvalRookBlack (const SBoard &Board, const int &nRookSq,  const unsigned char &cOpenW, const unsigned char &cOpenB  )
	{
	 int HMoves = 0;
	 int eval = 0;

	 if ( Board.Sq [ nRookSq+1 ] == EMPTY) {HMoves++;if ( Board.Sq [ nRookSq+2 ] == EMPTY) HMoves++;}
	 if ( Board.Sq [ nRookSq-1 ] == EMPTY) {HMoves++;if ( Board.Sq [ nRookSq-2 ] == EMPTY) HMoves++;}

	 if ( (cOpenB & BITMASK[ SQFILE[nRookSq] ]) == 0)
		{
		 eval -= R_Tropism [ abs( SQFILE [nRookSq] - SQFILE [Board.WPiece[15]] ) ];
		 if ( (cOpenW & BITMASK[ SQFILE[nRookSq] ]) == 0) eval -= ROOK_FULL;
			else eval -= ROOK_HALF;
		 if (HMoves <= 0) eval-=ROOK_CRAMPED;
		}
	 else if (HMoves<=1) eval-=ROOK_CRAMPED;
	 return eval;
	}

int inline EvalRookWhite (const SBoard &Board, const int &nRookSq,  const unsigned char &cOpenW, const unsigned char &cOpenB  )
	{
	 int HMoves = 0;
	 int eval = 0;

	 if ( Board.Sq [ nRookSq+1 ] == EMPTY) {HMoves++;if ( Board.Sq [ nRookSq+2 ] == EMPTY) HMoves++;}
	 if ( Board.Sq [ nRookSq-1 ] == EMPTY) {HMoves++;if ( Board.Sq [ nRookSq-2 ] == EMPTY) HMoves++;}

	 if ( (cOpenW & BITMASK[ SQFILE[nRookSq] ]) == 0)
		{
 		 eval += R_Tropism [ abs(SQFILE [nRookSq] - SQFILE [Board.BPiece[15]] ) ];
		 if ( (cOpenB & BITMASK[ SQFILE[nRookSq] ]) == 0) eval += ROOK_FULL;
			else eval += ROOK_HALF;
		 if (HMoves <= 0) eval+=ROOK_CRAMPED;
		}
	 else if (HMoves<=1) eval+=ROOK_CRAMPED;
	 return eval;
	}


int inline EvalBishop (const SBoard &Board, const int &nSq)
{
int nEval = 0;
int Moves = 0;
if (Board.Sq [nSq+11]==EMPTY) Moves++;
if (Board.Sq [nSq+9] ==EMPTY) Moves++;
if (Board.Sq [nSq-11]==EMPTY) Moves++;
if (Board.Sq [nSq-9] ==EMPTY) Moves++;
if (Moves == 0) nEval += BISHOP_CRAMPED;
if (Moves == 1) nEval += BISHOP_SEMICRAMPED;
return nEval;
}
//
//
// For Displaying separate parts of the eval
//
//
void PawnRookEval ( SBoard &Board, int &PawnEval, int &RookEval )
{
		PawnHashEntry TEntry;
		if ( PawnHash_Lookup ( TEntry, Board.PawnHash ) != TRUE ) EvalPawns ( Board, TEntry);
		PawnEval = TEntry.eval;

	    if (Board.nBP <=3 && Board.nWP <= 3) PawnEval+= TEntry.MatEnd; 
			else PawnEval+=TEntry.Mat;

		int Rook7 = 0;

		RookEval = 0;
		int nRookSq = Board.WPiece [12];
		if (nRookSq != CAPTURED) RookEval+=EvalRookWhite ( Board, nRookSq, TEntry.OpenFileW, TEntry.OpenFileB );

		nRookSq = Board.WPiece [13];
		if (nRookSq != CAPTURED) RookEval+=EvalRookWhite ( Board, nRookSq, TEntry.OpenFileW, TEntry.OpenFileB);

		Rook7 = 0;
		nRookSq = Board.BPiece [12];
		if (nRookSq != CAPTURED) RookEval+=EvalRookBlack ( Board, nRookSq, TEntry.OpenFileW, TEntry.OpenFileB );

		nRookSq = Board.BPiece [13];
		if (nRookSq != CAPTURED) RookEval+=EvalRookBlack ( Board, nRookSq, TEntry.OpenFileW, TEntry.OpenFileB );

		if ( SQRANK[ Board.WPiece [13]  ] == 7 && SQRANK[ Board.WPiece [12] ] == 7) 
						{if (SQRANK [ Board.BPiece [15]  ] == 8) RookEval+= ROOKS_7K8;
								else RookEval+= ROOKS_7;
						}
		if ( SQRANK[ Board.BPiece [13]  ] == 2 && SQRANK[ Board.BPiece [12] ] == 2) 
						{if (SQRANK [ Board.WPiece [15]  ] == 1) RookEval-= ROOKS_7K8;
								else RookEval-= ROOKS_7;
						}

}


//
// MatingEval is called when neither side has any pawns left on the board
//
//
int MatingEval (const SBoard &Board, const int &nWKingSq, const int &nBKingSq, int eval)
	{
	int nSq;

		// Is there sufficicent Material to Mate?
		if (Board.MatVal <=0 && Board.nBP <= 1 && Board.nBP == Board.nBMinors) return 0;
		if (Board.MatVal >= 0 && Board.nWP <= 1 && Board.nWP == Board.nWMinors) return 0;
			
		// BLACK has no pieces or pawns
		if (Board.nBP == 0) 
			{
			if (Board.nWMinors == 2 && Board.nWP == 2)  
				{
				nSq = 25;
				// FixMe: Promoted to Bishop?
				if (Board.WPiece [10]!= CAPTURED) nSq = Board.WPiece[10];
				if (Board.WPiece [11]!= CAPTURED) nSq = Board.WPiece[11];
				if (SQCOLOR [ nSq ] == 2)
					eval += ((WKingValueKBNK[ nBKingSq ]*3) + 50); 
				else eval += WKingValueKBNK[ SQ ( SQFILE[nBKingSq]-1, SQRANK[nBKingSq]-1 )]*3 + 50;
				}
				else eval -= ((WKingValueKRK[nBKingSq]<<1) - 300);

			// Encourage White King to move next to Black King
			eval -= (abs (SQRANK [nBKingSq] - SQRANK [nWKingSq] ) + abs (SQFILE [nBKingSq] - SQFILE [nWKingSq] )) * 4;
			return eval;
			}
		// WHITE has no pieces or pawns
		if (Board.nWP == 0) 
			{
			if (Board.nBMinors == 2 && Board.nBP == 2)  
				{
				nSq = 25;
				if (Board.BPiece [10]!= CAPTURED) nSq = Board.BPiece[10];
				if (Board.BPiece [11]!= CAPTURED) nSq = Board.BPiece[11];

				if (SQCOLOR [ nSq ] == 2) eval -= ((WKingValueKBNK[nWKingSq]*3) + 50); 
					else eval -= WKingValueKBNK[ SQ ( SQFILE[nWKingSq]-1, SQRANK[nWKingSq]-1 )]*3 + 50;
				}
				else eval += ((WKingValueKRK[nWKingSq]<<1) - 300);

			eval += (abs (SQRANK [nBKingSq] - SQRANK [nWKingSq] ) + abs (SQFILE [nBKingSq] - SQFILE [nWKingSq] )) * 4;
			return eval;
			}
		// Both Sides have pieces of unequal value, do King Centralization
		if (abs (Board.MatVal) > 30 )
			{
			eval += WKingValueEnd [ nWKingSq ];
			eval -= WKingValueEnd [ nBKingSq ];
			}
		// One Side is up only 1 minor piece, should probably be a draw
		if (Board.nWP == (Board.nBP+1) && Board.nWMinors == (Board.nBMinors+1) && Board.MatVal < 400 ) 
			eval = (eval >> 3);
		if (Board.nBP == (Board.nWP+1) && Board.nBMinors == (Board.nWMinors+1) && Board.MatVal > -400 ) 
			eval = (eval >> 3);

		return eval;
	}


//
// Number of moves for a king to walk form nSq1 to nSq2
//
int inline ChessDist (const int &nSq1, const int &nSq2)
	{
	int Dist1 = abs (SQRANK[nSq1] - SQRANK[nSq2]);
	int Dist2 = abs (SQFILE[nSq1] - SQFILE[nSq2]);
	if (Dist1 > Dist2) return Dist1;
	return Dist2;
	}

//
// Finish this Sometime
//
void EvalPawnEnding (const SBoard &Board, const int &nWKingSq, const int &nBKingSq)
{
int nLoc;
if (Board.nBP == 0) 
	{
	for (int i = 0; i < 8; i++)
		{
		nLoc = Board.WPiece[i];
		if (nLoc !=CAPTURED)
			if (Board.Sq [nLoc] == WPAWN)
				{

				}
		}
	}
}

//
// Slow Chess now has its own table to tell if an endgame with only 1 pawn (+kings) is won or drawn
//
// You can use this code and bitbase freely in your own program.
// 
const int KPK_SIZE = (8 * 64 * 2 * 24);
unsigned char KPKResults [ KPK_SIZE];
int g_bKPKInit = FALSE;
//
// Black calls the same routine but flips vertically the sqs and swaps the king sqs before calling it
//
void KingPawnProbe (int nWKingSq, int nBKingSq, int nPawnSq, int nTM, int &eval, const int nWin)
{
// Get Entry Key... May have to flip horizontally because table only stores positions for 
// the pawn on one side.
int nEntry; 
if ( (nPawnSq&7) > 3) nEntry = (FlipH64[nWKingSq]) + (FlipH64[nBKingSq]<<6) + (nTM<<12) + ((PSQ[nPawnSq]) <<13);
	else nEntry = (nWKingSq) + (nBKingSq<<6) + (nTM<<12) + ((PSQ[nPawnSq]) <<13);
if (nEntry <0 || nEntry > KPK_SIZE*8) return; // Something went wrong

unsigned char ucTest = KPKResults [ (nEntry>>3) ];
if ( (ucTest & (1<<(nEntry&7)) ) != 0) eval+=nWin; // Winning endgame
	else eval = 0; // Drawn Endgame
}

// Load Table from disk
int InitKingPawnProbe ( )
	{
	FILE* pFile;
	pFile = fopen ("KPK.sbb", "rb");
	if (pFile == NULL) return FALSE;
	fread (KPKResults, 1, KPK_SIZE, pFile);
	fclose (pFile);
	g_bKPKInit = TRUE;
	return TRUE;
	}

//
// Called when Pieces <= 1 for each side
// RookPawn Endgame Draws
// and opposite bishop endgames
//
short OnePieceEndgame ( const SBoard &Board, int &eval, const int &nWKingSq, const int &nBKingSq, const PawnHashEntry &SPawns)
{
// King Pawn King Endgame?
if (Board.nWP == 0 && Board.nBP == 0 &&	g_bKPKInit == TRUE)
	{
	// Probe the King Pawn King win/draw bitbase
	if (Board.nBPawns == 0 && Board.nWPawns == 1)
		{
		int i = 0, nPawnSq = CAPTURED;
		while (i < 8 && nPawnSq == CAPTURED) nPawnSq = Board.WPiece [i++];
		KingPawnProbe (SQ64[nWKingSq], SQ64[nBKingSq], SQ64[nPawnSq], Board.SideToMove == WHITE ? 1:0, eval, 500);
		return eval;
		}
	if (Board.nBPawns == 1 && Board.nWPawns == 0)
		{
		int i = 0, nPawnSq = CAPTURED;
		while (i < 8 && nPawnSq == CAPTURED) nPawnSq = Board.BPiece [i++];
		KingPawnProbe (63-SQ64[nBKingSq], 63-SQ64[nWKingSq], 63-SQ64[nPawnSq], Board.SideToMove == BLACK ? 1:0, eval, -500 );
		return eval;
		}
	}

// Get Bishop Sqs
	int nwBishSq = Board.WPiece[10];
	if (nwBishSq == CAPTURED) nwBishSq = Board.WPiece[11];
	int nbBishSq = Board.BPiece[10];
	if (nbBishSq == CAPTURED) nbBishSq = Board.BPiece[11];

// Pawns on A or H file draws
if (eval>0)
	{
	if ( (Board.nWP == 0 || SQCOLOR [nwBishSq] == 1 ) && (SPawns.flags&4) ) 
		{
		if (nBKingSq == 22 || nBKingSq == 23 || nBKingSq == 32 || nBKingSq == 33 ) return 0;
		if (ChessDist (nBKingSq, 22) <= ChessDist (nWKingSq, 22) ) eval = (eval>>1);
		}
	if ( (Board.nWP == 0 || SQCOLOR [nwBishSq] == 2 ) && (SPawns.flags&8) ) 
		{
		if (nBKingSq == 29 || nBKingSq == 28 || nBKingSq == 39 || nBKingSq == 38 ) return 0;
		if (ChessDist (nBKingSq, 29) <= ChessDist (nWKingSq, 29) ) eval = (eval>>1);
		}
	}
if (eval<0)
	{
	if ( (Board.nBP == 0 || SQCOLOR [nbBishSq] == 2 ) && (SPawns.flags&1) ) 
		{
		if (nWKingSq == 92 || nWKingSq == 93 || nWKingSq == 82 || nWKingSq == 83 ) return 0;
		if (ChessDist (nWKingSq, 92) <= ChessDist (nBKingSq, 92) ) eval = (eval>>1);
		}
	if ( (Board.nBP == 0 || SQCOLOR [nbBishSq] == 1 ) && (SPawns.flags&2) ) 
		{
		if (nWKingSq == 99 || nWKingSq == 98 || nWKingSq == 89 || nWKingSq == 88 ) return 0;
		if (ChessDist (nWKingSq, 99) <= ChessDist (nBKingSq, 99) ) eval = (eval>>1);
		}
	}

// Pawn Location and King Centralization
eval += SPawns.MatEnd + WKingValueEnd [ nWKingSq ] - WKingValueEnd [ nBKingSq ];

if ( (Board.Sq [ nWKingSq - 10 ]&7) == WPAWN || (Board.Sq [ nWKingSq - 20 ]&7) == WPAWN) eval+=8;
if ( (Board.Sq [ nWKingSq - 11 ]&7) == WPAWN || (Board.Sq [ nWKingSq - 1 ]&7) == WPAWN) eval+=8;
if ( (Board.Sq [ nWKingSq - 9 ]&7)  == WPAWN || (Board.Sq [ nWKingSq + 1 ]&7) == WPAWN) eval+=8;

if ( (Board.Sq [ nBKingSq + 10 ]&7) == WPAWN || (Board.Sq [ nBKingSq + 20 ]&7) == WPAWN) eval-=8;
if ( (Board.Sq [ nBKingSq + 11 ]&7) == WPAWN || (Board.Sq [ nBKingSq + 1 ]&7) == WPAWN) eval-=8;
if ( (Board.Sq [ nBKingSq + 9 ]&7)  == WPAWN || (Board.Sq [ nBKingSq - 1 ]&7) == WPAWN) eval-=8;

// Opposite Colored Bishops
if (nbBishSq != CAPTURED && nwBishSq != CAPTURED && SQCOLOR [nwBishSq] != SQCOLOR [nbBishSq] )
	{
	if ( (Board.nWPawns + Board.nBPawns) == 1) eval = (eval>>3);
	else if ( abs (Board.nWPawns- Board.nBPawns) <= 1)	eval = (eval>>1);
	else												eval = (eval>>1) + (eval>>2);
	}

return (short)eval;
}

// 
// Don't return good evals for a side if it can't win (or is very unlikely to win.)
//
int WhiteCantWin (const SBoard &Board)
{
	if (Board.nWP <= 1 && Board.nWP == Board.nWMinors) return TRUE;
	if (Board.nWP == (Board.nBP+1) && Board.nWMinors == (Board.nBMinors+1) && (Board.MatVal+Board.nBPawns*100) < 400 ) return TRUE;
	return FALSE;
}

int BlackCantWin (const SBoard &Board )
{
	if (Board.nBP <= 1 && Board.nBP == Board.nBMinors) return TRUE;
	if (Board.nBP == (Board.nWP+1) && Board.nBMinors == (Board.nWMinors+1) && (Board.MatVal-Board.nWPawns*100) > -400 ) return TRUE;
	return FALSE;
}
// ==========================
//
//  BOARD EVALUATION
//
// (This isn't the full Eval Function, some parts are elsewhere and some is updated incrementally )
// 1 Pawn = 100
// ==========================
short EvaluateBoard (const SBoard &Board, int alpha, int beta)
     {
     int eval = 0, nSq;

	    // piece count  ( incrementally updated by domove )
        eval += Board.MatVal;

		// More Pieces/ Number of pieces left
		eval += MatEval (Board, Board.nWP, Board.nBP, Board.MatVal);

		int nWKingSq = Board.WPiece [ 15 ];
		int nBKingSq = Board.BPiece [ 15 ];

		// No Pawn Endgame or pieces, so low ply can mate with KRK and KQK
		if (Board.nBPawns == 0 && Board.nWPawns == 0)
			{
			return (short)MatingEval ( Board, nWKingSq, nBKingSq, eval );
			}

		// Rooks 7th Rank
		if ( SQRANK[ Board.WPiece [13]  ] == 7 && SQRANK[ Board.WPiece [12] ] == 7) 
						{if (SQRANK [ nBKingSq ] == 8) eval+= ROOKS_7K8;
								else eval+= ROOKS_7;
						}
		if ( SQRANK[ Board.BPiece [13]  ] == 2 && SQRANK[ Board.BPiece [12] ] == 2) 
						{if (SQRANK [ nWKingSq ] == 1) eval-= ROOKS_7K8;
								else eval-= ROOKS_7;
						}

		// Piece Square Values
		eval += Board.TempValue; 

		// Check for position in pawn table
		PawnHashEntry SPawns;
		if ( PawnHash_Lookup ( SPawns, Board.PawnHash ) != TRUE ) EvalPawns ( Board, SPawns);
		eval += SPawns.eval;

		// Maybe one side has only pawns but the other only has 1 minor piece which can't mate (Or is only up 1 minor)
		// (If both sides had no pawns mating eval was called and the function exited before this)
		if (Board.nWPawns == 0 && eval >=0)
			{
			if (WhiteCantWin (Board)) return -1;
			}
		if (Board.nBPawns == 0 && eval <=0)
			{
			if (BlackCantWin (Board)) return 1;
			}

		if (Board.nWP <= 1 && Board.nBP <=1)
			{
			return OnePieceEndgame ( Board, eval, nWKingSq, nBKingSq, SPawns);
			}

		// Early exit on the eval for fail high. Doesn't help much, but does add a little extra speed.
		// For fail lows I get 1 good value, then don't call eval from the quiescence if the capture isn't good enough
		if (Board.SideToMove == BLACK && eval < (alpha-130) ) return eval;
		if (Board.SideToMove == WHITE && eval > (beta +130) ) return eval;

		int endNum = 3;
		if (Board.WPiece[14]!=CAPTURED || Board.BPiece[14]!=CAPTURED ) endNum = 2;
		// Endgame
	    if (Board.nBP <=endNum && Board.nWP <= endNum) 
				{
				eval += SPawns.MatEnd; // Pawn sq vals end
				eval += WKingValueEnd [ nWKingSq ];
				eval -= WKingValueEnd [ nBKingSq ];
				// FixMe: PassedPawn Support? (Near Pawns == Good)
				if ( (Board.Sq [ nWKingSq - 10 ]&7) == WPAWN || (Board.Sq [ nWKingSq - 20 ]&7) == WPAWN) eval+=8;
				if ( (Board.Sq [ nWKingSq - 11 ]&7) == WPAWN || (Board.Sq [ nWKingSq - 1 ]&7) == WPAWN) eval+=8;
				if ( (Board.Sq [ nWKingSq - 9 ]&7)  == WPAWN || (Board.Sq [ nWKingSq + 1 ]&7) == WPAWN) eval+=8;

				if ( (Board.Sq [ nBKingSq + 10 ]&7) == WPAWN || (Board.Sq [ nBKingSq + 20 ]&7) == WPAWN) eval-=8;
				if ( (Board.Sq [ nBKingSq + 11 ]&7) == WPAWN || (Board.Sq [ nBKingSq + 1 ]&7) == WPAWN) eval-=8;
				if ( (Board.Sq [ nBKingSq + 9 ]&7)  == WPAWN || (Board.Sq [ nBKingSq - 1 ]&7) == WPAWN) eval-=8;
				}
		else
			{
	        // Penalty for losing Castling rights
		    if (Board.WCastle == 3) eval+=CASTLE_LOST;
			if (Board.BCastle == 3) eval-=CASTLE_LOST;

			eval+= SPawns.Mat; // Pawn sq vals opening/mid
			// King behind pawns in opening & midgame
			if (Board.WCastle >= 3)
				{
				eval-=KING_PAWNCOVERFRONT;
				if (Board.Sq [ nWKingSq - 10 ] == WPAWN || Board.Sq [ nWKingSq - 20 ] == WPAWN) eval+=KING_PAWNCOVERFRONT;
				if (Board.Sq [ nWKingSq - 11 ] == WPAWN || Board.Sq [ nWKingSq - 21 ] == WPAWN ) eval+=KING_PAWNCOVER;
				if (Board.Sq [ nWKingSq - 9 ] == WPAWN || Board.Sq [ nWKingSq - 19 ] == WPAWN) eval+=KING_PAWNCOVER;
				}
			if (Board.BCastle >= 3)
				{
				eval+=KING_PAWNCOVERFRONT;
				if (Board.Sq [ nBKingSq + 10 ]== BPAWN ||  Board.Sq [ nBKingSq + 20 ] ==BPAWN) eval-=KING_PAWNCOVERFRONT;
				if (Board.Sq [ nBKingSq + 11 ]== BPAWN || Board.Sq [ nBKingSq + 21 ] == BPAWN) eval-=KING_PAWNCOVER;
				if (Board.Sq [ nBKingSq + 9 ] == BPAWN || Board.Sq [ nBKingSq + 19 ] == BPAWN) eval-=KING_PAWNCOVER;
				}

			if (Board.GameTime < 48) // Keep King back in opening
				{
				eval += SquareVals [ WKING * 128 + nWKingSq ];
				eval += SquareVals [ BKING * 128 + nBKingSq ];
				}
			}

		// ROOKs EVALUATION

		nSq = Board.WPiece [12];
		if (nSq != CAPTURED) eval+=EvalRookWhite ( Board, nSq,  SPawns.OpenFileW, SPawns.OpenFileB );

		nSq = Board.WPiece [13];
		if (nSq != CAPTURED) eval+=EvalRookWhite ( Board, nSq,  SPawns.OpenFileW, SPawns.OpenFileB );

		nSq = Board.BPiece [12];
		if (nSq != CAPTURED) eval+= EvalRookBlack ( Board, nSq,  SPawns.OpenFileW, SPawns.OpenFileB );

		nSq = Board.BPiece [13];
		if (nSq != CAPTURED) eval+= EvalRookBlack ( Board, nSq,  SPawns.OpenFileW, SPawns.OpenFileB );

		// Queen
		nSq = Board.WPiece[14];
		if (nSq != CAPTURED) eval+=Q_Tropism [ abs ( SQFILE [nBKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nBKingSq] - SQRANK [nSq]) ];

		nSq = Board.BPiece[14];
		if (nSq != CAPTURED) eval-=Q_Tropism [  abs ( SQFILE [nWKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nWKingSq] - SQRANK [nSq]) ];

 		// Knights
		nSq = Board.WPiece[8];
		if (nSq != CAPTURED) eval+=N_Tropism [ abs ( SQFILE [nBKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nBKingSq] - SQRANK [nSq]) ];
		nSq = Board.WPiece[9];
		if (nSq != CAPTURED) eval+=N_Tropism [ abs ( SQFILE [nBKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nBKingSq] - SQRANK [nSq]) ];

		nSq = Board.BPiece[8];
		if (nSq != CAPTURED) eval-=N_Tropism [  abs ( SQFILE [nWKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nWKingSq] - SQRANK [nSq]) ];
		nSq = Board.BPiece[9];
		if (nSq != CAPTURED) eval-=N_Tropism [  abs ( SQFILE [nWKingSq] - SQFILE [nSq] ) + abs ( SQRANK [nWKingSq] - SQRANK [nSq]) ];

		// Bishops
		int nBish = 0;
		nSq = Board.WPiece[10];
		if (nSq != CAPTURED) {eval += EvalBishop ( Board, nSq);
							  eval += B_Tropism [ abs(SQDIAG1[nSq] - SQDIAG1 [nBKingSq]) ] + B_Tropism [ abs(SQDIAG2[nSq] - SQDIAG2 [nBKingSq]) ];
							  nBish++;
							}
		nSq = Board.WPiece[11];
		if (nSq != CAPTURED) {eval += EvalBishop ( Board, nSq);
							  eval += B_Tropism [ abs(SQDIAG1[nSq] - SQDIAG1 [nBKingSq]) ] + B_Tropism [ abs(SQDIAG2[nSq] - SQDIAG2 [nBKingSq]) ];
							  nBish++;
							}
		if (nBish == 2) eval += BISHOP_PAIR;
		nBish = 0;
		nSq = Board.BPiece[10];
		if (nSq != CAPTURED) {eval -= EvalBishop ( Board, nSq);
							  eval -= (B_Tropism [ abs(SQDIAG1[nSq] - SQDIAG1 [nWKingSq]) ] + B_Tropism [ abs(SQDIAG2[nSq] - SQDIAG2 [nWKingSq]) ]);
							  nBish++;
							}
		nSq = Board.BPiece[11];
		if (nSq != CAPTURED) {eval -= EvalBishop ( Board, nSq);
							  eval -= (B_Tropism [ abs(SQDIAG1[nSq] - SQDIAG1 [nWKingSq]) ] + B_Tropism [ abs(SQDIAG2[nSq] - SQDIAG2 [nWKingSq]) ]);
							  nBish++;
							}
		if (nBish == 2) eval -= BISHOP_PAIR;

		return (short)eval;
	}


// --------------------------------------------------
//
//  Evaluate the pawn structure on the passed board
//
// --------------------------------------------------
void EvalPawns ( const SBoard &Board, PawnHashEntry &TEntry)
{
TEntry.eval = 0;
TEntry.MatEnd = 0;
TEntry.Mat = 0;
TEntry.OpenFileW = 0;
TEntry.OpenFileB = 0;
TEntry.WeakFileB = 0;
TEntry.WeakFileW = 0;
TEntry.flags = 0;

int x, nFile, nWSq = 0, nBSq = 8;
int WFile [10], BFile [10];
int WRankMin [10], BRankMin [10];
int WRankMax [10], BRankMax [10];

ZeroMemory (WFile, 10 * sizeof (int) );
ZeroMemory (BFile, 10 * sizeof (int) );
memset    (WRankMin,  20, 10 * sizeof (int) );
memset (BRankMin, 20, 10 * sizeof (int) );
memset (WRankMax, -20, 10 * sizeof (int) );
memset (BRankMax, -20, 10 * sizeof (int) );
//ZeroMemory (TEntry.Sqs, 16);

int nLoc;

// Compute Info
for (x = 0; x < 8; x++)
	{
	nLoc = Board.WPiece[x];
	if ( nLoc != CAPTURED && Board.Sq[nLoc] == WPAWN) 
		{
		TEntry.MatEnd	+= WPawnValueEnd [ nLoc ];
		if ( (g_nQueenSide&1) ) TEntry.Mat += WPawnValueQS [ nLoc ];
						   else TEntry.Mat += WPawnValue [ nLoc ]; // Square Table
		nFile = SQFILE[ nLoc ];
		TEntry.OpenFileW |= BITMASK [ nFile ];
		WFile[ nFile ]++;
		int nRank = SQRANK [ nLoc  ];
		if ( nRank < WRankMin[ nFile ] ) WRankMin[ nFile ]= SQRANK [ nLoc  ];
		if ( nRank > WRankMax[ nFile ] ) WRankMax[ nFile ]= SQRANK [ nLoc  ];
		}
	nLoc = Board.BPiece[x];
	if ( nLoc != CAPTURED  && Board.Sq[nLoc] == BPAWN) 
		{
		TEntry.MatEnd	+= BPawnValueEnd [ nLoc ];
		if ( (g_nQueenSide&2) ) TEntry.Mat += BPawnValueQS [ nLoc ];
						   else TEntry.Mat += BPawnValue [ nLoc ]; // Square Table
		nFile = SQFILE[  nLoc  ];
		TEntry.OpenFileB |= BITMASK [ nFile ];
		BFile[ nFile ]++;
		int nRank = SQRANK [ nLoc  ];
		if ( nRank < BRankMin[ nFile ] ) BRankMin[ nFile ]= SQRANK [ nLoc  ];
		if ( nRank > BRankMax[ nFile ] ) BRankMax[ nFile ]= SQRANK [ nLoc  ];
		}
	}

// Hmm.. might as well... for drawn rookspawn endings
if (BFile[1] == Board.nBPawns) TEntry.flags |=1;
if (BFile[8] == Board.nBPawns) TEntry.flags |=2;
if (WFile[1] == Board.nWPawns) TEntry.flags |=4;
if (WFile[8] == Board.nWPawns) TEntry.flags |=8;

// Double Pawns 
// Pawn Islands
int bStart = FALSE;
for (nFile = 1; nFile <= 8; nFile++) // WHITE
	{
	if (WFile[nFile]!= 0)
		{
		if (WFile[nFile] > 1) TEntry.eval -= DOUBLED_V * (WFile[nFile]-1);
		if (bStart == FALSE)
			{
			bStart = TRUE; 
			TEntry.eval -= ISLAND_V;
			}
		}
	else bStart = FALSE;
	}

bStart = FALSE;
for (nFile = 1; nFile <= 8; nFile++) // BLACK
	{
	if (BFile[nFile]!= 0)
		{
		if (BFile[nFile] > 1) TEntry.eval += DOUBLED_V * (BFile[nFile]-1);
		if (bStart == FALSE)
			{
			bStart = TRUE; 
			TEntry.eval += ISLAND_V;
			}
		}
	else bStart = FALSE;
	}

// Passed Pawns
// Isolated Pawns
// Backward Pawns  

for  (x = 0; x < 8; x++)
	{
	// WHITE
	nLoc = Board.WPiece[x];
	if (nLoc != CAPTURED && Board.Sq[nLoc] == WPAWN)
		{
		int nFile = SQFILE [ nLoc ];
		int nRank = SQRANK [ nLoc ];

		if (nRank >= BRankMax [ nFile+1 ]  && nRank >= BRankMax [ nFile-1 ] && nRank > BRankMax [ nFile ] && nRank == WRankMax [ nFile ])
			{
			TEntry.eval += (PASSED_RANK [ nRank ]);
			TEntry.OpenFileW ^= BITMASK [ nFile ]; // encourage rooks behind (Fixme: Rook must be behind)
			}

		if (WFile [ nFile-1] == 0 && WFile [ nFile +1] == 0  && nFile!=1 && nFile!= 8 ) // Isolated not on 1st or 8th file
				{ TEntry.eval -=ISOLATED_V;
				  TEntry.WeakFileW |= nFile;
				  //TEntry.Sqs [ nWSq++ ] = nLoc - 10;
				}

		else if (nRank < WRankMin [ nFile+1 ]  && nRank < WRankMin [ nFile-1 ] && nRank == WRankMax [ nFile ] ) 
				{
				//TEntry.Sqs [ nWSq++ ] = nLoc - 10;
				if ( ( (WFile [ nFile+1] == 0 || WRankMax [ nFile-1] <= WRankMax [ nFile+1]   ) && BRankMin [ nFile-1] == (WRankMax [ nFile-1] + 1 ) )
					|| ( ( WFile [ nFile-1] == 0 || WRankMax [ nFile+1] <= WRankMax [ nFile-1] ) && BRankMin [ nFile+1] == (WRankMax [ nFile+1] + 1 ) )  )
						TEntry.eval -= BACKWARD1_V;
				 TEntry.WeakFileW |= nFile;
				}
		}
	// BLACK
	nLoc = Board.BPiece[x]; 
	if (nLoc != CAPTURED && Board.Sq[nLoc] == BPAWN) 
		{
		int nFile = SQFILE [ nLoc ];
		int nRank = SQRANK [ nLoc ];

		if (nRank <= WRankMin [ nFile+1 ] && nRank <= WRankMin [ nFile-1 ] && nRank < WRankMin [ nFile ] && nRank == BRankMin [ nFile ])
				{
				TEntry.OpenFileB ^= BITMASK [ nFile ]; // encourage rooks behind (Fixme: Rook must be behind)
				TEntry.eval -= (PASSED_RANK [ 9 - nRank ]);
				}

		if (BFile [ nFile-1] == 0 && BFile [ nFile +1] == 0 && nFile!=1 && nFile!= 8  ) // Isolated not on 1st or 8th file
				{ TEntry.eval +=ISOLATED_V;
				   TEntry.WeakFileB |= nFile;
				  //TEntry.Sqs [ nBSq++ ] = nLoc + 10;
				}

		else if (nRank > BRankMax [ nFile+1 ] && nRank > BRankMax [ nFile-1 ] && nRank == BRankMin [ nFile ] )
				{
				//TEntry.Sqs [ nBSq++ ] = nLoc + 10;
				if ( ( (BFile [nFile+1] == 0 || BRankMin [ nFile-1] >= BRankMin [ nFile+1] ) && BRankMin [ nFile-1] == (WRankMax [ nFile-1] + 1 ))
					|| ( (BFile [nFile-1] == 0  || BRankMin [ nFile+1] >= BRankMin [ nFile-1] ) && BRankMin [ nFile+1]  == (WRankMax [ nFile+1] + 1 ) )  )
						TEntry.eval +=BACKWARD1_V;
				   TEntry.WeakFileB |= nFile;
				}
		}
	}

// Store in Hash Table
TEntry.checksum =  Board.PawnHash;
PawnTable [ Board.PawnHash & PAWNHASHTABLESIZE ] = TEntry;
}
