// -------- Defines for board and various other things (don't change these)  --------
#define EMPTY 0
#define WHITE 1
#define BLACK 2
#define BOTH 3
#define NONE 4
#define NO_BEST 65535

#define WPAWN 2
#define WBISHOP 3
#define WKNIGHT 4
#define WROOK 5 
#define WQUEEN 6
#define WKING 7

#define BPAWN 10
#define BBISHOP 11
#define BKNIGHT 12
#define BROOK 13
#define BQUEEN 14
#define BKING 15

#define INVALID 8

#define TIMEOUT 31000

#define CAPTURED 1
#define PD 0

#define WHITEWINS 1
#define BLACKWINS 2
#define DRAW 3

#define KSCASTLE 15
#define QSCASTLE 16

#define ENPASSANT 30997
#define PAWNPUSH 31002
#define PAWNPUSH2 31003


#define PLY 32
const int HALFPLY = PLY / 2;
const int QUARTPLY = HALFPLY / 2;
// -------------------------------------------------

#define MAXDEPTH 50

// Tables 
const char       PieceChars[8]  = { '.', '.', 'p', 'B', 'N', 'R', 'Q', 'K' };    
// The second part of PieceValues must be the same as the first only negative
const signed int PieceValues[16] = {  0  , 0  , 100,   330 ,  315,  504, 960,  4000, 0, 0, -100, -330, -315, -504, -960, -4000  };   
const signed int StopDiag  [16]  = {  0 ,  0  , 1 ,  0,   1,   1,   0,   1,  1,  0,  1,  0,  1,  1,  0,  1 };
const signed int StopHori  [16]  = {  0 ,  0  , 1 ,  1,   1,   0,   0,   1,  1,  0,  1,  1,  1,  0,  0,  1 };
const unsigned int HoriDir[32] = { 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0};
const unsigned int DiagDir[32] = { 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0};

// Structures --------------------------

// Board Sq is 10x12, with the middle 8x8 square being the actual board. the rest padding
// Because of a typo never corrected, the real corner of the board is at element 22 instead of 21
// EnPassant must be in this location in the structure because of the asm code.
// also total should probably be a multiple of 4
struct SBoard
	{
	unsigned char Sq[122];
	unsigned char SideToMove, EnPassant; 
	unsigned int PawnHash, HashCheck, HashKey;
	unsigned short GameTime;
	unsigned short LastReversible;
	signed short TempValue, MatVal;
	unsigned char WCastle, BCastle;
	unsigned char nWP, nBP;
	unsigned char nBPawns, nWPawns, nBMinors, nWMinors;
	unsigned char BPiece [ 16 ], WPiece [ 16 ];
	};


// Converts (x,y) (range: 0 to 7) coordinates to a single index into the board
int inline SQ (int x, int y)
{
	return 22 + x + y * 10;
}

struct Movestruct // This structure is also used in the asm code.
        {
		 int nummoves;
         int numAttackmoves;            
         int eval;
		 int Moves [ 150 ];
         int ischeck;
         int extended;
		 int eChecks;
		 int recap;
        };

int inline MSrc ( const int &nMove)
{
return nMove&255;
}
int inline MDst ( const int &nMove)
{
return (nMove>>8)&255;
}
int inline MPiece ( const int &nMove)
{
return (nMove>>16)&255;
}

// Globals
extern SBoard GameBoard, SearchBoard;
extern unsigned char upgrade;

// Piece Values for being on a square, Black is the mirror of these


const short WKnightValue[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  -22,    0,    0,    0,    0,    0,    0,  -22,  PD,
	 PD,  -10,    5,    8,    8,    8,    8,    5,  -10,  PD,
	 PD,    0,    5,   19,   24,   24,   19,    5,    0,  PD,
	 PD,  -10,    2,   15,   16,   16,   15,    2,  -10,  PD,
	 PD,  -15,    0,    7,    7,    7,    7,    0,  -15,  PD,
	 PD,  -15,    0,    2,    0,    0,    2,    0,  -15,  PD,
	 PD,  -15,    0,    0,    0,    0,    0,    0,  -15,  PD,
	 PD,  -22,  -14,  - 8,  - 8,  - 8,  - 8,  -14,  -22,  PD
};


const short WBishopValue[ 128 ] =
{
	 PD, 
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,    0,  -10,  -5, -5, -5,  -5, -10,   0,  PD,
	 PD,  -10,    5,   0,  0,  0,   0,   5, -10,  PD,
	 PD,    0,    0,   7,  0,  0,   7,   0,   0,  PD,
	 PD,   -5,    0,   5, 10, 10,   5,   0,  -5,  PD,
	 PD,   -5,    0,   5, 10, 10,   5,   0,  -5,  PD,
	 PD,   -5,    0,   5,  0,  0,   5,   0,  -5,  PD,
	 PD,  -10,    5,   0,  0,  0,   0,   5, -10,  PD,
	 PD,    0,  -13, -13, -5, -5, -13, -13,   0,  PD
};


const short WRookValue[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, 15, 15, 15, 15, 15, 15, 15, 15,  PD,
	 PD, 30, 30, 30, 30, 30, 30, 30, 30,  PD,
	 PD,  2,  5,  5,  5,  5,  5,  5,  2,  PD,
	 PD,  2,  5,  5,  5,  5,  5,  5,  2,  PD,
	 PD,  2,  5,  5,  5,  5,  5,  5,  2,  PD,
	 PD,  2,  5,  5,  5,  5,  5,  5,  2,  PD,
	 PD,  8,  9, 10, 10, 10, 10,  9,  8,  PD,
	 PD,  0,  0,  3,  2,  2,  3,  0,  0,  PD
};

const short WQueenValue[ 128 ] =
{
	 PD, 
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  -15, -5,  5,  5,  5,  5,  -5,  -15,  PD,
	 PD,  -5,  -5,  5,  5,  5,  5,  -5,  -5,  PD,
	 PD,  -5,  -5,  5, 15, 15,  5,  -5,  -5,  PD,
	 PD,  -5,  -5,  5, 15, 15,  5,  -5,  -5,  PD,
	 PD,  -2,  -5,  5, 15, 15,  5,  -5,  -2,  PD,
	 PD,  0,   5, 10, 10, 10,  10,   5,  0,  PD,
	 PD,  0,   5, 10, 10, 10,  10,   5,  0,  PD,
	 PD,  0,   5,  8,  8,  8,   8,   5,  0,  PD
};

const short WPawnValue[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  0,   0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  10,  23,  25,  10,  0,  0,  PD,
	PD,  0,  0,  10,  23,  25,  10,  2,  2,  PD,
	PD,  0,  0,  10,  31,  31,  10, -7, -7,  PD,
	PD,  0,  0,  15,  31,  31,  13, -7, -7,  PD,
	PD,  0,  0,   5,  14,  14,   5, 10,  5,  PD,
	PD,  0,  0,   5,  14,  14,  15, 15, 15,  PD,
	PD,  0,  0,   0,   0,   0,   0,  0,  0,  PD
};

const short WPawnValueQS[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  0,   0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  10,  23,  25,  15, 10, 10,  PD,
	PD,  0,  0,  10,  23,  25,  15, 10, 10,  PD,
	PD,  0,  0,  10,  31,  31,  15, 10, 10,  PD,
	PD,  0,  0,  10,  31,  31,  15,  5,  5,  PD,
	PD,  5,  5,   5,  14,  14,   5,  0,  0,  PD,
	PD, 10, 10,   5,  14,  14,   5,  0,  0,  PD,
	PD,  0,  0,   0,   0,   0,   0,  0,  0,  PD
};


const short WPawnValueEnd[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  0,   0,   0,   0,   0,  0,  0,  PD,
	PD,  20, 26, 26,  26,  26,  26, 26, 20,  PD,
	PD,  15, 22, 22,  22,  22,  22, 22, 15,  PD,
	PD,  10, 16, 16,  16,  16,  16, 16, 10,  PD,
	PD,  10, 13, 13,  13,  13,  13, 13, 10,  PD,
	PD,   6,  7,  7,   7,   7,   7,  7,  6,  PD, 
	PD,   4,  5,  5,   5,   5,   5,  5,  4,  PD,
	PD,   0,  0,  0,   0,   0,   0,  0,  0,  PD,
};

extern short BPawnValue[ 128 ];
extern short BPawnValueQS[ 128 ];
extern short BPawnValueEnd[ 128 ];

// opening
const short WKingValue[ 128 ] =
{
	 PD, 
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  -45,-45,-45,-45,-45 ,-45,-45,-45,   PD,
	 PD,  -45,-45,-45,-45,-45 ,-45,-45,-45,   PD,
	 PD,  -45,-45,-45,-45,-45 ,-45,-45,-45,   PD,
	 PD,  -45,-45,-45,-45,-45 ,-45,-45,-45,   PD,
	 PD,  -40,-45,-45,-45,-45 ,-45,-45,-40,   PD,
	 PD,  -25,-25,-25,-25,-25 ,-25,-25,-25,  PD,
	 PD,  0, 0, -10,-12,-12, -10, 0,  0,  PD,
	 PD,  5, 10,  0, -12,-12,  0, 10,  5,  PD
};

const short WKingValueKRK[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,-45,-38,-30,-28,-28,-30,-38,-45,  PD,
	 PD,-38,-28,-20,-18,-18,-20,-28,-38,  PD,
	 PD,-30,-22,-15,-10,-10,-15,-22,-30,  PD,
	 PD,-28,-18,-10, 00, 00,-10,-18,-28,  PD,
	 PD,-28,-18,-10, 00, 00,-10,-18,-28,  PD,
	 PD,-30,-22,-15,-10,-10,-15,-22,-30,  PD,
	 PD,-38,-28,-20,-18,-18,-20,-28,-38,  PD,
	 PD,-45,-38,-30,-28,-28,-30,-38,-45,  PD
};

const short WKingValueKBNK[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, 72, 62, 52,  42,  32,  22, 12,  5,  PD,
	PD, 62, 50, 40,  28,  20,  10,  5, 12,  PD,
	PD, 52, 40, 26,  20,  10,   0, 10, 22,  PD,
	PD, 42, 28, 20,   0,  -5,  10, 18, 32,  PD,
	PD, 32, 20, 10,  -5,   0,  16, 28, 42,  PD,
	PD, 22,  8,  0,  10,  16,  26, 40, 52,  PD, 
	PD, 12,  5, 10,  18,  28,  40, 50, 62,  PD,
	PD,  5, 12, 22,  32,  42,  52, 62, 72,  PD,
};


const short WKingValueEnd[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  2,    2,   2,   2,   2,  2,  0,  PD,
	PD,  2,  13,   15,  15,  15,  15,  13,  0,  PD,
	PD,  2,  15,   33,  33,  33,  33,  15,  2,  PD,
	PD,  2,  15,   33,  43,  43,  33,  15,  2,  PD,
	PD,  2,  15,   33,  43,  43,  33,  15,  2,  PD,
	PD,  2,  15,   33,  33,  33,  33,  15,  2,  PD,
	PD,  2,  13,   15,  15,  15,  15,  13,  2,  PD,
	PD,  0,  2,    2,   2,   2,   2,  2,  0,  PD,
};


const int SQFILE[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD,
	 PD,  1,  2,  3,  4,  5,  6,  7,  8,  PD
};


const int SQRANK[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  8,  8,  8,  8,  8,  8,  8,  8,  PD,
	 PD,  7,  7,  7,  7,  7,  7,  7,  7,  PD,
	 PD,  6,  6,  6,  6,  6,  6,  6,  6,  PD,
	 PD,  5,  5,  5,  5,  5,  5,  5,  5,  PD,
	 PD,  4,  4,  4,  4,  4,  4,  4,  4,  PD,
	 PD,  3,  3,  3,  3,  3,  3,  3,  3,  PD,
	 PD,  2,  2,  2,  2,  2,  2,  2,  2,  PD,
	 PD,  1,  1,  1,  1,  1,  1,  1,  1,  PD
};

const int SQCOLOR[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  2,  1,  2,  1,  2,  1,  2,  1,  PD,
	 PD,  1,  2,  1,  2,  1,  2,  1,  2,  PD,
	 PD,  2,  1,  2,  1,  2,  1,  2,  1,  PD,
	 PD,  1,  2,  1,  2,  1,  2,  1,  2,  PD,
	 PD,  2,  1,  2,  1,  2,  1,  2,  1,  PD,
	 PD,  1,  2,  1,  2,  1,  2,  1,  2,  PD,
	 PD,  2,  1,  2,  1,  2,  1,  2,  1,  PD,
	 PD,  1,  2,  1,  2,  1,  2,  1,  2,  PD
};

const int SQDIAG1[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD,  8,  7,  6,  5,  4,  3,  2,  1,  PD,
	 PD,  9,  8,  7,  6,  5,  4,  3,  2,  PD,
	 PD, 10,  9,  8,  7,  6,  5,  4,  3,  PD,
	 PD, 11, 10,  9,  8,  7,  6,  5,  4,  PD,
	 PD, 12, 11, 10,  9,  8,  7,  6,  5,  PD,
	 PD, 13, 12, 11, 10,  9,  8,  7,  6,  PD,
	 PD, 14, 13, 12, 11, 10,  9,  8,  7,  PD,
	 PD, 15, 14, 13, 12, 11, 10,  9,  8,  PD
};

const int SQDIAG2[ 128 ] =
{
	 PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	 PD, 15, 14, 13, 12, 11, 10,  9,  8,  PD,
	 PD, 14, 13, 12, 11, 10,  9,  8,  7,  PD,
	 PD, 13, 12, 11, 10,  9,  8,  7,  6,  PD,
	 PD, 12, 11, 10,  9,  8,  7,  6,  5,  PD,
	 PD, 11, 10,  9,  8,  7,  6,  5,  4,  PD,
	 PD, 10,  9,  8,  7,  6,  5,  4,  3,  PD,
	 PD,  9,  8,  7,  6,  5,  4,  3,  2,  PD,
	 PD,  8,  7,  6,  5,  4,  3,  2,  1,  PD
};

const short SQ64[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  1,  2,   3,   4,   5,  6,  7,  PD,
	PD,  8,  9, 10,  11,  12,  13, 14, 15,  PD,
	PD, 16, 17, 18,  19,  20,  21, 22, 23,  PD,
	PD, 24, 25, 26,  27,  28,  29, 30, 31,  PD,
	PD, 32, 33, 34,  35,  36,  37, 38, 39,  PD,
	PD, 40, 41, 42,  43,  44,  45, 46, 47,  PD, 
	PD, 48, 49, 50,  51,  52,  53, 54, 55,  PD,
	PD, 56, 57, 58,  59,  60,  61, 62, 63,  PD,
};

const short PSQ[ 64 ] =
{
	0,  0,  0,   0,   0,   0,  0,  0,
	0,  1,  2,   3,   3,   2,  1,  0,
	4,  5,  6,   7,   7,   6,  5,  4,
	8,  9,  10,  11, 11,  10,  9,  8,
	12, 13, 14,  15, 15,  14, 13, 12,
	16, 17, 18,  19, 19,  18, 17, 16,
	20, 21, 22,  23, 23,  22, 21, 20,
	0,  0,  0,   0,   0,   0,  0,  0, 
};

const short FlipH64[ 64 ] =
{
	7,  6,  5,   4,   3,   2,  1,  0,
	15, 14, 13,  12, 11,  10,  9,  8,
	23, 22, 21,  20, 19,  18, 17, 16,
	31, 30, 29,  28, 27,  26, 25, 24,
	39, 38, 37,  36, 35,  34, 33, 32,
	47, 46, 45,  44, 43,  42, 41, 40,
	55, 54, 53,  52, 51,  50, 49, 48,
	63, 62, 61,  60, 59,  58, 57, 56,
};

const short ZeroValue[ 128 ] =
{
	PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD, PD, PD, PD, PD, PD, PD, PD, PD,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD, 
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
	PD,  0,  0,  0,   0,   0,   0,  0,  0,  PD,
};


const unsigned char BITMASK [9] = {0, 1, 2, 4, 8, 16, 32, 64, 128 };
// Data
extern int SquareVals [128 * 16];
extern char acDirec [128 * 128];
extern unsigned int KnightAttacks [ 256 ];

class COpeningBook;
extern COpeningBook *pNewBook;