// Slow Chess 
// by Jonathan Kreuzer
// copyright 2003

//
// This source may be used to satisfy your curiosity, as a learning tool, or it
// may be modified and redistributed as long as you give credit.
// (And I'll add this now to cover the unlikely event this program gets much better...
//  you can't charge money for something based on this code. i.e. You're not allowed
//  to use this code, make some changes or cut&paste it into a project, and sell it.)
//
// That said, this isn't the strongest chess program, nor was the source designed
// to be easy to read of modify, Nowadays though, if/when I edit it I try to keep those
// things in mind, so it's getting better.


// Includes ----------------------
#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include <time.h>
#include <process.h>

#include "data.h"
#include "gui.h"
#include "search.h"
#include "moves.h"
#include "transcript.h"
#include "resource.h"
#include "scNet.h"
#include "evaluate.h"

void wbMainLoop ( );
void  FAR ThinkingThread( void FAR *parm);

COpeningBook *pNewBook = NULL;

// Globals -------------------------
HBITMAP PieceBitmaps[32];

int hThread; 
HWND sideHWND = NULL, mainhwnd = NULL, ChatWnd = NULL, NetStartWnd = NULL;
int Minutes = 5, Seconds = 5, MaxAhead = 28, ELO = 0;
unsigned int g_nMaxNodes = 0;
int bShowLastMove;
int g_bAlwaysWait = FALSE;

int g_bICC = FALSE;
int g_bWinBoard = FALSE;
int bExperiment = FALSE;
int bExtendChecks = TRUE;
int bUseNull = TRUE;
int bPonder = FALSE;
int bDisplayInfo = TRUE;
int bHashing = TRUE;
int bEvalWhite = TRUE;

char g_sNetWhite [66], g_sNetBlack[66];
char sUserName [66];
char sOppName [66];
char sConnectIP[66];

int g_bNetGame = FALSE, g_bDebug = FALSE;
char g_cNetColor = NONE;
int g_GameResult;
clock_t g_startTime;
int g_bSetup = FALSE;
int g_ponder = 0;
int g_ponderSrc, g_ponderDst;
SBoard g_PonderBoard;

SBoard g_StartBoard;
int g_nWidth = 774, g_nHeight = 586, g_selSquare = 66, g_nSqSize = 64, g_xAdd = 10, g_yAdd = 10;
int flipboard = 0;
char *g_sSlow = "SlowChess 2.82a ";

//
// Defaults Class
//
struct SDefaults
	{
	SDefaults ()
		{
		 strcpy (sName, "Human");
		 strcpy (sIP, "127.0.0.1");
		 nMinutes = 5;
		 nSeconds = 5;
		 nMaxPly = 28;
		 MaxAhead = 28;
		 nELO = 0;
		 bPonder = FALSE;
		 bDisplayInfo = TRUE;
		 nHashBits = 21;
		 bHashing = TRUE;
		 bExtendChecks = TRUE;
		 bShowLastMove = TRUE;
		 bEvalWhite = TRUE;
		}

	void GetCurDir ()
	{
	GetCurrentDirectory (4195, sCurDir );
	}

	void SetCurDir ()
	{
	 SetCurrentDirectory ( sCurDir );
	}

	void Save ( char *sFile )
		{
		SetCurDir ();
		FILE *FP = fopen (sFile, "wt");
		if (FP!=NULL)
			{
			fprintf (FP, "%s\n%s\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n", sName, sIP, nMinutes, nSeconds, nMaxPly, nELO, bPonder, bDisplayInfo, nHashBits, bExtendChecks);
			fprintf (FP, "%d\n%d\n", bShowLastMove, bEvalWhite);
			fprintf (FP, "%d\n", g_bDebug);
			fclose (FP);
			}
		}

	void Load ( char *sFile )
		{
		FILE *FP = fopen (sFile, "rt");
		if (FP!=NULL)
			{
			fgets (sName, 39, FP); fgets (sIP, 39, FP);
			sName [strlen (sName) -1] = NULL;
			sIP	  [strlen (sIP) -1]	  = NULL;
			fscanf (FP, "%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n", &nMinutes, &nSeconds, &nMaxPly, &nELO, &bPonder, &bDisplayInfo, &nHashBits, &bExtendChecks);
			fscanf (FP, "%d\n%d\n", &bShowLastMove, &bEvalWhite);
			fscanf (FP, "%d\n", &g_bDebug);
			fclose (FP);
			}
		}

	void SetFromDefaults ( )
		{
		strcpy (sUserName, sName);
		strcpy (sConnectIP, sIP);
		Minutes = nMinutes;
		Seconds = nSeconds;
		//MaxAhead  = nMaxPly;
		ELO		= nELO;
		}

	void SetTime ( )
		{
		nMinutes = Minutes;
		nSeconds = Seconds;
		}

	char sCurDir[4196];
	char sName[66];
	char sIP[66];
	int nHashBits;
	int nMinutes;
	int nSeconds;
	int nMaxPly;
	int nMaxNodes;
	int nELO;
	};

SDefaults Defaults;

char OppReady = 0, Ready = 0;

// Globals --------------------------------------------
SBoard GameBoard;

int g_nBlackTime = 0, g_nWhiteTime = 0;
int nSecsPerMove = 0;

static unsigned char  sPGNFile[] = "PGN Chess Game (*.pgn)" \
                                        "\0" \
                                        "*.pgn" \
                                        "\0\0";

static unsigned char  sBookFile[] = "Opening Book (*.scb)" \
                                        "\0" \
                                        "*.scb" \
                                        "\0\0";
// ====================================================
// TIMER Proc
// ====================================================
void CALLBACK TimerProc( HWND hwnd, unsigned msg, UINT wparam, LONG lparam )
    {
	// Don't care what's passed in
    hwnd = hwnd; wparam = wparam; lparam = lparam; msg = msg;

	// Game is over, don't change timer
	if (g_GameResult != NONE) return;

	int GameTime = NumTranMoves ( );

	int TempBlack = g_nBlackTime;
	int TempWhite = g_nWhiteTime;

	clock_t endTime = clock ();

	// Increment Time of Side to Move
	if ((GameTime % 2) == 1 ) TempBlack += (100*(endTime - g_startTime))/CLOCKS_PER_SEC;
						 else TempWhite += (100*(endTime - g_startTime))/CLOCKS_PER_SEC;

	DisplayTimer (  TempWhite/100, TempBlack/100);
    }

// Increment and Display the Timer (Time left, Time Total)
void DisplayTimer ( int WhiteTime, int BlackTime )
{
    char buffer[15];
    int Blackleft, Whiteleft;
	static int BlackOld = -1, WhiteOld = -1, OldGameTime = -1;
   	int GameTime = NumTranMoves ( );

	if (g_bNetGame == TRUE)
	{
	Whiteleft = g_NetMinutes * 60 - WhiteTime + ( (GameTime + 1) /2) * g_NetSeconds;
	Blackleft = g_NetMinutes * 60 - BlackTime + ( (GameTime   ) /2 ) * g_NetSeconds;
	}
	else
	{
	Whiteleft = Minutes * 60 - WhiteTime + ( (GameTime + 1) /2) * Seconds;
	Blackleft = Minutes * 60 - BlackTime + ( (GameTime   ) /2) * Seconds;
	}
    if (Whiteleft < 0) Whiteleft = 0;
    if (Blackleft < 0) Blackleft = 0;

	// (Display For White)
	if (BlackOld != BlackTime || WhiteOld != WhiteTime || OldGameTime != GameTime)
	{
	sprintf (buffer, "%d:%.2d", (Whiteleft/60), (Whiteleft%60) );
    SetDlgItemText (sideHWND, 112, buffer );
	sprintf (buffer, "%d:%.2d", (WhiteTime/60), (WhiteTime%60) );
    SetDlgItemText (sideHWND, 123, buffer );

	sprintf (buffer, "%d:%.2d", (BlackTime/60), (BlackTime%60) );
    SetDlgItemText (sideHWND, 124, buffer );
	sprintf (buffer, "%d:%.2d", (Blackleft/60), (Blackleft%60) );
    SetDlgItemText (sideHWND, 113, buffer );
	}

	BlackOld = BlackTime;
	WhiteOld = WhiteTime;
	OldGameTime = GameTime;

	// Net Games, is a side out of time?
	if (g_bNetGame == TRUE && g_GameResult == NONE)
		{
		if (Blackleft <= 0 && g_cNetColor == BLACK)
			{
			 g_GameResult = WHITEWINS;
			 scNetMessage (SC_TIMEUP, BLACK);
			}
		if (Whiteleft <= 0 && g_cNetColor == WHITE)
			{
			g_GameResult = BLACKWINS;
			scNetMessage (SC_TIMEUP, WHITE);
			}
		}
}
// ---------------------
//
// ---------------------
void ResizeFunc ( )
{
	int nExtra;
	if (ChatWnd!=NULL) nExtra = 125 + g_yAdd*2; else nExtra = g_yAdd*2 + 8;
	if (sideHWND!=NULL) MoveWindow ( sideHWND, g_nWidth-225, 0, 225, g_nHeight, TRUE);
	if (ChatWnd!=NULL)  MoveWindow ( ChatWnd, 0, g_nHeight-125, g_nWidth-(225), 125 , TRUE );
	int	nMin = (g_nWidth - (228+g_xAdd*2)) < (g_nHeight-nExtra) ? (g_nWidth - (228+g_xAdd*2)) : (g_nHeight-nExtra);
	g_nSqSize = (nMin) / 8;
}

//
//
void SizeWindowToDefault ( )
	{
	int nExtra;
	if (ChatWnd!=NULL) nExtra = g_yAdd*2 + 125; else nExtra = g_yAdd*2 + 8;
	ShowWindow (mainhwnd, SW_RESTORE);
	SetWindowPos (mainhwnd, HWND_TOP, 10, 1, 64*8 + (227+g_xAdd*2) + 15, 64*8 + nExtra + 46, SWP_NOMOVE); 
	}

// --------------------
// GetFileName - get a file name using common dialog
// --------------------
static BOOL GetFileName(  HWND hwnd, BOOL save, char *fname, unsigned char *filterList)
{
    OPENFILENAME        of;
    int                 rc;

    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = hwnd;
    of.lpstrFilter = (LPSTR) filterList;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = fname;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;
    if( save ) {
               rc = GetSaveFileName( &of );
               }
    else {
          rc = GetOpenFileName( &of );
          }
    return( rc );
}

// =============================================
// CONNECT Dialog
// =============================================

BOOL FAR PASCAL ConnectDlgProc( HWND hwnd, unsigned msg,
                                UINT wparam, LONG lparam )
{
    lparam = lparam; // turn off warning 
    int temp;
	
	switch( msg ) {
    case WM_INITDIALOG:
        SetDlgItemText ( hwnd, 100, sConnectIP );
        return( TRUE );

    case WM_COMMAND:
 
        if( LOWORD( wparam ) == IDOK )  
			{
			InitializeSockets ( );
			temp = OpenSocket ( sConnectIP, 14324 );

			if ( (temp) == 1 ) 
				{
				 g_cNetColor = NONE;
				 scNetMessageS ( SC_NAME, sUserName);
				 scNetMessage  ( SC_TABLE, 0);
				 NetGameOn ( );
				 NewGame ( TRUE );
                }
			if (temp == 3) DisplayText ( "Invalid IP" );
			if (temp == 2) DisplayText ( "No Server Found" );
			}

        if( LOWORD( wparam ) == 100 ) GetDlgItemText (hwnd, 100, sConnectIP, 50 );

        if ( LOWORD( wparam ) == 105) 
			{
			strcpy (Defaults.sIP, sConnectIP);
   			}
        
        if( LOWORD( wparam ) == IDCANCEL ||  LOWORD( wparam ) == IDOK) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
			}
          
    break;
	}
return (FALSE);
}

// =============================================
// Chat Dialog
// =============================================
BOOL FAR PASCAL ChatProc( HWND hwnd, unsigned msg,
                                UINT wparam, LONG lparam )
{
 lparam = lparam; // turn off warning 
 static char buffer[300], temp [300];
 char *string;
 int length;

 switch( msg ) {
    case WM_INITDIALOG:

      return( TRUE );

    case WM_COMMAND:
                                        
    if( LOWORD( wparam ) == 105 ) 
		{
		GetDlgItemText( hwnd, 103, buffer, 240);
        scNetMessageS (SC_CHAT, buffer);
        SetDlgItemText( hwnd, 103, "");
        }

    if( LOWORD( wparam ) == 103 ) 
		{
		length = GetDlgItemText ( ChatWnd, 103, buffer, 240);

        if ( (string = strstr(buffer, "\15\12")) != NULL) // Return hit? then send
               {
               string [0] = NULL; // Don't send the return;
               scNetMessageS (SC_CHAT, buffer);
               SetDlgItemText( ChatWnd, 103, "");
               }
        }

     break;
     }
         
return (FALSE);
}

/* =====================================================================
 * Load defaults, init boards & hash & opening book, then Start the Thinking Thread
   ======================================================================*/
HANDLE hAction, hEngineReady;

void InitEngine ( )
{
	// Initialize Search Boards & Square Values
	InitKingPawnProbe ( );
	InitSquareArray();
    memset (&BoardStack[0], INVALID, MAXDEPTH * sizeof(SBoard)); // invalid squares
    memset (&SearchBoard, INVALID, sizeof(SBoard));

	// Load Defaults
	Defaults.Load ("Schess.dft");
	Defaults.SetFromDefaults ( );
	Defaults.GetCurDir ();

	// Hashing
    Create_HashFunction ( );
	SetHashSizeBits ( Defaults.nHashBits ); 

	// Opening Book (uses hashing)
	pNewBook = new COpeningBook;
	pNewBook->Load ( "TestBook.scb" );

	// Start the thinking Thread
	_beginthread (ThinkingThread, NULL, NULL);
	hAction = CreateEvent(NULL, FALSE, FALSE, NULL);
	hEngineReady = CreateEvent (NULL, TRUE, FALSE, NULL);
}

/* ------------------------------------
 * WinMain - initialization, message loop
 * ------------------------------------- */
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
                    int cmdshow )
{
    MSG         msg;

    if( !prev_inst ) {
        if( !FirstInstance( this_inst ) ) return( FALSE );
    }
    if( !AnyInstance( this_inst, cmdshow, cmdline ) ) return( FALSE );

    while( GetMessage( &msg, NULL, NULL, NULL ) ) {

        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    return( msg.wParam );
} 

/* ------------------------------------
 * Console Mode, comment out WinMain and put this in to make this a console .exe for winboard.
 * not sure if this works anymore, haven't done it in a while
 * ------------------------------------- */
/*int main( )
{
	InitEngine ( );
	g_bWinBoard = TRUE;
	wbMainLoop ( );
	return 0;
} */


/*============================================================
 * FirstInstance - register window class for the application,
 *                 
 *============================================================*/
static BOOL FirstInstance( HINSTANCE this_inst )
{
    WNDCLASS    wc;
    BOOL        rc;

    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc =  WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof( DWORD );
    wc.hInstance = this_inst;
    wc.hIcon = LoadIcon( this_inst, "ChessIcon" );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
    wc.lpszMenuName = "ChessMenu";
    wc.lpszClassName = "ChessClass";
    rc = RegisterClass( &wc );
    return( rc );
} 


/* =====================================================================
 * AnyInstance - Do work required for every instance of the application:
 *               create the window, initialize data
 ======================================================================*/
static BOOL AnyInstance( HINSTANCE this_inst, int cmdshow, LPSTR cmdline )
{
    HWND  hwnd;
	cmdline = cmdline;
         
	char	sBuf[1024];
	if (fgets(sBuf, sizeof(sBuf), stdin) == sBuf) g_bWinBoard = TRUE; else g_bWinBoard = FALSE;

	InitEngine ( );

	if (g_bWinBoard == TRUE) 
		{
		wbMainLoop ( );
		exit (0);
		}

   	if (g_bWinBoard == FALSE)
		{
		// create main window
		hwnd = CreateWindow(  "ChessClass",  "Slow Chess 2.82",      // caption 
			WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,  // style 
			10, 1, g_nWidth, g_nHeight,    // x,y Position  x,y Size
		    NULL,                   // parent window 
			NULL,                   // menu handle 
			this_inst,              // program handle 
			NULL                    // create parms 
        );
		if( !hwnd ) return( FALSE );

		mainhwnd = hwnd; // set global pointer to main window

		// Create Side Window
		if (sideHWND == NULL)
			 sideHWND = CreateDialog( this_inst,"CLIENT", hwnd, (DLGPROC)GameDlgProc);

		// Load Chess Bitmaps
		static char* psBitmapNames[32] = {"square1", NULL, "wpawn1", "wbishop1", "wknight1", "wrook1", "wqueen1", "wking1",
									  "square1", NULL, "bpawn1", "bbishop1", "bknight1", "brook1", "bqueen1", "bking1",
									  "square2", NULL, "wpawn2", "wbishop2", "wknight2", "wrook2", "wqueen2", "wking2",
									  "square2", NULL, "bpawn2", "bbishop2", "bknight2", "brook2", "bqueen2", "bking2"};
		for (int i = 0; i < 32; i++)
			{
			if (psBitmapNames[i] != NULL) PieceBitmaps [ i ] = (HBITMAP)LoadImage  ( this_inst, psBitmapNames[i], IMAGE_BITMAP, 0, 0, 0 );
			}

		// Timer
		SetTimer( NULL, 1, 200 ,  (TIMERPROC) TimerProc );

		ShowWindow( hwnd, cmdshow );
		// Menu CheckMarks
		if (bEvalWhite == TRUE)  CheckMenuItem (GetMenu(hwnd), ID_OPTIONS_EVALWHITE, MF_CHECKED);
		if (bDisplayInfo == TRUE) CheckMenuItem (GetMenu(hwnd), ID_SEARCHINFO, MF_CHECKED);
		if (bPonder == TRUE)	  CheckMenuItem (GetMenu(hwnd), ID_PONDER, MF_CHECKED);
		if (bExtendChecks == TRUE) CheckMenuItem (GetMenu(hwnd), ID_EXTENDCHECK, MF_CHECKED);
		if (bShowLastMove == TRUE) CheckMenuItem (GetMenu(hwnd), 71, MF_CHECKED);
	//	if (g_bDebug == TRUE)  CheckMenuItem (GetMenu(hwnd), ID_WINDEBUG, MF_CHECKED);
		CheckMenuItem (GetMenu(hwnd), 54, MF_CHECKED);
		CheckMenuItem (GetMenu(hwnd), 160, MF_CHECKED);
		if (bHashing == TRUE) CheckMenuItem (GetMenu(hwnd), 74, MF_CHECKED); 
		CheckMenuItem (GetMenu(hwnd), 1234 - 19 + Defaults.nHashBits, MF_CHECKED);

		NetGameOff ( FALSE );
		}

	// Board & Setup
	NewGame (  TRUE  );
    return( TRUE );
} 

// ==============================================================
//   Pawn Promotion Dialog Procedure
// ==============================================================
BOOL CALLBACK PawnDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{
 lparam = lparam;                    /* turn off warning */
  
 switch( msg ) {
    case WM_INITDIALOG:

        return( TRUE );

    case WM_COMMAND:
 
        if( LOWORD( wparam ) == 100 )  
                          { upgrade = WQUEEN;
                            EndDialog( hwnd, TRUE );
                            return( TRUE );
                          }
        if( LOWORD( wparam ) == 101 )  
                          { upgrade = WROOK;
                            EndDialog( hwnd, TRUE );
                            return( TRUE );
                          }
        if( LOWORD( wparam ) == 102 )  
                          { upgrade = WBISHOP;
                            EndDialog( hwnd, TRUE );
                            return( TRUE );
                          }
        if( LOWORD( wparam ) == 103 )  
                          { upgrade = WKNIGHT;
                            EndDialog( hwnd, TRUE );
                            return( TRUE );
                          }                                                    
           }
        
 return (FALSE);
}

// ==============================================================
//   Join Table Dialog Procedure
// ==============================================================
BOOL CALLBACK TableDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{
  lparam = lparam;      
  
 switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND: //fixme? Setting (SetDlgItemText) the table name should be from a server message for future expansion.
 
		if ( LOWORD( wparam ) >= 100 && LOWORD( wparam ) <= 103 ) scNetMessageS ( SC_GOSPEC, "");
        if( LOWORD( wparam ) == 100 )  {SetDlgItemText (sideHWND, 118, "Table 0");  scNetMessage ( SC_TABLE, 0); }
        if( LOWORD( wparam ) == 101 )  {SetDlgItemText (sideHWND, 118, "Table 1");  scNetMessage ( SC_TABLE, 1); }
        if( LOWORD( wparam ) == 102 )  {SetDlgItemText (sideHWND, 118, "Table 2");  scNetMessage ( SC_TABLE, 2); }
        if( LOWORD( wparam ) == 103 )  {SetDlgItemText (sideHWND, 118, "Table 3");  scNetMessage ( SC_TABLE, 3); }

        if( LOWORD( wparam ) >= 100 && LOWORD( wparam ) <= 104 ) 
			{
			EndDialog( hwnd, TRUE );
            return( TRUE );
			}
       }
 return (FALSE);
}


// ==============================================================
//   Time Control Dialog  Procedure
// ==============================================================
BOOL CALLBACK TimeDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{

 lparam = lparam; // turn off warning 

 switch( msg ) {
    case WM_INITDIALOG:
        SetDlgItemInt ( hwnd, 100, Minutes, FALSE );
        SetDlgItemInt ( hwnd, 101, Seconds, FALSE );
		SetDlgItemInt ( hwnd, 104, (int)fPanicTime, FALSE );
        return( TRUE );

    case WM_COMMAND:
 
        if( LOWORD( wparam ) == 1 )  
              {
				Minutes = GetDlgItemInt (hwnd, 100, NULL, FALSE );
				Seconds = GetDlgItemInt (hwnd, 101, NULL, FALSE );
              }
      
        if( LOWORD( wparam ) == 2 ||  LOWORD( wparam ) == 1) {
			Defaults.SetTime ();
            EndDialog( hwnd, TRUE );
            return( TRUE );
          }
          
        break;
 }
return (FALSE);
}

// ==============================================================
//   Time Control Dialog  Procedure
// ==============================================================
BOOL CALLBACK LevelDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{

 lparam = lparam;                    // turn off warning
 
 switch( msg ) {
    case WM_INITDIALOG:
        SetDlgItemInt ( hwnd, ID_MAXPLY, MaxAhead, FALSE );
        SetDlgItemInt ( hwnd, ID_MAXTIME, nSecsPerMove, FALSE );
		SetDlgItemInt ( hwnd, ID_MAXNODES, (g_nMaxNodes/1000), FALSE );
		CheckDlgButton ( hwnd, ID_WAITMOVE, g_bAlwaysWait);
        return( TRUE );

    case WM_COMMAND:
 
        if( LOWORD( wparam ) == 1 )  
               {
				MaxAhead = GetDlgItemInt (hwnd, ID_MAXPLY, NULL, FALSE );
				nSecsPerMove = GetDlgItemInt (hwnd, ID_MAXTIME, NULL, FALSE );
				if (nSecsPerMove < 0) 
					{
					MessageBox (hwnd, "Seconds set to 0", "Out of Range", MB_OK);
					nSecsPerMove = 0;
					}
				if (MaxAhead < 1) 
					{
					MessageBox (hwnd, "Max ply set to 1", "Out of Range", MB_OK);
					MaxAhead = 1;
					}
				g_nMaxNodes = GetDlgItemInt (hwnd, ID_MAXNODES, NULL, FALSE ) * 1000;
				g_bAlwaysWait = IsDlgButtonChecked (hwnd, ID_WAITMOVE);
               }
          
        if( LOWORD( wparam ) == 2 ||  LOWORD( wparam ) == 1) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
          }
          
        break;
 }
return (FALSE);
}

// ==============================================================
//   Player Info
// ==============================================================
BOOL CALLBACK PlayerDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{

 lparam = lparam;                    /* turn off warning */

 switch( msg ) {
    case WM_INITDIALOG:
 		SetDlgItemInt (hwnd, 101, Defaults.nELO, FALSE );
		SetDlgItemText ( hwnd, 100, sUserName);
		return( TRUE );

    case WM_COMMAND:
 
        if( LOWORD( wparam ) == 1 )  
               {
				Defaults.nELO = GetDlgItemInt (hwnd, 101, NULL, FALSE );
				GetDlgItemText ( hwnd, 100, Defaults.sName , 39 );
				strcpy (sUserName, Defaults.sName);
				ELO = Defaults.nELO;
               }

        
        if( LOWORD( wparam ) == 2 ||  LOWORD( wparam ) == 1) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
          }
          
        break;
 }
return (FALSE);
}

// =============================================
// Clear all entries from book
// =============================================
void ClearBook ()
	{
	if (MessageBox (mainhwnd, "If you made any new book entries, they will be lost if not saved. Really Clear Book?", "Clear Book?", MB_YESNO )
			== IDYES)
		{
		delete pNewBook;
		pNewBook = new COpeningBook;
		DisplayText ("Book Cleared");
		}
	}

void SaveBook ( HWND hwnd )
	{
	char buffer[4196];
	strcpy (buffer, "*.scb");
	if (GetFileName (hwnd, 1, buffer, sBookFile) == TRUE) pNewBook->Save ( buffer, 0 );
	}

// =============================================
// BOOK OPTIONS DIALOG PROC
// =============================================
BOOL FAR PASCAL BookDlgProc( HWND hwnd, unsigned msg, UINT wparam, LONG lparam )
{
 lparam = lparam; // turn off warning 
 static char buffer[4196];
 static unsigned int nMoves = 10, nTimes = 1, nGames = 50000, nStart = 1;
 static int bWin = 1, bDraw=1, bLoss = 0;

 switch( msg ) {
    case WM_INITDIALOG:
		sprintf (buffer, "# of Positions in Book: %d", pNewBook->m_nPositions);
		SetDlgItemText (hwnd, IDC_NPOSTITIONS, buffer);
		SetDlgItemInt (hwnd, IDC_EDITMOVES , nMoves, FALSE);
		SetDlgItemInt (hwnd, IDC_EDITSTART , nStart, FALSE);
		SetDlgItemInt (hwnd, IDC_MAXGAMES , nGames, FALSE);
		SetDlgItemInt (hwnd, IDC_EDITTIMES , nTimes, FALSE);
		CheckDlgButton  (hwnd, IDC_CHECKWIN, bWin);
		CheckDlgButton  (hwnd, IDC_CHECKDRAW, bDraw);
		CheckDlgButton  (hwnd, IDC_CHECKLOSS, bLoss);
      return( TRUE );

    case WM_COMMAND:

	switch ( LOWORD( wparam ) )
		{
		case IDC_CHECKWIN  : bWin  = IsDlgButtonChecked (hwnd, IDC_CHECKWIN);  break;
		case IDC_CHECKDRAW : bDraw = IsDlgButtonChecked (hwnd, IDC_CHECKDRAW); break;
		case IDC_CHECKLOSS : bLoss = IsDlgButtonChecked (hwnd, IDC_CHECKLOSS); break;

		case  IDC_MAXGAMES : 
			nGames = GetDlgItemInt (hwnd, IDC_MAXGAMES, NULL, FALSE);
			break;

		case  IDC_EDITMOVES:
			nMoves = GetDlgItemInt (hwnd, IDC_EDITMOVES, NULL, FALSE);
			if (nMoves > 30) nMoves = 30;
			if (nMoves < 1) nMoves = 1;
			break;

		case  IDC_EDITSTART:
			nStart = GetDlgItemInt (hwnd, IDC_EDITSTART, NULL, FALSE);
			if (nStart > 30) nStart = 30;
			if (nStart < 1) nStart = 1;
			break;

		case IDC_EDITTIMES:
			nTimes = GetDlgItemInt (hwnd, IDC_EDITTIMES, NULL, FALSE);
			if (nTimes < 1) nTimes = 1;
			break;

		case IDC_SAVEBOOK:
			SaveBook( hwnd );
			break;

		case IDC_PGNADD:
			strcpy (buffer, "*.pgn");
			if (GetFileName (hwnd, 0, buffer, sPGNFile) == TRUE)
				PGNtoBook (buffer,  nGames, nStart, nMoves, nTimes, bWin | (bDraw<<1) | (bLoss<<2) );

			sprintf (buffer, "# of Positions in Book: %d", pNewBook->m_nPositions);
			SetDlgItemText (hwnd, IDC_NPOSTITIONS, buffer);
			break;

		case IDC_LOADBOOK:
			strcpy (buffer, "*.scb");
			if (GetFileName (hwnd, 0, buffer, sBookFile) == TRUE) pNewBook->Load ( buffer );

			sprintf (buffer, "# of Positions in Book: %d", pNewBook->m_nPositions);
			SetDlgItemText (hwnd, IDC_NPOSTITIONS, buffer);
			break;

		case IDC_CLEAR:
			ClearBook ();
			sprintf (buffer, "# of Positions in Book: %d", pNewBook->m_nPositions);
			SetDlgItemText (hwnd, IDC_NPOSTITIONS, buffer);
			break;

		case IDC_DONE:
		case IDCANCEL:
			EndDialog( hwnd, TRUE );
			 return( TRUE );
		}

    break;
    }
         
return (FALSE);
}

/* =============================================
 *
 * NETWORK Game Start dialog - processes messages for the starting games over network.
 *
   ============================================== */
BOOL FAR PASCAL NetStartDlgProc( HWND hwnd, unsigned msg,
                                UINT wparam, LONG lparam )
{
  int nMin, nSec;
         
    switch( msg ) {
    case WM_INITDIALOG:
       if (g_cNetColor == BLACK) CheckRadioButton (hwnd, 104, 105, 105 );
        else CheckRadioButton (hwnd, 104, 105, 104 );
   //     if (g_cNetColor == BLACK) SendSetupInfo ('t', Minutes-128, Seconds-128 );
    	SetDlgItemInt( hwnd, 135,  g_NetMinutes, FALSE);
        SetDlgItemInt( hwnd, 137,  g_NetSeconds, FALSE);
		SetDlgItemInt ( hwnd, 123, g_NetMinutes, FALSE);
        SetDlgItemInt ( hwnd, 132, g_NetSeconds, FALSE);

       return( TRUE );

     // Messages Sent to the Window from it's controls
    case WM_COMMAND:
     if (LOWORD ( wparam ) >103 && LOWORD ( wparam ) < 119  )
        {
		 if (Ready == 1) 
			{
			 DisplayText ( "INFO: Deselect ready to make changes." );
			 return (TRUE);
			}
         if (OppReady == 1) 
			{
			 DisplayText ( "INFO: Cannot make changes when opponent is ready." );   
			 return (TRUE);
			}
        }
        
        if( LOWORD( wparam ) == 105) {SendSetupInfo ('c', WHITE);
                                      g_cNetColor = BLACK;
                                      CheckRadioButton (hwnd, 104, 105, 105 );
                                    }
        if( LOWORD( wparam ) == 104 ) {SendSetupInfo ('c', BLACK);
                                       g_cNetColor = WHITE;   
                                       CheckRadioButton (hwnd, 104, 105, 104 );
                                     }
         if( LOWORD( wparam ) == 106) {
                                     if (IsDlgButtonChecked (hwnd, 106) == 1)
                                                                {CheckDlgButton (hwnd, 106, 0 );
                                                                 SendSetupInfo ('z', 0);}
                                                          else  {CheckDlgButton (hwnd, 106, 1 );
                                                                 SendSetupInfo ('z', 1); }
                                     }

		if ( LOWORD( wparam ) >= 109 && LOWORD( wparam ) <= 115 ) {
  
                                        }
                                        

		 if( LOWORD( wparam ) == 118  ) { // SET TIME
			 nMin = GetDlgItemInt ( hwnd, 123, NULL, FALSE);
			 nSec = GetDlgItemInt ( hwnd, 132, NULL, FALSE);
			 if (nSec > 180) nSec = 180;
			 if (nSec < 0) nSec = 0;
			 if (nMin < 1) nMin = 1;
			 if (nMin > 180) nMin = 180;
			 SendSetupInfo ( 't' , nMin-128, nSec-128);
			}
         
         if( LOWORD( wparam ) == ID_READY ) {
                            Ready = 1;
                            SetDlgItemText (hwnd, 127, "READY" );
                            SendSetupInfo ( 'r' , 1);

                             if (OppReady == 1 )
                               {
								 Ready = 0; OppReady = 0;
								 EndGameBox ( );
                                 EndDialog( hwnd, TRUE );
                                 return( TRUE );
                                }
                            return( TRUE );
                            
                            }
                            
		if( LOWORD( wparam ) == ID_NOTREADY ) {
                            Ready = 0;
                            SetDlgItemText (hwnd, 127, "Not Ready" );
                            SendSetupInfo ( 'r' , 0);
                            return( TRUE );
                           }

		if( LOWORD( wparam ) == 2 ) 
				{
				 SendSetupInfo ( 'q' , 1);
				 scNetMessageS ( SC_GOSPEC, "");
                 Ready = 0; OppReady = 0;
                 EndDialog( hwnd, TRUE );
                 return( TRUE );
                }                           
        break;
    }
    return( FALSE );
}

// ---------------
// NETWORK GAME START
// create the negotiation dialog, Reset the Game Board, and Go.
// ---------------
void StartGameBox ()
    {
	 g_GameResult = DRAW;
     NetStartWnd = CreateDialog ( (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE ) ,"Netgame", mainhwnd, NetStartDlgProc);
    }

void EndGameBox ( )
	{
	 NetStartWnd = NULL;
 	 NewGame (  TRUE  );

	 if (g_cNetColor == BLACK)
		{
		 flipboard = 1;
	     CheckMenuItem (GetMenu(mainhwnd), ID_FLIPBOARD, MF_CHECKED);
		}
	 else 
		{
		 flipboard = 0;
		 CheckMenuItem (GetMenu(mainhwnd), ID_FLIPBOARD, MF_UNCHECKED);
		}
	 DrawBoard ( GameBoard, GameBoard, 255);

	 scNetMessage ( 'n', 'n');
	}
   
//---------------------------------------------------------------
// Function to Draw a single Bitmap
// --------------------------------------------------------------
 void DrawBitmap( HDC hdc, HBITMAP bitmap, int x, int y, int nSize )
{
    BITMAP      bitmapbuff;
    HDC         memorydc;
    POINT       origin, size;
    
    memorydc = CreateCompatibleDC( hdc );
    SelectObject( memorydc, bitmap );
    SetMapMode( memorydc, GetMapMode( hdc ) );
    GetObject( bitmap, sizeof( BITMAP ), (LPSTR) &bitmapbuff );

    origin.x = x;
    origin.y = y;
	if (nSize ==0)
		{size.x = bitmapbuff.bmWidth;
		 size.y = bitmapbuff.bmHeight;
		}
	else
		{size.x = nSize;
		 size.y = nSize;
		}

    DPtoLP( hdc, &origin, 1 );
    DPtoLP( memorydc, &size, 1 );
    if (nSize == bitmapbuff.bmWidth)
		{
		BitBlt( hdc, origin.x, origin.y, size.x, size.y, memorydc, 0, 0, SRCCOPY);
		}
	else {
		SetStretchBltMode( hdc, COLORONCOLOR/*STRETCH_HALFTONE*/ );
		SetBrushOrgEx( hdc, 0, 0, NULL);
		StretchBlt( hdc, origin.x, origin.y, size.x, size.y, memorydc, 0, 0, bitmapbuff.bmWidth, bitmapbuff.bmHeight, SRCCOPY);
		}
    DeleteDC( memorydc );
} 

/*****************************************************************

 Draw a frame around a square clicked on by user, and update int g_selSquare

 *****************************************************************/
 void HighlightSquare ( HDC hdc, int Square, int sqSize, int bRedraw, unsigned long color, int border)
	{
    HBRUSH              brush;
    RECT                rect;

	if (g_bWinBoard == TRUE) return;
    brush = CreateSolidBrush ( color );
    
     if (Square >= 0 && Square <=63) 
		{
		int x=Square%8, y=Square/8;
		if (flipboard == 1) {x = 7-x; y = 7-y;}
		rect.left = x*sqSize + g_xAdd;
		rect.right = (x+1)*sqSize + g_xAdd;
		rect.top = y * sqSize + g_yAdd;
		rect.bottom = (y+1)*sqSize + g_yAdd;
		if (bRedraw == 5) FillRect (hdc, &rect, brush);
		else if (bRedraw == FALSE) 
			{
			FrameRect(hdc, &rect, brush);
			if (border == 2)
				{
				 rect.left+=1; rect.right -=1; rect.top+=1; rect.bottom-=1;
				 FrameRect(hdc, &rect, brush);
				}
			}
			else 
			{
             int index;
			 if (flipboard == 0) index = GameBoard.Sq [ SQ (x,y) ];
				else index = GameBoard.Sq [ SQ (7-x,7-y) ];
			 if (index >=0 && index < 16)
				{
				if ((x+y) % 2 == 0)  DrawBitmap (hdc, PieceBitmaps [ index ] , x*sqSize + g_xAdd , y*sqSize + g_yAdd, sqSize );
					            else DrawBitmap (hdc, PieceBitmaps [ index + 16 ] , x*sqSize + g_xAdd , y*sqSize + g_yAdd, sqSize );
				}
			}
		if (flipboard == 1) {x = 7-x; y = 7-y;}
		}
      DeleteObject( brush );
	}

// -------------------------------------
//
// Draw the chess board from the bitmaps
//
// -------------------------------------
void DrawBoard (SBoard &board, SBoard &board2, int i)
        {
		if (g_bWinBoard == TRUE) return;

        int x, y, mul, add, xadd, square, index;
		int nSize = g_nSqSize;

		HDC hdc = GetDC (mainhwnd);
        if (flipboard == 0) {mul = 1; add = g_yAdd; xadd = g_xAdd;}
                else        {mul = -1; add = (7*nSize) + g_yAdd; xadd = (7*nSize) + g_xAdd;}
      
        for (x = 0; x < 8; x++)
        for (y = 0; y < 8; y++)
                {
				 if ( i!=255 ) {x = i%8; y = i/8; }

                 square = 22 + x + y * 10;
                 index = board.Sq [ square ];
				 //  --- Draw the proper bitmap for the square (light & dark squares) ---
				 if (index >=0 && index < 16)
					{
					if (index == 0) 
						{
						if ((x+y) % 2 == 0) HighlightSquare (hdc, x+y*8, nSize, 5, 0xC0C0C0, 0);
							else HighlightSquare (hdc, x+y*8, nSize, 5, 0x808080, 0);
						}
					else
						{
						if ((x+y) % 2 == 0)  DrawBitmap (hdc, PieceBitmaps [ index ] , x*nSize*mul +xadd , y*nSize*mul +add, nSize );
						            else DrawBitmap (hdc, PieceBitmaps [ index + 16 ] , x*nSize*mul +xadd , y*nSize*mul +add, nSize );
						}
					}
                 //  --- Draw a Rectangle around squares where a piece moves ---
                 if ( board.Sq [ square ] != board2.Sq [square] && bShowLastMove == TRUE)
                        { 
						HighlightSquare (hdc, x + y*8, nSize, FALSE, 0x000000, 2);
                        }
				if (x+y*8 == g_selSquare)	HighlightSquare (hdc, g_selSquare, nSize, FALSE, 0xFFFFFF, 1);
                   
                if (i!=255) {x = 30; y = 30;}// end loop
                }

		ReleaseDC(mainhwnd, hdc );  
        }

// -------------------------------------
//
// HighLight squares from movelist moves[]
//
// -------------------------------------
int Colors [ 10 ] = { 0xDD0000, 0x00DD00, 0x0000DD, 0xDDDD00, 0xDD00DD, 0x00DDDD, 0xEEEEEE, 0x000000 };

void FrameMoves (HDC hdc, int moves[], int numMoves)
        {
        int x, y;
        HBRUSH brush;
        RECT rect;
		int nSize = g_nSqSize;

        if (hdc == NULL) hdc = GetDC (mainhwnd);
  
		for (int i = 0; i < numMoves; i++)
			{
			if (i > 8) brush = CreateSolidBrush( Colors [ i - 8 ] );
	        else brush = CreateSolidBrush( Colors [ i ] );

			if (MSrc(moves [ i ]) < 120)
				{
 				x = (MSrc(moves [ i ]) - 22) % 10;
				y = (MSrc(moves [ i ]) - 22) / 10;
			
				if (flipboard == 1) {x = 7-x; y = 7-y;}

				rect.left = (x)*nSize + g_xAdd;
				rect.right = (x+1)*nSize + g_xAdd;
				rect.top = y*nSize+ g_yAdd;
				rect.bottom = (y+1)*nSize+ g_yAdd;
                        
				FrameRect(hdc, &rect, brush);
				}
	
			x = (MDst(moves [ i ]) -22) % 10;
			y = (MDst(moves [ i ]) -22) / 10;

			if (flipboard == 1) {x = 7-x; y = 7-y;}
			
            rect.left = (x)*nSize + g_xAdd;
            rect.right = (x+1)*nSize + g_xAdd;
            rect.top = y*nSize + g_yAdd;
            rect.bottom = (y+1)*nSize + g_yAdd;
                        
            FrameRect(hdc, &rect, brush);
			DeleteObject( brush );
 			}

         ReleaseDC(mainhwnd, hdc );     
        }


// -----------------------------------
//  Write Names in Side Box
// -----------------------------------
void NamePlayers ( )
{
	if (g_bNetGame == TRUE) return;

	if (PlayerCOLOR == BLACK || PlayerCOLOR == NONE) SetDlgItemText (sideHWND, 114, "Slow Chess");
	if (PlayerCOLOR == WHITE || PlayerCOLOR == NONE) SetDlgItemText (sideHWND, 115, "Slow Chess");
	if (PlayerCOLOR == BLACK || PlayerCOLOR == BOTH) SetDlgItemText (sideHWND, 115, sUserName);
	if (PlayerCOLOR == WHITE || PlayerCOLOR == BOTH) SetDlgItemText (sideHWND, 114, sUserName);
}
// ===========================================
// Display Board Flags
// ===========================================
void DisplayBoardFlags ( )
{
	char buffer[512];
	int j = 0;
	j+=sprintf (buffer + j, "GT: %d  Rev: %d\n", GameBoard.GameTime, GameBoard.LastReversible);
	j+=sprintf (buffer + j, "Pieces W: %d  B: %d\n", GameBoard.nWP, GameBoard.nBP);
	j+=sprintf (buffer + j, "Minors W: %d  B: %d\n", GameBoard.nWMinors, GameBoard.nBMinors);
	j+=sprintf (buffer + j, "Pawns  W: %d  B: %d\n", GameBoard.nWPawns, GameBoard.nBPawns);
	j+=sprintf (buffer + j, "Castle W: %d  B: %d\n", GameBoard.WCastle, GameBoard.BCastle);
	j+=sprintf (buffer + j, "EnPassant: %d\n", GameBoard.EnPassant);
	DisplayText (buffer);
}

void DisplayEval ( )
{
	char buffer[512];
	int PawnEval, RookEval;
	InitSearchGlobals ( GameBoard );
	PawnRookEval ( GameBoard, PawnEval, RookEval);
	int j = 0;
	j+=sprintf (buffer + j, "Eval: %d\n", EvaluateBoard ( GameBoard, -30000, 30000));
	j+=sprintf (buffer + j, "Material: %d\nSqTable: %d  \n", GameBoard.MatVal, GameBoard.TempValue);
	j+=sprintf (buffer + j, "Pawn: %d Rook: %d\n", PawnEval, RookEval );
	j+=sprintf (buffer + j, "PLq: %d \n", Liquidate( GameBoard) );
	DisplayText (buffer);
}

// ===========================================
// Update Game Info in Side Window
// ===========================================
void UpdateMove ( int bTranscript)
{
if (g_bWinBoard == TRUE) return;
char buffer[80], buffer2[80], buffer3[120];
 
	// Unselect any selected square
	HDC hdc = GetDC (mainhwnd);
	HighlightSquare ( hdc, g_selSquare, g_nSqSize, TRUE, 0xFFFFFF, 1); 
	ReleaseDC(mainhwnd, hdc ); 
	g_selSquare = 66;

// ------- Is Side to Move in Check?
int j=sprintf (buffer3, "Move # %d", GameBoard.GameTime / 2 );
if (IsCheck (WHITE, GameBoard) != 0 || IsCheck (BLACK, GameBoard) != 0)
		  strcat (buffer3, "  CHECK!" );
SetDlgItemText (sideHWND, 129, buffer3 );

// ------- Piece Count
int Bqueen = 0, Wqueen = 0, Brook = 0, Wrook = 0, Bbishop = 0, Wbishop = 0, Bknight = 0, Wknight = 0, Bpawn = 0, Wpawn = 0;

for (int i = 22; i < 102; i++)
        {
		 if (GameBoard.Sq[i] == WQUEEN) Wqueen++;
         if (GameBoard.Sq[i] == BQUEEN) Bqueen++;
         if (GameBoard.Sq[i] == WROOK) Wrook++;
         if (GameBoard.Sq[i] == BROOK) Brook++;
         if (GameBoard.Sq[i] == WKNIGHT ) Wknight++;
         if (GameBoard.Sq[i] == BKNIGHT) Bknight++;
         if (GameBoard.Sq[i] == WBISHOP ) Wbishop++;
         if (GameBoard.Sq[i] == BBISHOP) Bbishop++;
         if (GameBoard.Sq[i] == WPAWN ) Wpawn++;
         if (GameBoard.Sq[i] == BPAWN) Bpawn++;          
        }

 strcpy (buffer3, "White:");
 strcpy (buffer, "Black:");
 if (Bqueen != Wqueen) {if (Bqueen > Wqueen) strcat(buffer, "Q, ");
                                      else   strcat(buffer3, "Q, ");
                         }
 if (Brook != Wrook) { ltoa ( abs (Brook - Wrook), buffer2, 10);
                        if (Brook > Wrook) {strcat (buffer, buffer2);
                                            strcat(buffer, "R, ");
                                            }
                                      else  {strcat (buffer3, buffer2);
                                              strcat(buffer3, "R, ");
                                              }
                         }
  if (Bbishop != Wbishop) { ltoa ( abs (Bbishop - Wbishop), buffer2, 10);
                        if (Bbishop > Wbishop) {strcat (buffer, buffer2);
                                            strcat(buffer, "B, ");
                                            }
                                      else  {strcat (buffer3, buffer2);
                                              strcat(buffer3, "B, ");
                                              }
                         }
 if (Bknight != Wknight) { ltoa ( abs (Bknight - Wknight), buffer2, 10);
                        if (Bknight > Wknight) {strcat (buffer, buffer2);
                                            strcat(buffer, "N, ");
                                            }
                                      else  {strcat (buffer3, buffer2);
                                              strcat(buffer3, "N, ");
                                              }
                         }
 if (Bpawn != Wpawn) { 
						ltoa ( abs (Bpawn - Wpawn), buffer2, 10);
                        if (Bpawn > Wpawn) {strcat (buffer, buffer2);
                                            strcat(buffer, "p, ");
                                            }
                                      else  {strcat (buffer3, buffer2);
                                              strcat(buffer3, "p, ");
                                              }
                         }
                        
  if (strcmp (buffer3, "White:") != 0) buffer3[strlen(buffer3)-2] = NULL;
  if (strcmp (buffer, "Black:") != 0) buffer[strlen(buffer)-2] = NULL;
        
  // Remove Last Comma
  strcat (buffer3, "\n");
  strcat (buffer3, buffer);                                                                   

  SetDlgItemText (sideHWND, 127, buffer3);

  // ------------Move List
 UpdateListTranscript ( TRUE ); 
 }

// ------------
// Set the amount of time to Think For
// ------------
void guiSetThinkTime (int color)
{
	 if (g_bWinBoard == TRUE) return; // Already set from Winboard's time remaining.

	 if (nSecsPerMove == 0) // Use the Time Control
		{
		float nTimeLeft = 2.0f;
		int GameTime = NumTranMoves ( );
		if (color == WHITE) nTimeLeft = float(Minutes * 6000 - g_nWhiteTime + ( (GameTime + 1) /2) * Seconds * 100);
		if (color == BLACK) nTimeLeft = float(Minutes * 6000 - g_nBlackTime + ( (GameTime   ) /2) * Seconds * 100); 
		SetThinkTime ( nTimeLeft *.01f, 0, Seconds);
		}
	 else Thinktime = (float)nSecsPerMove; // Use set time per move
}


// ------------
// Do the next move from the PV,
// if it's a legal move, store the board as the ponder board and start thinking on that board.
// (After doing the real move, the board is compared to the ponder board to see if it's a ponder hit.
// and g_ponder is set accordingly.)
// ------------
void PonderNextMove(  ) 
	{
	 int nTemp, nTemp2;
	 SBoard PonderBoard = GameBoard;

	 int src = MSrc(PV [0][ 2 ]);
	 int dst = MDst(PV [0][ 2 ]);

	 int dX = (dst-22)%10;
	 int dY = (dst-22)/10;

	 g_ponderSrc = src;
	 g_ponderDst = dst;

	 upgrade = WQUEEN;

	 if ( PV[0][0] > 1 && DoMoveXY ( PonderBoard, (src-22)%10, (src-22)/10, dX, dY, PonderBoard.SideToMove, NULL, FALSE, NULL, nTemp, nTemp2) == 1)
		{
		guiSetThinkTime ( PonderBoard.SideToMove);
		g_PonderBoard = PonderBoard;

		DisplayText ("Pondering...");

		g_ponder = 1;

		thinking = 2;
        CompMove (PonderBoard,  2, MaxAhead);

		if (g_ponder == 1 || stopthinking > 0) 
			{
			g_ponder = -1;
			}
		else if (g_ponder == 2) // Opponent played pondered move
			{
			g_ponder = 0;
			GameBoard = PonderBoard;
			if (g_bWinBoard == FALSE)
				{
				TrimList ();
				UpdateMove( TRUE );
				CheckGameOver (GameBoard, TRUE, g_GameResult);
				}
			SetEvent (hAction);
			}
		else if (g_ponder == 3) // Pondered wrong Move
			{	
			 SetEvent (hAction);
			}
       	}
	 else 
		{
		 DisplayText ("No valid move to ponder.");
		 g_ponder = -1;
		}
	
	g_ponder = 0;
	thinking = 0;
	ClearBoard (g_PonderBoard);
    } 

// =================================================================================\
// Threaded Function (for multitasking while the computer thinks)
// =================================================================================/
int g_nFlags = 0;

void  FAR ThinkingThread( void FAR *parm) 
	{
	parm = parm;

	// Think of Move
     while (1==1)
	 {
	 SetEvent (hEngineReady);
	 WaitForSingleObject (hAction, INFINITE);

	 // Analyze
	 if (g_nFlags == 1)
		{
		thinking = 1;
		g_ponder = 1;
        CompMove (GameBoard, 1, 28);
		thinking = 0;
        g_ponder = 0;
		}

	 // Move & Ponder
	 if (g_nFlags == 0)
	 {
	 ResetEvent ( hEngineReady ); //Don't take input until everything is ready

	 if (g_ponder == 0)
			{
			thinking = 1;
			while ( (PlayerCOLOR == NONE || PlayerCOLOR!=GameBoard.SideToMove) && g_GameResult == NONE && PlayerCOLOR !=BOTH )
				{
				guiSetThinkTime ( GameBoard.SideToMove);

				CompMove (GameBoard, 0, MaxAhead); 
				if (g_bICC == FALSE && g_bWinBoard == FALSE) CheckGameOver (GameBoard, TRUE, g_GameResult);
				}

			thinking = 0;
			g_ponder = 0;
			if (PlayerCOLOR == NONE) PlayerCOLOR = BOTH;
			}	

	 if ( g_ponder == 0 && bPonder == TRUE && stopthinking != 3 && thinking == 0)
			{
			// Start to Ponder?
			if ((PlayerCOLOR == WHITE || PlayerCOLOR == BLACK) && g_GameResult == NONE ) 
			PonderNextMove ( );
			}
		}
	}
          
     _endthread();
    } 

// ----------------
//
// ----------------
void ThinkOfMove ( int nFlags )
{
 
  if (g_ponder > 0) return;
  if (thinking != 0 || g_bNetGame == TRUE)  return;

  if (nFlags == 0 && PlayerCOLOR != BOTH) 
	{ 
	  g_nFlags = nFlags;
	  SetEvent (hAction);
	}
  if (nFlags == 1)  // Analyze
	{
	  g_nFlags = nFlags;
	  SetEvent (hAction);
	}
}

/* ===============================================================
 * The Player has entered a move? move this to engine thread.

 ================================================================ */ 
int PlayerMove (int sx, int sy, int dx, int dy, int piece )
{
 int src=0, dst=0;

 // Is engine ready to receive a move?
 WaitForSingleObject ( hEngineReady, 1000 );

 if (DoMoveXY( GameBoard, sx, sy, dx , dy , GameBoard.SideToMove, piece, TRUE, NULL, src, dst) == 1)
         {
		  g_GameResult = NONE;
		  g_selSquare = 67;
		  if (g_bWinBoard == FALSE)
			{
			DrawBoard (  GameBoard,GameBoard, 255);
			TrimList ();                   
			UpdateMove( TRUE );
			}
		
		  if (g_ponder == 1)
				{
				if (memcmp ( &GameBoard, &g_PonderBoard, sizeof (SBoard)) == 0)
					{g_ponder = 2; // Did the move pondered
					}
				else g_ponder = 3; // Didn't do the move pondered
				}

		  if (g_bNetGame == TRUE) scNetSendMove ( );

		  if ( CheckGameOver ( GameBoard, FALSE, g_GameResult) == FALSE)
				{
				ThinkOfMove ( 0 );
				}
		  else
			{
		    if (g_bICC == FALSE && g_bWinBoard == FALSE) CheckGameOver ( GameBoard, TRUE, g_GameResult);
			if (g_bWinBoard == TRUE)
				{
				wbTestSendResult ( GameBoard );
				}
			}
		  return TRUE;
          }
 return FALSE;
}

// ----------------
// Display Text in the sidemenu info Box
// ----------------
void DisplayText ( char *String)
{
if (sideHWND != NULL) SetDlgItemText (sideHWND, 122, String );
}    
    
/* =============================================
 *
 * GAME DIALOG - processes messages for the sidemenu dialog.
 *
   ============================================== */
BOOL CALLBACK GameDlgProc( HWND hwnd, UINT msg,  UINT wparam, LONG lparam )
{
    lparam = lparam;              /* turn off warning */
    HWND hwndList;
    
    hwnd = hwnd;
   
    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:

          // Computer Move button
          if( LOWORD( wparam ) == 107 ) 
					{ // Computer Moves
					g_GameResult = NONE;
                    if (thinking != 0) 
						{
						stopthinking = 1;
						if (thinking == 1) if (GameBoard.SideToMove == WHITE) PlayerCOLOR = BLACK; else PlayerCOLOR = WHITE;
                        }
                    else {
						  if (GameBoard.SideToMove == WHITE) PlayerCOLOR = BLACK; else PlayerCOLOR = WHITE;
						  ThinkOfMove ( 0 );
                         }
					NamePlayers();
					SetFocus (mainhwnd);
                    }

          if (thinking == 1) return ( FALSE );                   
          
          if ( LOWORD( wparam ) == 109) 
				{
				hwndList = GetDlgItem(sideHWND, 109); 
                int temp = (SendMessage (hwndList, LB_GETCURSEL, 0, 0 ) + 1) * 2;
                ReplayGame (GameBoard, temp);
                DrawBoard ( GameBoard, GameBoard, 255);
                UpdateMove( TRUE );
                } 

        // Game Time Controls        
          if( LOWORD( wparam ) == ID_FIRSTMOVE ) 
								{ 
								 GameBoard = g_StartBoard;
                                 DrawBoard (GameBoard, GameBoard, 255);
                                 UpdateMove( TRUE );
								 SetFocus (mainhwnd);
                                }
                                        
          if( LOWORD( wparam ) == ID_LASTMOVE )  
					{
					 ReplayGame (GameBoard, 1000);
                     DrawBoard (  GameBoard,GameBoard, 255);
                     UpdateMove( TRUE );
					 SetFocus (mainhwnd);
                    }

          if( LOWORD( wparam ) == ID_BACKFULLMOVE ) 
					{
                    ReplayGame (GameBoard, GameBoard.GameTime - 2);
					DrawBoard ( GameBoard, GameBoard, 255);
					UpdateMove( TRUE );
					SetFocus (mainhwnd);
                    }

          if( LOWORD( wparam ) == ID_FORWARDFULLMOVE ) 
					{ 
                    ReplayGame (GameBoard, GameBoard.GameTime + 2);               
                    DrawBoard ( GameBoard, GameBoard, 255);
					UpdateMove( TRUE );
					SetFocus (mainhwnd);
                    }

         if( LOWORD( wparam ) == ID_BACKHALFMOVE )
					{
                     ReplayGame (GameBoard, GameBoard.GameTime - 1);
                     DrawBoard (  GameBoard,GameBoard, 255);
                     UpdateMove( TRUE );
					 SetFocus (mainhwnd);
					}

         if( LOWORD( wparam ) == ID_FORWARDHALFMOVE  ) 
					{
			         ReplayGame (GameBoard, GameBoard.GameTime + 1);
                     DrawBoard ( GameBoard, GameBoard, 255);
                     UpdateMove( TRUE );
					 SetFocus (mainhwnd);
                    }    
		 
		 if ( LOWORD( wparam ) == ID_BOOK )
				{
				int BMoves [ 50 ];
				short wValues [ 50 ];
				int nMoves;
				nMoves = pNewBook->FindMoves (GameBoard, BMoves, wValues);
				char buffer [ 250 ];
				sprintf (buffer, "# of Book Moves: %d \n", nMoves);
				DisplayText ( buffer );
				FrameMoves (NULL, BMoves, nMoves);
				SetFocus (mainhwnd);
				} 
    }    
    return( FALSE );
}

// -------------------------
// Turn Netgame State on/off (menu, title, etc.)
// -------------------------
void NetGameOn (   )
{
HMENU MainMenu = GetMenu(mainhwnd);
	EnableMenuItem ( MainMenu, ID_INTERNET_DIRECTCONNECTION, MF_GRAYED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_LISTEN,			MF_GRAYED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_DISCONNECT,		MF_ENABLED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_REQUESTNEWGAME,	MF_ENABLED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_OFFERDRAW, MF_ENABLED);
	EnableMenuItem ( MainMenu, ID_INTERNET_RESIGN, MF_ENABLED);
	EnableMenuItem ( MainMenu, ID_INTERNET_CONTINUEGAME, MF_ENABLED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_JOINTABLE, MF_ENABLED); 
	
	if (bServer == TRUE) SetWindowText(mainhwnd, "Slow Chess 2.82 - NETGAME (Server)" );
		else			 SetWindowText(mainhwnd, "Slow Chess 2.82 - NETGAME" );
	DrawBoard (  GameBoard, GameBoard, 255);
	g_bNetGame = TRUE;
	if (ChatWnd == NULL)
        {
		 HINSTANCE hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
	     ChatWnd = CreateDialog(  hinst ,"CHAT", mainhwnd, ChatProc);
		 if (g_nSqSize == 64) SizeWindowToDefault();
		 MoveWindow ( ChatWnd, 2, g_nHeight-125, g_nWidth-238, 125 , TRUE );
         SetFocus(ChatWnd);
		 ResizeFunc ( );
		 RECT rect;
		 rect.top = 0; rect.left = 0; rect.right = g_nWidth; rect.bottom = g_nHeight;
		 InvalidateRect (mainhwnd, &rect, FALSE);
		}

	SetDlgItemText (sideHWND, 118, "Table 0"); 
}

// -------------------------
//
void NetGameOff ( int bClose  )
{
HMENU MainMenu = GetMenu(mainhwnd);
	EnableMenuItem ( MainMenu, ID_INTERNET_DIRECTCONNECTION, MF_ENABLED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_LISTEN,			MF_ENABLED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_DISCONNECT,		MF_GRAYED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_REQUESTNEWGAME,	MF_GRAYED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_OFFERDRAW, MF_GRAYED);
	EnableMenuItem ( MainMenu, ID_INTERNET_RESIGN, MF_GRAYED);
	EnableMenuItem ( MainMenu, ID_INTERNET_CONTINUEGAME, MF_GRAYED); 
	EnableMenuItem ( MainMenu, ID_INTERNET_JOINTABLE, MF_GRAYED); 

	SetWindowText(mainhwnd, g_sSlow );

	g_bNetGame = FALSE;

	if (bClose == TRUE) CloseSocket ( );

	if (ChatWnd != NULL)
        {
		 DrawBoard (  GameBoard, GameBoard, 255);
		 DestroyWindow (ChatWnd);
		 ChatWnd = NULL;
		 if (g_nSqSize == 64) SizeWindowToDefault();
		 ResizeFunc ();
		}
}

// -------------------------
// Reset everything needed to begin a new game
// -------------------------
void NewGame ( int bDraw  )
{
	stopthinking = 3;
	g_ponderSrc = -1;
	g_nBlackTime = 0;
	g_nWhiteTime = 0;
	g_selSquare = 66;
	Setup_Board (GameBoard, 1);
	ClearBoard (g_PonderBoard);
	g_StartBoard = GameBoard;
	ClearTranscript ( NULL );
	g_GameResult = NONE;
	g_startTime = clock ();
	NamePlayers();

	while (thinking != 0)
		{
		Sleep (50);
		}
	if (bDraw == TRUE) 
		{
		MoveBox( 0 );
		UpdateMove ( FALSE );
		DrawBoard (  GameBoard, GameBoard, 255);
		SetDlgItemText (sideHWND, 122, "The Game Begins.." );
		}
}

/* ===============================================================
 *
 * MENU COMMANDS for the main application window
 *
 ================================================================ */ 
void MenuCommands (HWND hwnd, LPARAM lparam, WPARAM wparam )
{
  HINSTANCE hinst;
  static char fname[1500], cfname[1500]; 
  static int nLoadGame = 1;
  int trys = 0;
  int x;
  lparam = lparam;


switch( LOWORD( wparam ) ) {

		// --------------------------
		// FILE Commands
		// --------------------------
        case ID_NEW: // new

		if (g_bNetGame == TRUE) break;
		NewGame (  TRUE  );
		ThinkOfMove ( 0 );
        break;

        case ID_LOAD_PGN: // load
		if (g_bNetGame == TRUE) break;
		nLoadGame = 1;
		strcpy (fname, "*.pgn");
        if (GetFileName (hwnd, 0, fname, sPGNFile) == TRUE)
			if (LoadGame (fname,  1, TRUE) == 1)
				   {  
					strcpy (cfname, fname);
					DrawBoard (  GameBoard, GameBoard, 255);
		            UpdateMove ( FALSE );
	               }
    
        break;

        case ID_NEXT_PGN: // load
		case ID_PREV_PGN:
		if (g_bNetGame == TRUE) break;
		if ( LOWORD( wparam ) == ID_NEXT_PGN) 
			{
			nLoadGame++;
			}
		if ( LOWORD( wparam ) == ID_PREV_PGN) 
			{
			nLoadGame--;
			if (nLoadGame < 1) nLoadGame = 1;
			}
		if ((nLoadGame = LoadGame (cfname,  nLoadGame, TRUE)) != 0)
			{  
				DrawBoard ( GameBoard, GameBoard, 255);
		        UpdateMove( FALSE );
	        }
			else nLoadGame = 1;
    
        break;

        
     /*   case ID_SAVEAS: // save
	    strcpy (fname, "*.pgn");
        GetFileName (hwnd, 1, fname, sPGNFile);
        SaveGame (g_GameResult, fname, FALSE);
        
        break;*/

        case ID_APPEND_SAVEAS: // save
	    strcpy (fname, "*.pgn");
        GetFileName (hwnd, 1, fname, sPGNFile);
        SaveGame (g_GameResult, fname, TRUE);
        break;

        case 71:
        if (bShowLastMove == TRUE) {bShowLastMove = FALSE;
                                 CheckMenuItem (GetMenu(hwnd), 71, MF_UNCHECKED); }
                else {bShowLastMove = TRUE;
                       CheckMenuItem (GetMenu(hwnd), 71, MF_CHECKED); }
        break;

		// --------------------------
		// INTERNET Commands
		// --------------------------
		case ID_INTERNET_DIRECTCONNECTION:

		hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
        DialogBox( hinst  ,"ConnectBox", hwnd,  ConnectDlgProc);
       
		break;

		case ID_INTERNET_LISTEN:
		InitializeSockets ( );
		if (WaitForConnect ( 14324 ) == 2)
			{
			// Connect Yourself
			g_cNetColor = NONE;
			NetGameOn ( );
			NewGame (  TRUE  );
			scNetMessageS ( SC_NAME, sUserName);
			scNetMessage ( SC_TABLE, 0);
			}
		break;

		case ID_INTERNET_DISCONNECT:
		if (g_bNetGame == TRUE ) 
			{
			NetGameOff ( TRUE );
			}
		break;

		case ID_INTERNET_REQUESTNEWGAME:

		if (g_bNetGame == TRUE) 
			{
			// Game not over & not on first move, cannot have a new game
			if (g_GameResult == NONE && (GameBoard.GameTime > 1 || CurrentBoard ( GameBoard ) == FALSE) ) 
				return; 
			scNetMessage ( SC_NEW, 1 );
			}

		break;

		case ID_INTERNET_OFFERDRAW:
		if (g_bNetGame == TRUE && g_GameResult == NONE) scNetMessage ( SC_DRAW, 1 );
			break;

		case ID_INTERNET_RESIGN:
		if (g_bNetGame == TRUE && g_GameResult == NONE) scNetMessage ( SC_RESIGN, 1 );
			break;

		case ID_INTERNET_CONTINUEGAME:
		if (g_bNetGame == TRUE) scNetMessage ( SC_CONTINUE, 1 );
			break;

		case ID_INTERNET_JOINTABLE:
			hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
			DialogBox( hinst  ,"Table", hwnd,  TableDlgProc);
			break;
   
		case ID_INTERNET_TELLMEMYIP:
			InitializeSockets ( );
			DisplayMyIP();
		break;
	   //---------------------------------------
	   // COPY/PASTE commands
	   //---------------------------------------
       case ID_COPY_GAME:
	   Defaults.SetCurDir ();
       SaveGame (0, "clipboard.pgn", FALSE);
       break;
              
       case ID_PASTE_GAME:
	   if (g_bNetGame == TRUE) break;
	   Defaults.SetCurDir ();
       SaveGame (0, "undopaste.pgn", FALSE);
       if (LoadGame ("clipboard.pgn",  1, FALSE) == 1)
               {  
                DrawBoard ( GameBoard, GameBoard, 255);
                UpdateMove( TRUE );
               }

       break;

       case ID_UNDOPASTE_GAME:
	   Defaults.SetCurDir ();
             if (LoadGame ("undopaste.pgn",  1, FALSE) == 1)
               { 
                DrawBoard ( GameBoard, GameBoard, 255);
                UpdateMove( TRUE );
               }
       break;

	   case ID_CLIP_PASTE:
		if (g_bNetGame == TRUE) break;
		ClipPasteTranscript ( hwnd, FALSE );
		DrawBoard  ( GameBoard, GameBoard, 255);
	   break;

	   case ID_CLIP_PASTE_POS:
		if (g_bNetGame == TRUE) break;
		ClipPasteTranscript ( hwnd, TRUE );
		DrawBoard  ( GameBoard, GameBoard, 255);
		break;

	   case ID_CLIP_COPY:
		ClipCopyTranscript  ( hwnd, FALSE );
		break;

	   case ID_CLIP_COPY_POS:
		ClipCopyTranscript  ( hwnd, TRUE );
		break;

	   //
	   // HASH TABLE commands
	   //
        case 74:
        if (bHashing == TRUE) {bHashing = FALSE;
                           CheckMenuItem (GetMenu(hwnd), 74, MF_UNCHECKED); }
                else {bHashing = TRUE;
                       CheckMenuItem (GetMenu(hwnd), 74, MF_CHECKED);  }
        break;

	   case 1234:
	   case 1235:
	   case 1236:
	   case 1237:
	   case 1238:
	   case 1239:
		   Defaults.nHashBits = LOWORD( wparam ) - 1234 + 19;
		   SetHashSizeBits ( Defaults.nHashBits );
		   for (x = 1234; x <= 1239; x++) CheckMenuItem (GetMenu(hwnd), x, MF_UNCHECKED);
		   CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_CHECKED);
	   break;
	   //---------------------------------------
	   // PLAYING LEVEL commands
	   //---------------------------------------

	   // Set Game Time Control
       case 61: 
        hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
        DialogBox( hinst ,"TimeSet", hwnd, (DLGPROC) TimeDlgProc);
       break;

	   case ID_TIMELIMIT_RESETCLOCK:
		if (g_bNetGame == TRUE) break;
		DisplayText ("Game clock reset.");
		g_nBlackTime = 0;
		g_nWhiteTime = 0;
		g_startTime = clock ();
	   break;

	   // Set Max Search Depth
       case 160: if (LOWORD( wparam ) == 160) MaxAhead = 26;
       case 158: if (LOWORD( wparam ) == 158) MaxAhead = 8;
       case 157: if (LOWORD( wparam ) == 157) MaxAhead = 7;
       case 156: if (LOWORD( wparam ) == 156) MaxAhead = 6;
       case 155: if (LOWORD( wparam ) == 155) MaxAhead = 5;
       case 154: if (LOWORD( wparam ) == 154) MaxAhead = 4;
       case 153: if (LOWORD( wparam ) == 153) MaxAhead = 3;
       case 152: if (LOWORD( wparam ) == 152) MaxAhead = 2;
       case 151: if (LOWORD( wparam ) == 151) MaxAhead = 1;
       for (x = 151; x <= 160; x++)
                CheckMenuItem (GetMenu(hwnd), x, MF_UNCHECKED);
      
       CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_CHECKED);
       break;

	   case ID_USERDEFINED:  // Set Computer Levels
        hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
        DialogBox( hinst ,"SearchSet", hwnd, (DLGPROC) LevelDlgProc);
		CheckMenuItem (GetMenu(hwnd), ID_USERDEFINED, MF_CHECKED);
        for (x = 151; x <= 160; x++)
                if (x!=159) CheckMenuItem (GetMenu(hwnd), x, MF_UNCHECKED);
       break;

	   // Set Computer Color
       case 41:
        PlayerCOLOR = WHITE;
		NamePlayers ();
		ThinkOfMove ( 0 );
       break;
       
       case 40:
        PlayerCOLOR = BLACK;
		NamePlayers ();
		ThinkOfMove ( 0 );
       break;

       case 42:
        PlayerCOLOR = BOTH;
		NamePlayers ();
		g_GameResult = NONE;
       break;

       case 43: // Computer plays itself
        PlayerCOLOR = NONE;
		NamePlayers ();
		ThinkOfMove ( 0 );
       break;
     

       case ID_ANALYZE: // Suggest a Move
		if (thinking == 1) stopthinking = 3;
			else ThinkOfMove ( 1 );
       break;
       

	   case ID_LOADBOOK:
		strcpy (fname, "*.scb");
        if (GetFileName (hwnd, 0, fname, sBookFile) == TRUE) pNewBook->Load (fname);
		break;

	   case ID_SAVEBOOK:
		strcpy (fname, "*.scb");
        if (GetFileName (hwnd, 1, fname, sBookFile) == TRUE) pNewBook->Save ( fname, 0 );
		break;

	   case ID_CLEARBOOK:
		   ClearBook ();
		break;

       case ID_SEARCHINFO:
       if (bDisplayInfo == TRUE) 
			{
	        CheckMenuItem (GetMenu(hwnd), ID_SEARCHINFO, MF_UNCHECKED);
		    bDisplayInfo = FALSE;
			}
        else 
			{bDisplayInfo = TRUE;
	         CheckMenuItem (GetMenu(hwnd), ID_SEARCHINFO, MF_CHECKED);
			}
       break;

       case 110: // Stop Thinking
       stopthinking = 1;
       break;
       
       case ID_FLIPBOARD: // FlipBoard
       if (flipboard == 0) {flipboard = 1;
                             CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_CHECKED);
                            }
                            else
                            {flipboard = 0;
                             CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_UNCHECKED);   
                            }                           
       DrawBoard (  GameBoard, GameBoard, 255);
        
       break;

	   case ID_OPTIONS_EVALWHITE:
	   if (bEvalWhite == TRUE) 
			{bEvalWhite = FALSE;
	         CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_UNCHECKED);
            }
            else
            {bEvalWhite = TRUE;
             CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_CHECKED);
			}
	   break;

	   case ID_PONDER:
       if (bPonder == FALSE) {bPonder = TRUE;
                             CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_CHECKED);
                            }
                            else
                            {bPonder = FALSE;
                             CheckMenuItem (GetMenu(hwnd), LOWORD( wparam ), MF_UNCHECKED);   
                            }
		break;

		// --------------------------
		// DEVELOP commands
		// --------------------------
		case ID_EXPERIMENT:

		Clear_Hash();

		if (bExperiment == 1) 
			{bExperiment = 0;
			 CheckMenuItem (GetMenu(hwnd), ID_EXPERIMENT, MF_UNCHECKED);
			}
		else
			{
			bExperiment = 1;
  		    CheckMenuItem (GetMenu(hwnd), ID_EXPERIMENT, MF_CHECKED);
			}
		break;

		case 75:
		Clear_Hash();
		break;

		case ID_DEVELOP_DISPLAYTTENTRY:
			DisplayTTEntry ();
		break;

		case ID_DISPLAYPV:
		    DisplayPV ();
		break;

		case ID_PLAYPV:
			PlayPV();
		break;

		case ID_DISPLAYFLAGS:
		   DisplayBoardFlags ( );
		break;

		case ID_DISPLAYEVAL:
			DisplayEval ( );
		break;

		case ID_OPTIONS_PLAYERINFO:
        hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
        DialogBox( hinst ,"PLAYER", hwnd, (DLGPROC) PlayerDlgProc);
		break;

		case ID_DEFAULT_SIZE:
			SizeWindowToDefault ( );
		break;

	/*	case ID_SETUPBOARD:
			if (g_bSetup == FALSE)
				{
				ClearBoard (GameBoard);
				DrawBoard (  GameBoard, GameBoard, 255);
				g_bSetup = TRUE;
				CheckMenuItem (GetMenu(hwnd), ID_SETUPBOARD, MF_CHECKED);
				}
			else 
				{
				g_bSetup = FALSE;
				CheckMenuItem (GetMenu(hwnd), ID_SETUPBOARD, MF_UNCHECKED);
				}
		break;*/

		case ID_DEVELOP_PGNTOBOOK:
		if (g_bNetGame == TRUE) break;

        hinst = (HINSTANCE)GetWindowLong( mainhwnd, GWL_HINSTANCE );
        DialogBox( hinst ,"BOOK", hwnd, (DLGPROC) BookDlgProc);
		break;

	/*	case ID_WINDEBUG:
			if (g_bDebug == TRUE) 
				{g_bDebug = FALSE;
				 CheckMenuItem (GetMenu(hwnd), ID_WINDEBUG, MF_UNCHECKED);
				}
			else 
				{g_bDebug = TRUE;
				 CheckMenuItem (GetMenu(hwnd), ID_WINDEBUG, MF_CHECKED);
				}

		break;*/

        case ID_EXIT: // exit
		Defaults.Save ("Schess.dft");
		// In case the net was operating
		CloseSocket ();
		WSACleanup ();
        PostQuitMessage( 0 );
        break;
      } 
}


/* ===============================================================
 * WindowProc - handle messages for the main application window

 ================================================================ */ 

long CALLBACK WindowProc( HWND hwnd, unsigned msg,
                                     UINT wparam, LONG lparam )
{
    HDC                 hdc;
    
    static int x,y;
    int ox = 0, oy = 0; 
	int temp, bRead;
	char acReadBuffer [ HOLDSIZE ];
    
// Process messages posted to the main window

	// Internet Messages from Clients
	if ( msg >= INWM_SPEC_READDATA && msg <= (INWM_SPEC_READDATA + MAXCLIENTS) )
		{
		if (WSAGETSELECTEVENT(lparam) == FD_READ)
			{
			temp = 2;
			bRead = TRUE;
			while (temp == 2)
				{
				temp = ReceiveDataMessage ( acReadBuffer,  msg - INWM_SPEC_READDATA, bRead );
				if (temp >= 1) ReceiveDataServer ( acReadBuffer[2], &acReadBuffer[3], msg - INWM_SPEC_READDATA ); // Client Receives Data
				bRead = FALSE;
				}
			return TRUE;
			}
		if (WSAGETSELECTEVENT(lparam) == FD_CLOSE)
			{
			ReceiveDataServer ( SC_DISCONNECT, "", msg - INWM_SPEC_READDATA );
			sprintf (acReadBuffer, "Client %d is disconnected", msg - INWM_SPEC_READDATA);
			if (WSAGETSELECTERROR(lparam) == WSAECONNRESET)	strcat (acReadBuffer, "\n(Reason: Connection Reset by Client)");
			if (WSAGETSELECTERROR(lparam) == WSAENETDOWN)	strcat (acReadBuffer, "\n(Reason: Network Failed)");
			if (WSAGETSELECTERROR(lparam) == WSAECONNABORTED) strcat (acReadBuffer, "\n(Reason: Failed or Timed Out.)");
			DisplayText (acReadBuffer);
			}
		}
			
    switch( msg ) {

	case INWM_READDATA: // Internet Socket Data

		if (WSAGETSELECTEVENT(lparam) == FD_READ)
		{
		temp = 2;
		bRead = TRUE;
		while (temp == 2)
			{
			temp = ReceiveDataMessage ( acReadBuffer, 0, bRead );
			if (temp >= 1) ReceiveDataClient ( acReadBuffer[2], &acReadBuffer[3] ); // Client Receives Data
			bRead = FALSE;
			}
		}
		if (WSAGETSELECTEVENT(lparam) == FD_CLOSE)
			{
			ReceiveDataClient ( SC_DISC, "");
			if (WSAGETSELECTERROR(lparam) == WSAECONNRESET)	DisplayText ("Connection Reset by Server");
			if (WSAGETSELECTERROR(lparam) == WSAENETDOWN)	DisplayText ("Network Failed");
			if (WSAGETSELECTERROR(lparam) == WSAECONNABORTED) DisplayText ("Failed or Timed Out");
			}
	break;

/*	case  INWM_ICC_READDATA:
		if ( ICCReceiveDataMessage ( acReadBuffer ) > 0)
            {
			 ICCReceiveData (   acReadBuffer );
            }
	break;*/

	case INWM_CONNECTION: // Internet Connection Request
		if (AcceptConnection ( ) == 1)
			{
			NetGameOn (); // Should be on already
			}
	break;

     case WM_KEYDOWN:

     if (thinking == 1) break;
     if (int(wparam) == '1') pNewBook->AddPosition ( GameBoard, -2, FALSE); 
     if (int(wparam) == '2') pNewBook->AddPosition ( GameBoard, -1, FALSE); 
     if (int(wparam) == '3') pNewBook->AddPosition ( GameBoard, 0, FALSE); 
     if (int(wparam) == '4') pNewBook->AddPosition ( GameBoard, 1, FALSE); 
     if (int(wparam) == '5') pNewBook->AddPosition ( GameBoard, 2, FALSE); 
	 if (int(wparam) == '6') pNewBook->RemovePosition ( GameBoard , FALSE);
	 if (int(wparam) == 'L') PostMessage (hwnd, WM_COMMAND, ID_NEXT_PGN, 0);
	 if (int(wparam) == 'K') PostMessage (hwnd, WM_COMMAND, ID_PREV_PGN, 0);
	 if (int(wparam) == 'C') PostMessage (hwnd, WM_COMMAND, ID_COPY_GAME, 0);
	 if (int(wparam) == 'V') PostMessage (hwnd, WM_COMMAND, ID_PASTE_GAME, 0);
     
     break;

    // ---------------------------------------    
	//  Check to see which square is clicked, and do move if possible
	// ---------------------------------------  
    case WM_LBUTTONDOWN:

    SetFocus (hwnd);
    
    if (thinking == 1) break;
	if (g_bNetGame == TRUE)
		{
		if (GameBoard.SideToMove != g_cNetColor) break;
		if (CurrentBoard ( GameBoard ) == FALSE) break;
		if (g_GameResult != NONE) break;
		}
                                       
     x = (int)(short)LOWORD(lparam );
     y = (int)(short)HIWORD(lparam );

     x = (x-g_xAdd) / g_nSqSize;
     y = (y-g_yAdd) / g_nSqSize;

     if (flipboard == 1) {x = 7-x; y = 7-y;}

     if (x > 7 || x < 0 || y > 7 || y < 0)  // Something Weird?
		{
		 x = ox; 
		 y = oy;
		 break;
		}

	 if (g_bSetup == TRUE)
		{
		 if (GetKeyState ( 'K' ) < 0) AddPiece (GameBoard, x, y, WKING);
		 else if (GetKeyState ( 'P' ) < 0) AddPiece (GameBoard, x, y, WPAWN);
		 else if (GetKeyState ( 'N' ) < 0) AddPiece (GameBoard, x, y, WKNIGHT);
		 else if (GetKeyState ( 'B' ) < 0) AddPiece (GameBoard, x, y, WBISHOP);
		 else if (GetKeyState ( 'R' ) < 0) AddPiece (GameBoard, x, y, WROOK);
		 else if (GetKeyState ( 'Q' ) < 0) AddPiece (GameBoard, x, y, WQUEEN);
		 
		 DrawBoard (  GameBoard, GameBoard, 255);
		 return (TRUE);
		}

     oy = y;  ox = x;
     
     if (g_selSquare< 66)
        {
		 upgrade = 0;
		 PlayerMove ( g_selSquare%8, g_selSquare/8, x, y , 0);
		 upgrade = WQUEEN;
        }
   
     if (GameBoard.Sq[22 + x + y * 10] != EMPTY && g_selSquare !=67)
       if ( (GameBoard.SideToMove == WHITE && GameBoard.Sq[22 + x + y * 10] < 8) || (GameBoard.SideToMove == BLACK && GameBoard.Sq[22 + x + y * 10] > 8))
            {
			HDC hdc = GetDC (mainhwnd);
			HighlightSquare ( hdc, g_selSquare, g_nSqSize, TRUE, 0xFFFFFF, 1); 
		    g_selSquare = x + y*8;
			HighlightSquare ( hdc, g_selSquare, g_nSqSize, FALSE, 0xFFFFFF, 1);
			ReleaseDC(mainhwnd, hdc ); 
			}
 
      if (g_selSquare == 67) g_selSquare = 66;    
      break;
    // ---------------------------------------    

	//  Setup Board?
	// ---------------------------------------  
    case WM_RBUTTONDOWN:            

	//TrackPopupMenu ( LoadMenu ( hinst, "SETUPMENU"), , 0, 1, 0, w, NULL );
	break;

    // ---------------------------------------    
	//  Menu Commands
	// ---------------------------------------       
    case WM_COMMAND:

	MenuCommands (hwnd, lparam, wparam);
 
    break;

	// ----------------------
	//
	// ---------------------
	case WM_SIZE:
		g_nWidth = LOWORD(lparam);  // width of client area 
		g_nHeight = HIWORD(lparam); // height of client area 
		ResizeFunc ();
 	break;
	// ---------------------------------------  
	// Repaint the Window if part or all was hidden
	// ---------------------------------------  
	case WM_PAINT:

	   PAINTSTRUCT         ps;
       HBRUSH              brush;
	   RECT rect;

	    if (GetUpdateRect ( hwnd, NULL, FALSE) == 0) break;
        hdc = BeginPaint (hwnd, &ps);

		rect.top = g_yAdd; 
		rect.bottom = g_yAdd + 8*g_nSqSize; 
        brush = CreateSolidBrush( 0x444444 );
		rect.left = 0; rect.right = g_xAdd;
        FillRect ( hdc, &rect, brush );
		rect.left = g_xAdd+8*g_nSqSize; rect.right = g_nWidth;
        FillRect ( hdc, &rect, brush );
		rect.left = 0; rect.right = g_nWidth;
		rect.top = 0; rect.bottom = g_yAdd;
        FillRect ( hdc, &rect, brush );
		rect.top = g_xAdd + 8*g_nSqSize; rect.bottom = g_nHeight;
		FillRect ( hdc, &rect, brush );
        DeleteObject( brush );

        DrawBoard ( GameBoard, GameBoard, 255);
  
        EndPaint(hwnd, &ps);
          
        break;

	case WM_ERASEBKGND:
	return TRUE;
	// ---------------------------------------  
    // Quitting
	// ---------------------------------------  
    case WM_DESTROY:
		// In case the net was operating
		CloseSocket ();
		Defaults.Save ("Schess.dft");
		WSACleanup ();
		// Now just quit
        PostQuitMessage( 0 );
        break;

    default:
        return( DefWindowProc( hwnd, msg, wparam, lparam ) );
    }
    return( 0L );

} /* WindowProc */


