// Slow Chess 
// Net Communication Code
// by Jonathan Kreuzer
// copyright 2003

#include <windows.h>
#include <time.h>
#include <stdio.h>
#include "scNet.h"
#include "data.h"
#include "gui.h"
#include "search.h"
#include "moves.h"
#include "transcript.h"

// Global Data

int bServer, bToServer = TRUE;
int g_NetMinutes = 5, g_NetSeconds = 5;
extern clock_t g_startTime; // from gui.cpp

SOCKET Server = INVALID_SOCKET;
SOCKET listener = INVALID_SOCKET;


#define SERVER -1
// ---------------------------------------
// CLIENT DATA 
// ---------------------------------------


// ---------------------------------------
struct SClient
{
	SClient ( )
	{
	 acHoldBuffer = new char[HOLDSIZE];
	}
	~SClient ( )
	{
	 delete[] acHoldBuffer;
	}
	void operator = (const SClient &In)
	{
	strcpy (Name, In.Name);
	nColor = In.nColor;
	nTable = In.nTable;
	sock = In.sock;
	memcpy (acHoldBuffer, In.acHoldBuffer, HOLDSIZE);
	}

SOCKET sock;
char Name[40];
int nTable;
int nColor;
char *acHoldBuffer;
};

char acHoldBuffer [ HOLDSIZE ];

int nConnections = 0;
SClient Clients[ MAXCLIENTS + 1 ];
// "Client 0" is the server itself, since whoever runs the server automatically connects as a client


// ---------------------------------------
// TABLE DATA 
// ---------------------------------------

// The server handles the Tables
struct STables
{
	int nPlayer[4]; // WHITE 1, BLACK 2, NONE 3
	int bDraw  [4];
	int bCont  [4];
	int bNew   [4];
	int bReady [4];
	int GameResult;
	SBoard GameBoard;
	int nBlackTime, nWhiteTime;
	int Minutes, Seconds;
	int nPeople;
	STranscript *Transcript;
	clock_t TimeLastMove;
	
	void Init ( )
		{
		Minutes = 5;
		Seconds = 5;
		nPeople = 0;
		Reset();
		NewGame ( );
		nPlayer [ WHITE ] = 0; nPlayer [ BLACK ] = 0;		
		}

	STables ()
		{
		Transcript = (STranscript *) calloc (sizeof (STranscript), 1);
		Init();
		}

	~STables ()
		{
		free (Transcript);
		}

	void Reset ( )
		{
		bDraw[WHITE] = FALSE; bDraw[BLACK] = FALSE;
		bCont[WHITE] = FALSE; bCont[BLACK] = FALSE;
		bNew[WHITE] = FALSE;  bNew[BLACK] = FALSE;
		}
	void NewGame ( )
		{
		Reset ();
		ClearTranscript ( Transcript );
		GameResult = NONE;
		nBlackTime = 0;
		nWhiteTime = 0;
		Setup_Board (GameBoard, 0);
		}
};

STables Tables [ MAXTABLES ];


// ---------------------------------------
// A client has disconnected
// ---------------------------------------
int RemoveClient ( int nClient )
{
if (nConnections == 0) return 2;

nConnections--;
for (int i = nClient; i < nConnections; i++)
	{
	Clients [ i ] = Clients [ i+1 ];
	WSAAsyncSelect (Clients [ i ].sock , mainhwnd, INWM_SPEC_READDATA + i, FD_READ | FD_CLOSE);
	}

for (i = 0; i < MAXTABLES; i++)
	{
	for (int k = 0; k < 4; k++)
		if (Tables[i].nPlayer[k] >= nClient) Tables[i].nPlayer[k]--;
	}

return 1;
}

//------------------------------------------
// CHAT FUNCTION
//------------------------------------------
#define MAXMESSAGES 20

struct SChat 
{
	SChat ( )
	{
	m_numMessages = 0;
	}
// data
int m_numMessages;
char m_sMessages[MAXMESSAGES + 2][300];
// functions
int AddChatString ( char *String );
int Update (  );
void Clear ( )
	{
	m_numMessages = 0;
	}
};


SChat scChat; 

// ----------------------
// Add a String to the Chat Buffer
// -------------------
int SChat::AddChatString ( char *String )
{
	for (int i = m_numMessages; i > 0; i--)
		{
		strcpy (m_sMessages [i], m_sMessages [i-1]);
		}

	sprintf (m_sMessages[0], "%s\15\12", String );

	m_numMessages++;
	if (m_numMessages > MAXMESSAGES) m_numMessages = MAXMESSAGES;

	Update ();
	return TRUE;
}

// -----------------------
// Display Text in the Chat Window
// ----------------------
int SChat::Update (  )
{
    char *pcChatBuffer = new char [300 * (MAXMESSAGES+2)];
	int j = 0;
	int i = m_numMessages -1;

	for (; i >= 0; i--)	j+= sprintf (pcChatBuffer + j, "%s", m_sMessages [i] );

	SetDlgItemText (ChatWnd, 106, pcChatBuffer );
	int nLines = SendMessage ( GetDlgItem (ChatWnd, 106), EM_GETLINECOUNT, 0, 0);
	SendMessage ( GetDlgItem (ChatWnd, 106), EM_LINESCROLL, 0, nLines - 6 );

	delete[] pcChatBuffer;
	return TRUE;
}

//
// 
void Swap (int &x1, int &x2)
{
	int temp = x1;
	x1 = x2;
	x2 = temp;
}

// ---------------------------------------
//
// ---------------------------------------
//
int DisplayMyIP( )
{
    char ac[80];
    if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) return 1;
    
    struct hostent *phe = gethostbyname(ac);
    if (phe == NULL) return 1;
    
	char buffer [2000];
	strcpy (buffer, "IP(s):\n");

    for (int i = 0; phe->h_addr_list[i] != 0; ++i) {
        struct in_addr addr;
        memcpy(&addr, phe->h_addr_list[i], sizeof(struct in_addr));
        strcat (buffer, inet_ntoa(addr) );
		strcat (buffer, "\n");
    }

	if (strstr (buffer, "192.") )
		{
		strcat (buffer, "\nNote: if you're on a LAN, this IP given won't work for external connections.");
		}
	DisplayText (buffer);
   
    return 0;
}

// ---------------------------------------
//
// SOCKET FUNCTIONS
//
// ---------------------------------------
// Initialize the Winsocks
// ---------------------------------------
void InitializeSockets ( )
    {
	static int bInit = FALSE;

	if (bInit == TRUE) return;

    WORD wVersionRequested; 
    WSADATA wsaData; 
    int err; 
    wVersionRequested = MAKEWORD(1, 1);
    	
    err = WSAStartup(wVersionRequested, &wsaData); 
 
        if (err != 0)
            { 
            // we couldn't find a useable winsock.dll.
            return;
            }

        if ( LOBYTE( wsaData.wVersion ) != 1 || 
                HIBYTE( wsaData.wVersion ) != 1 ) { 
            // we couldn't find a useable winsock.dll.
            WSACleanup(); 
            return; 
        } 

	bInit = TRUE;
    }

// ---------------------------------------
// Close an Open Socket
// ---------------------------------------
int CloseSingleSocket ( SOCKET &sock)
{
  if (sock == INVALID_SOCKET) 
	{
	  DisplayText ( "Not Connected" );
	  return 2;
	}

  if (shutdown(sock, 1) == SOCKET_ERROR) 
		{DisplayText ( "Error on Socket shutdown" );
         return 2;
        }

	// Turn off asyncronous messages for this socket
	WSAAsyncSelect(sock, mainhwnd, 0, 0);
    // Receive any extra data still sitting on the socket.

    if (closesocket(sock) == SOCKET_ERROR) 
		{ 
		 DisplayText ( "Error Closing Socket" );
	     return 2;
		}
	
	sock = INVALID_SOCKET;
    return 1;
}

// ---------------------------------------
// Disconnection
// ---------------------------------------
int CloseSocket( )
{   
  // We are connected to a server
  if (bServer == FALSE || g_bICC == TRUE)
	{
	 return CloseSingleSocket ( Server );
	}

  // If this is the server, close all the client's connections
  if (bServer == TRUE)
	{
	 CloseListenSocket ( );
	 for (int i = 1; i < nConnections; i++)
		{
		 CloseSingleSocket ( Clients [ i ].sock );
		}
	 nConnections = 0;
	}
return 1;
}

// ---------------------------------------
// Open a Socket to IP address, Port
// ---------------------------------------
int OpenSocket ( char ip[], unsigned short usPort )
        {
		if (bServer == TRUE) CloseListenSocket ( );
		bServer = FALSE;

	    Server = socket(AF_INET, SOCK_STREAM, 0);
		if (Server == INVALID_SOCKET) {
			  return FALSE;
			}

		sockaddr_in sinRemote;
		sinRemote.sin_addr.s_addr = inet_addr (ip); // Is it an IP of #.#.#.# format?
		if (sinRemote.sin_addr.s_addr == INADDR_NONE)  // If not, we must do a DNS lookup
			{
			HOSTENT *pHostEnt = gethostbyname ( ip );

			if (pHostEnt == NULL)
				{
				DisplayText ("Address Lookup Unsuccessful" );
				return 3;
				}
			sinRemote.sin_addr.s_addr = *((unsigned long*)pHostEnt->h_addr_list[0]);
			}

         sinRemote.sin_family = AF_INET;
         sinRemote.sin_port = ((usPort >> 8) & 255) + ( usPort & 255 ) * 256;

        if (connect(Server, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) ==
                SOCKET_ERROR) 
				{
				MessageBox ( mainhwnd, "Error on connection", "Error", MB_OK);
                Server = INVALID_SOCKET;
                return 2;
                }

          else {
			  	 scChat.Clear ();
			  	 DisplayText ( "Connection Successful" );
				 WSAAsyncSelect (Server, mainhwnd , INWM_READDATA, FD_READ | FD_CLOSE);
               }

        return 1;    
        }

// ---------------------------------------
// Choose which window to send data recieved by the socket to
// ---------------------------------------
void NetMessageWindow( HWND hwnd, unsigned long nMessage )
    {
    WSAAsyncSelect (Server, hwnd , nMessage, FD_READ | FD_CLOSE);
    }


// =================================================================
//  Server Part
// =================================================================

// ---------------------------------------
// Setup Listener on uwPort to check for connection attempts
// ---------------------------------------
int WaitForConnect ( unsigned short uwPort)
{
	bServer = TRUE;

	sockaddr_in sinRemote;
	int sinsize =  sizeof(sockaddr_in);

	CloseListenSocket ( );

    listener = socket(AF_INET, SOCK_STREAM, 0);

	if (listener == INVALID_SOCKET) {DisplayText ("Error on Listener Setup.");
                                 return 1; }

	sinRemote.sin_family = AF_INET;
	sinRemote.sin_addr.s_addr = 0;
	sinRemote.sin_port = ((uwPort >> 8) & 255) + ( uwPort & 255 ) * 256; // Big Endian/Little Endian mismatch, so convert
         
	if ( bind(listener, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == SOCKET_ERROR) 
		{
		DisplayText ("ERROR:\n(on Server Startup)\nUndefined or this computer is already running a server.");
		return 1;
		}
   
	DisplayText ( "Waiting for connections...." );

	listen (listener, 4);

	WSAAsyncSelect (listener, mainhwnd, INWM_CONNECTION, FD_ACCEPT);

	// Reset all the tables
	for (int i = 0; i < MAXTABLES; i++) Tables [i].Init();

	// Client 0 is always the server
	nConnections = 0;
	strcpy (Clients [ nConnections ].Name, sUserName);
	Clients [ nConnections ].sock = INVALID_SOCKET;
	nConnections++;
	scChat.Clear ();

return 2;
}

// ----------------
// The listener hears someone trying to connect
// ----------------
int AcceptConnection(  )
    {
	if (nConnections >= MAXCLIENTS )
		{
		DisplayText ( "Server Full...\n(Connection Attempted)" );
		return 0;
		}

    if ( (Clients [ nConnections ].sock = accept(listener, NULL, NULL)) == INVALID_SOCKET) 
		{
		DisplayText ( "Failed Connection Attempt..." );
		return 0;  
		}

	char sTemp[100];
	sprintf (sTemp, "Client %d Connected", nConnections );
	DisplayText ( sTemp );

	// Okay, Connection is going through
	WSAAsyncSelect (Clients [ nConnections ].sock , mainhwnd, INWM_SPEC_READDATA + nConnections, FD_READ | FD_CLOSE);
	sprintf (Clients [ nConnections ].Name, "Guest%d", nConnections);

	Clients [ nConnections ].nTable = 0;
	Clients [ nConnections ].nColor = NONE;

	nConnections++;

	return 1;
    }

// -------------------------------------------------
// Stop the Server from Listening
// -------------------------------------------------
int CloseListenSocket ( )
{
	if (listener != INVALID_SOCKET) 
		{
		if (closesocket (listener) == 0)
			{
			listener = INVALID_SOCKET;
			return 1;
			}	
		}
	return 2;
}


// =================================================================
//  Receive messages
//  Since TCP/IP is a stream protocol, we impose a packet system on it
//
//  Packets are prefixed by length
//  If a full packet is available, put it into pOutBuffer
//  0 = no full packet
//  1 = exactly 1 packet
//  2 = more than 1 packet
// =================================================================
int nExtraBytes = 0;
#define HEADER_SIZE 2
char *pHoldBuffer;

int ReceiveDataMessage ( char *pOutBuffer, int nClient, int bRead )
{
   int nReadBytes = 0;
   int nPacketSize = 0;
   int retVal;

   if (bServer == TRUE) pHoldBuffer = Clients[ nClient ].acHoldBuffer;
				   else pHoldBuffer = acHoldBuffer;

	// Copy over any left over bytes
   if (nExtraBytes > 0)
	   memcpy(pOutBuffer, pHoldBuffer, nExtraBytes);   

   // If we were asked to read, read in from the socket
   if (bRead == TRUE)
		{
		if (bServer == FALSE) nReadBytes = recv(Server, pOutBuffer + nExtraBytes, HOLDSIZE - 2, 0);
		 		    else nReadBytes = recv(Clients[ nClient ].sock, pOutBuffer + nExtraBytes, HOLDSIZE - 2, 0);

		if (nReadBytes == SOCKET_ERROR || nReadBytes == 0) return -1;
		}
   else nReadBytes = 0;

   // only 1 byte? handle it just in case.
   if (nReadBytes + nExtraBytes < HEADER_SIZE) 
		{
	     pHoldBuffer [ 0 ] = pOutBuffer [ 0 ];
		 nExtraBytes = 1;
	     return 0; 
		}

   int nPacketLen = (pOutBuffer[0] + 128) + (pOutBuffer[1]+128) * 256;

   // less than a full packet?
   if ( (nReadBytes + nExtraBytes) < nPacketLen ) 
		{
		nExtraBytes = nReadBytes + nExtraBytes;
		memcpy(pHoldBuffer, pOutBuffer, nExtraBytes);   
		return 0; 
		}

   nExtraBytes = nReadBytes + nExtraBytes - nPacketLen;

	// more than a full packet?
   if (nExtraBytes !=0 ) 
		{
		memcpy(pHoldBuffer, pOutBuffer + nPacketLen, nExtraBytes);                
		retVal = 2;
		}
   else retVal = 1;

   pOutBuffer[nPacketLen + 1] = NULL;
   return retVal;   
}


// =================================================================
//  Send messages
// =================================================================
// -----------------------------------------------------
void scNetMessage ( int nTable, int nClient, char cType, char cInfo)
{
  char String[6];
  String [0] = 127; String [1] = 127;
  String [2] = cType; 
  String [3] = cInfo;
  String [4] = NULL;
  
  int temp = 0;
  while ( SendData ( String, 5, nTable, nClient ) == 3  && temp < 40)
	{
	  temp ++;
	  Sleep ( 15 );
	}
  if (temp == 40) { MessageBeep( MB_OK ); DisplayText (" Cannot Send Message "); }

}

void scNetMessageS ( int nTable, int nClient, char cType, char *sInfo)
{
  char *String = new char [ strlen(sInfo) + 6 ];
  String [0] = 127;
  String [1] = 127; // Pad For Header
  String [2] = cType; 
  strcpy (&String[3], sInfo);
  int nLen = strlen (sInfo);

  int temp = 0;
  while ( SendData ( String, nLen + 4, nTable, nClient ) == 3 && temp < 40)
	{
	  temp ++;
	  Sleep ( 15 );
	}
  if (temp == 40) { MessageBeep( MB_OK ); DisplayText (" Cannot Send Message "); }

  delete String;
}

void scNetMessageS ( char cType, char *sInfo)
{
	scNetMessageS (SC_ALL, SC_ALL, cType, sInfo);
}

void scNetMessage ( char cType, char cInfo)
{
	scNetMessage (  SC_ALL, SC_ALL, cType,  cInfo );
}

// -----------------------------------------------------
// Send Data over sockets (or directly if you're the server & client)
// ---------------------------------------------
int SendData ( char *pData, int nLen, int nTable, int nClient )
{
char header[2] = { nLen % 256 - 128, nLen / 256 -128};
int nBytes;

pData[0] = header[0]; pData [1] = header [1]; // Add Header.

if (bToServer == TRUE) 
	{
	if (bServer == TRUE) 
		{
		ReceiveDataServer (pData[2], &pData[3], 0);
		return TRUE;
		}
	nBytes = send (Server, pData, nLen, 0);
	}

if (bToServer == FALSE) 
	{
	if (nClient != SC_ALL)
		{
		if (nClient == 0)  // Data to self
			{
			ReceiveDataClient (pData[2], &pData[3]);
			return TRUE;
			}
		else /*if (nClient > 0 && nClient < nConnections )*/{
			nBytes = send (Clients[ nClient ].sock, pData, nLen, 0);
			}
		}
	else
		{
		for (int i = 0; i < nConnections; i++)
			{
			if (Clients[i].nTable == nTable || nTable == SC_ALL)
				{
				if (i == 0) ReceiveDataClient (pData[2], &pData[3]);
				else{
					nBytes = send (Clients[i].sock, pData, nLen, 0);
					}
				}
			}
		}
	}

if (nBytes == SOCKET_ERROR || nBytes == 0)
	{
	int i = WSAGetLastError ( );
	if (i == 10035) return 3;
	char TempBuffer [80];
	sprintf (TempBuffer, "Send Error. Server Down? (%d)", i);
	DisplayText ( TempBuffer );
	return FALSE;
	}

return TRUE;
}

// =================================================================
//  Net Game Messages
// =================================================================

// -----------------------------------------------------
// Send your move to opponent
//
void scNetSendMove (  )
{
	char String[50];
	char *sMove;
	GetLastMoveTranscript ( &sMove );
	// Send Time With Move
	int Time;
	String[0] = g_cNetColor;
	if (g_cNetColor == BLACK)  Time = g_nBlackTime;
	if (g_cNetColor == WHITE)  Time = g_nWhiteTime;

	String[1] = Time % 120 + 1; // No 0's sending as a string
	String[2] = (Time / 120) % 120 + 1;
	String[3] = (Time / 14400) + 1;

	// Send the Move itself
	strcpy (&String[4], sMove );
	scNetMessageS (SC_MOVED, String );
	DisplayText ( sMove );
}

// -----------------------------------------------------
// enters opponents move
//
void scNetOppMove ( char *sMove, SBoard &Board )
{
 if (CurrentBoard ( Board ) == FALSE) ReplayGame (Board, 1000);
 SBoard OldBoard = Board;
 DoMovePGN ( sMove, Board, TRUE, NULL );
 UpdateListTranscript ( TRUE );
 DrawBoard (  Board, OldBoard, 255);
 DisplayText ( sMove );
 UpdateMove( FALSE );
}

// -----------------------------------------------------
// -----------------------------------------------------
void scNetSendTranscript ( int nClient )
{
char *clipText = new char [ 5002 ];
int nTable = Clients [ nClient ].nTable;
CopyTranscript ( Tables[nTable].Transcript, &clipText[2], 5000);
scNetMessageS (SC_ALL, nClient, SC_TRANSCRIPT, clipText );
delete [] clipText;
}

// -----------------------------------------------------
// -----------------------------------------------------
void scNetSendTimeControl ( int nTable, int nClient )
{
char sTemp [ 10 ];
int nWT = Tables [ nTable ].nWhiteTime;
int nBT = Tables [ nTable ].nBlackTime;

sprintf (sTemp, "%c%c%c%c%c%c%c%c", Tables[nTable].Minutes - 128, Tables[nTable].Seconds - 128, 
								 (nWT % 120) + 1, ((nWT/ 120) % 120) + 1, (nWT /14400) +1,
								 (nBT % 120) + 1, ((nBT/ 120) % 120) + 1, (nBT /14400) +1);
scNetMessageS ( nTable, nClient, SC_TIMECONTROL, sTemp );
}

// ----------------
// CLIENT
//
// Interprets the message received from the server
// ----------------
void ReceiveDataClient ( char cMsg, char *String )
{
int time;

switch ( cMsg )
	{
		case SC_COLOR:
			g_cNetColor = String[0];
			break;

		case SC_WHITENAME:
			strcpy (g_sNetWhite, String );
			SetDlgItemText (sideHWND, 114, String);
			break;

		case SC_BLACKNAME:
			strcpy (g_sNetBlack, String );
			SetDlgItemText (sideHWND, 115, String);
			break;

		case SC_TRANSCRIPT: // Just logged on to a table, Received entire transcript

			Setup_Board ( GameBoard, 0);
			ClearTranscript ( NULL );
			GetMovesPGN ( String, TRUE );
			DrawBoard ( GameBoard, GameBoard, 255);
			MoveBox ( GameBoard.GameTime - 2 );
			break;

		case SC_INFO:
			DisplayText ( String );
			return;

		case SC_CHAT:
			scChat.AddChatString ( String ); 
			return;

		case SC_SETUP:
			ReceiveSetupInfo ( String );
			return;

		case SC_STARTBOX:
			g_cNetColor = String[0];
			StartGameBox ( ); 
			break;

		case SC_TIMECONTROL:
			g_NetMinutes = String[0] + 128;
			g_NetSeconds = String[1] + 128;
			g_nWhiteTime = (String[2]-1) + (String[3]-1) * 120 + (String[4]-1) * 120*120;
			g_nBlackTime = (String[5]-1) + (String[6]-1) * 120 + (String[7]-1) * 120*120;
			SetupInfoTime ( g_NetMinutes, g_NetSeconds );
			return;

		case SC_TIMEUP: // Opponent has lost on time

			if (String[0] == BLACK)
				{
				MessageBeep( MB_OK );
				g_GameResult = WHITEWINS;
				}
			if (String[0] == WHITE)
				{
				MessageBeep( MB_OK );
				g_GameResult = BLACKWINS;
				}
			return;

		case SC_MOVED: 

			if (String[0] != g_cNetColor ) // only do opponents moves
				{
				scNetOppMove ( &String[4], GameBoard );
				if (g_cNetColor != NONE) CheckGameOver ( GameBoard, TRUE, g_GameResult);
				time = (String[1]-1) + (String[2]-1) * 120 + (String[3]-1) * 120*120;
				if (time >= 0)
					{
					if (String[0] == BLACK) g_nBlackTime = time;
					if (String[0] == WHITE) g_nWhiteTime = time;
					}
				}
			return;

        case SC_NEW:

			NewGame ( TRUE );
			return;

		case SC_DRAW: 

			MessageBeep( MB_OK );
			DisplayText ( "DRAW: The players have agreed on a Draw" );
			g_GameResult = DRAW;
			return;

		case SC_CONTINUE: 

			MessageBeep( MB_OK );
			DisplayText ( "CONTINUE: The players have agreed to continue" );
			g_GameResult = NONE;
			g_nWhiteTime = 0;
			g_nBlackTime = 0;
			g_startTime = clock ();
			return;

		case SC_RESIGN: 

			if (g_GameResult == NONE)
				{
				MessageBeep( MB_OK );
				if (String[0] == BLACK) g_GameResult = WHITEWINS;
				if (String[0] == WHITE) g_GameResult = BLACKWINS;
				}
			return;

		case SC_DISC:
			MessageBeep( MB_OK );
			DisplayText ("Server Disconnected");
			NetGameOff ( TRUE );
			return;
		}
}


// ----------------
// SERVER
//
// Interprets the Message received from the client
// ----------------
void ReceiveDataServer ( char cMsg, char *String, int nClient )
{
int time;
int nTable, nColor;
char sTemp [ 600 ];
nTable = Clients [ nClient ].nTable;
nColor = Clients [ nClient ].nColor;

bToServer = FALSE;

switch ( cMsg)
	{
	case SC_TABLE:

		nTable = String[0];
		if (nTable >= MAXTABLES || nTable < 0) 
			{
			sprintf (sTemp, "< Table %d does not exist >", nTable );
			scNetMessageS (nTable, nClient, SC_INFO, sTemp);
			break;
			}

		if (nTable != Clients[ nClient ].nTable)
			{sprintf (sTemp, "** %s left table %d **", Clients[ nClient ].Name, Clients[ nClient ].nTable );
			scNetMessageS (Clients[ nClient ].nTable, SC_ALL, SC_CHAT, sTemp);
			}
		Clients [ nClient ].nTable = nTable;
		Clients [ nClient ].nColor = NONE;
		sprintf (sTemp, "** %s joined table %d **", Clients[ nClient ].Name, nTable );
		scNetMessageS (nTable, SC_ALL, SC_CHAT, sTemp);
		scNetMessage ( nTable, nClient, SC_COLOR, NONE);
		scNetSendTimeControl ( nTable, nClient );

		if (Tables [ nTable ].nPeople == 0)
			{
			scNetMessageS ( nTable, nClient, SC_BLACKNAME, "-Empty-");
			scNetMessageS ( nTable, nClient, SC_WHITENAME, "-Empty-");
			}
		if (Tables [ nTable ].nPeople == 1)
			{
			scNetMessageS ( nTable, nClient, SC_BLACKNAME, "-Empty-");
			scNetMessageS ( nTable, nClient, SC_WHITENAME, Clients[ Tables[nTable].nPlayer[WHITE] ].Name);
			}
		if (Tables [ nTable ].nPeople >=2) 
			{
			scNetSendTranscript ( nClient );
			scNetMessageS ( nTable, nClient, SC_WHITENAME, Clients[ Tables[nTable].nPlayer[WHITE] ].Name);
			scNetMessageS ( nTable, nClient, SC_BLACKNAME, Clients[ Tables[nTable].nPlayer[BLACK] ].Name);
			}
		else 
			{
			scNetMessageS ( nTable, nClient, SC_NEW, "");
			}
		break;

	case SC_NAME: 
		strcpy ( Clients [ nClient ].Name, String);
		sprintf (sTemp, "** %s Connected **", Clients[ nClient ].Name);
		scNetMessageS (SC_CHAT, sTemp); 
		break;

	case SC_CHAT:
		// Server is responsible for naming the Chatters.
		sprintf (sTemp, "[%d] %s> %s", Clients[ nClient ].nTable, Clients[ nClient ].Name, String);
		scNetMessageS (SC_CHAT, sTemp); 
		break;

	  case SC_DISCONNECT:
	  case SC_GOSPEC:

		  if ( Clients [ nClient ].nColor != NONE )
			{
	  		scNetMessage ( nTable, SC_ALL, SC_RESIGN, nColor );
			Clients [ Tables [ nTable ].nPlayer [ WHITE ] ].nColor = NONE;
			if (Tables [ nTable ].nPeople > 1) Clients [ Tables [ nTable ].nPlayer [ BLACK ] ].nColor = NONE;
			Tables [ nTable ].nPeople = 0;
			sprintf (sTemp, "%s got up.\nTable open.", Clients[ nClient].Name);
			scNetMessageS ( nTable, SC_ALL, SC_INFO, sTemp );
			Tables [ nTable ].Reset();

			scNetMessageS ( nTable, SC_ALL, SC_BLACKNAME, "-Empty-");
			scNetMessageS ( nTable, SC_ALL, SC_WHITENAME, "-Empty-");
			}

		// Disconnected entirely
		if (cMsg == SC_DISCONNECT)
			{
			if (nClient == 0) // Server Itself
				{
				scNetMessageS ( SC_DISC, "" );
				}
			else
				{
				sprintf (sTemp, "** %s Disconnected **", Clients[ nClient].Name);
				RemoveClient ( nClient );
				scNetMessageS (SC_CHAT, sTemp);
				}
			}
		break;

	case SC_SETUP:
		if (nColor == WHITE) scNetMessageS ( SC_ALL, Tables [ nTable ].nPlayer [ BLACK ], SC_SETUP, String);
		if (nColor == BLACK) scNetMessageS ( SC_ALL, Tables [ nTable ].nPlayer [ WHITE ], SC_SETUP, String);

		if (String[0] == 't')
			{
			Tables[ nTable ].Minutes = String[1] + 128;
			Tables[ nTable ].Seconds = String[2] + 128;
			scNetSendTimeControl ( nTable, SC_ALL );
			}
		if (String[0] == 'c') 
			{
			if (String[1] == Clients[ nClient ].nColor )
				{
				Swap ( Tables[ nTable ].nPlayer[ BLACK ], Tables[ nTable ].nPlayer[ WHITE ] );
				Clients[ Tables[ nTable ].nPlayer[WHITE] ].nColor = WHITE;
				Clients[ Tables[ nTable ].nPlayer[BLACK] ].nColor = BLACK;
				scNetMessageS ( nTable, SC_ALL, SC_WHITENAME, Clients[ Tables[nTable].nPlayer[WHITE] ].Name);
				scNetMessageS ( nTable, SC_ALL, SC_BLACKNAME, Clients[ Tables[nTable].nPlayer[BLACK] ].Name);
				}
			}
		break;

	case SC_TIMEUP: // Client lost on time

		if (nColor == WHITE && Tables [ nTable ].nPlayer [ WHITE ] == nClient)
			{
			Tables [ nTable ].GameResult = BLACKWINS;
			sprintf (sTemp,  "<%s Lost on Time>", Clients[ nClient ].Name );
			scNetMessageS ( nTable, SC_ALL, SC_INFO, sTemp);
			scNetMessage  ( nTable, SC_ALL, SC_TIMEUP, nColor );
			}
		if (nColor == BLACK && Tables [ nTable ].nPlayer [ BLACK ] == nClient)
			{
			Tables [ nTable ].GameResult = WHITEWINS;
			sprintf (sTemp,  "<%s Lost on Time>", Clients[ nClient ].Name );
			scNetMessageS ( nTable, SC_ALL, SC_INFO, sTemp);
			scNetMessage  ( nTable, SC_ALL, SC_TIMEUP, nColor );
			}

		break;

	case SC_MOVED: 

		if (nClient != Tables [ nTable ].nPlayer[WHITE] && nClient != Tables [ nTable ].nPlayer[BLACK]) return;
		scNetMessageS ( nTable, SC_ALL, SC_MOVED, String);
		Tables [ nTable ].Reset ( );
		time = (String[1]-1) + (String[2]-1) * 120 + (String[3]-1)*120*120;
		if (time >= 0)
			{
			if (String[0] == BLACK) Tables [ nTable ].nBlackTime = time;
			if (String[0] == WHITE) Tables [ nTable ].nWhiteTime = time;
			}
		DoMovePGN (  &String[4], Tables [ nTable ].GameBoard, TRUE, Tables [ nTable ].Transcript);
		CheckGameOver ( Tables [ nTable ].GameBoard, FALSE, Tables[ nTable ].GameResult);
		if ( Tables[ nTable ].GameResult == WHITEWINS) scNetMessageS ( nTable, SC_ALL, SC_INFO, "White Wins");
		if ( Tables[ nTable ].GameResult == BLACKWINS) scNetMessageS ( nTable, SC_ALL, SC_INFO, "Black Wins");
		if ( Tables[ nTable ].GameResult == DRAW) scNetMessageS ( nTable, SC_ALL, SC_INFO, "Draw.");

		break;

      case SC_NEW: 

		// Table Doesn't have 2 players yet
		if (Tables [ nTable ].nPeople < 2) 
			{
			char cColor;
			if (Tables [ nTable ].nPeople == 0) cColor = WHITE; else cColor = BLACK;
			// Can't play yourself
			if (Tables [ nTable ].nPeople == 1 && Tables [ nTable ].nPlayer [ WHITE ] == nClient) break;

			Clients [ nClient ].nColor = cColor;
			Tables [ nTable ].nPlayer [ cColor ] = nClient;
			sprintf (sTemp, "<%s is Ready to Play>", Clients [ nClient ].Name );
			scNetMessageS (nTable, SC_ALL, SC_INFO, sTemp);

			Tables [ nTable ].nPeople++;
			if (Tables [ nTable ].nPeople == 1 ) 
				{scNetMessageS ( nTable, SC_ALL, SC_WHITENAME, Clients[ Tables[nTable].nPlayer[WHITE] ].Name);
				}
			if (Tables [ nTable ].nPeople == 2) 
				{
				Tables [ nTable ].NewGame (); 
				scNetMessageS ( nTable, SC_ALL, SC_NEW, "" );
				scNetMessage ( SC_ALL, Tables [ nTable ].nPlayer [ WHITE ], SC_STARTBOX, WHITE);
				scNetMessage ( SC_ALL, Tables [ nTable ].nPlayer [ BLACK ], SC_STARTBOX, BLACK);
				scNetMessageS ( nTable, SC_ALL, SC_WHITENAME, Clients[ Tables[nTable].nPlayer[WHITE] ].Name);
				scNetMessageS ( nTable, SC_ALL, SC_BLACKNAME, Clients[ Tables[nTable].nPlayer[BLACK] ].Name);
				}
			
			break;
			}

		// Table has 2 players
		Tables [ nTable ].bNew [ nColor ] = TRUE;
		if ( Tables [ nTable ].bNew [ WHITE ] == TRUE && Tables [ nTable ].bNew [ BLACK ] == TRUE)
			{
			Tables [ nTable ].NewGame (); 
			scNetMessageS ( nTable, SC_ALL, SC_NEW, "" );
			scNetMessage ( SC_ALL, Tables [ nTable ].nPlayer [ WHITE ], SC_STARTBOX, WHITE);
			scNetMessage ( SC_ALL, Tables [ nTable ].nPlayer [ BLACK ], SC_STARTBOX, BLACK);
			}
		else
			{
			if ( Clients [ nClient ].nColor != NONE )
				{
				sprintf ( sTemp, "<%s requests New Game!>", Clients [ nClient ].Name );
				scNetMessageS (nTable, SC_ALL, SC_INFO, sTemp);
				}
			}

		break;


	case SC_DRAW: 

		if (nColor == NONE) break;
		Tables [ nTable ].bDraw [ nColor ] = TRUE;
		if ( Tables [ nTable ].bDraw [ WHITE ] == TRUE && Tables [ nTable ].bDraw [ BLACK ] == TRUE)
			{
			Tables [ nTable ].Reset();
			scNetMessageS ( nTable, SC_ALL, SC_DRAW, "" );
			Tables [ nTable ].GameResult = DRAW;
			}
		else
			{
			sprintf ( sTemp, "<%s requests a Draw!>", Clients [ nClient ].Name );
			scNetMessageS ( nTable, SC_ALL, SC_INFO, sTemp);
			}

		break;

	case SC_CONTINUE: 

		if (nColor == NONE) break;
		Tables [ nTable ].bCont [ nColor ] = TRUE;
		if ( Tables [ nTable ].bCont [ WHITE ] == TRUE && Tables [ nTable ].bCont [ BLACK ] == TRUE)
			{
			Tables [ nTable ].Reset();
			Tables [ nTable ].nWhiteTime = 0;
			Tables [ nTable ].nBlackTime = 0;
			Tables [ nTable ].GameResult = NONE;
			scNetMessageS ( nTable, SC_ALL, SC_CONTINUE, "" );
			}
		else
			{
			sprintf ( sTemp, "<%s requests to Continue Game!>", Clients [ nClient ].Name );
			scNetMessageS (nTable, SC_ALL, SC_INFO, sTemp);
			}
			
		break;

	case SC_RESIGN: 

		if (nColor == NONE) break;
		if (nColor == BLACK) Tables [ nTable ].GameResult = WHITEWINS;
		if (nColor == WHITE) Tables [ nTable ].GameResult = BLACKWINS;

		Tables [ nTable ].Reset();
		scNetMessage ( nTable, SC_ALL, SC_RESIGN, nColor );
		sprintf ( sTemp, "<%s Resigns!>", Clients [ nClient ].Name );
		scNetMessageS (nTable, SC_ALL, SC_INFO, sTemp);

		break;
		}

bToServer = TRUE;
}


// ------------------------------------
//
// NET GAME STARTUP DIALOG Send/receive
//
// ------------------------------------

// For Changing Options
void SendSetupInfo (char type, char info)
        {
		char buffer[5] = {type, info, 0};
		scNetMessageS  (SC_SETUP, buffer );
       }

void SendSetupInfo (char type, char info, char info2)
        {
		char buffer[5] = {type, info, info2, 0};
		scNetMessageS  (SC_SETUP, buffer );
       }

//
// Interpret Messages Sent
// 
// This is a companion to the NetStartDlgProc
void SetupInfoTime (	int Minutes, int Seconds )
{
    HWND hwnd = NetStartWnd;
	if (hwnd != NULL) 
				{
				SetDlgItemInt( hwnd, 135, g_NetMinutes, FALSE);
				SetDlgItemInt( hwnd, 137, g_NetSeconds, FALSE);
				SetDlgItemInt ( hwnd, 123, g_NetMinutes, FALSE);
				SetDlgItemInt ( hwnd, 132, g_NetSeconds, FALSE);
				}
}

int ReceiveSetupInfo (char acReadBuffer[] )
    {
	char type, info, info2;

	 type = acReadBuffer[0];
	 info  = acReadBuffer[1];
	 info2 = acReadBuffer[2];
	
	// The other player has made a change, and we've just received info about what it was, so we update accordingly
     
	  HWND hwnd = NetStartWnd;

      if (type == 'c') 
		{
		  g_cNetColor = info;
		  if (hwnd == NULL) return 1;
          if (g_cNetColor == WHITE) CheckRadioButton (hwnd, 104, 105, 104 );
							   else CheckRadioButton (hwnd, 104, 105, 105 );
        }
                        
      if (type == 'z') 
				{
				if (hwnd != NULL) CheckDlgButton (hwnd, 106, info );
                }
      
      if (type == 'r') 
				{
				OppReady = info;
                if (OppReady == 1) 
					{
					 if (hwnd != NULL) SetDlgItemText (hwnd, 125, "READY" );
                     if (Ready == 1)
                        {Ready = 0; OppReady = 0;
                         EndGameBox ( );
						 EndDialog( hwnd, TRUE );
                         return( TRUE );
                        }
                                                 
		             }
			    else if (hwnd != NULL) SetDlgItemText (hwnd, 125, "Not Ready" );
               }
                   
      if (type == 'q')
            {
			Ready = 0; OppReady = 0;
            EndDialog( hwnd, TRUE );
            return( TRUE );                                           
            }
               
    return 0;                           
    } 
      
