// Slow Chess
// by Jonathan Kreuzer
// copyright 2003

//
// This source may be used to satisfy your curiosity, as a learning tool, or it
// may be modified and redistributed as long as you give credit.
// (And I'll add this now to cover the unlikely event this program gets much better...
//  you can't charge money for something based on this code. i.e. You're not allowed
//  to use this code, make some changes, and sell it.)
//
// That said, this isn't the strongest chess program, nor was the source designed
// to be easy to read of modify, Nowadays though, if/when I edit it I try to keep those
// things in mind, so it's getting better.

//==============================================================================
// Possible Projects/improvements/bug fixes with this code:
//
// If I feel in the mood to do some programming, I may do any one of these myself.
//
// 1. Add More Chess Knowledge (esp. in Endgame)
// I don't know much about chess, so this program develops its pieces as well as I do.
// I also never added much specific Endgame knowledge, so it's pretty bad at endgame strategy
//
// 2. Better Quiescence Search & Extensions
//
// 3. Optimize the Alpha Beta pruning code, Better move ordering? More Selective?
//
// 4. Start speed optimizing again, probably starting with checks & evasions.
//
//==============================================================================

#include <windows.h>
#include <stdio.h>
#include <time.h>

#include "data.h"
#include "gui.h"
#include "moves.h"
#include "transcript.h"
#include "search.h"
#include "evaluate.h"
#include "scNet.h"

// ---------------------------------
// Globals
//
int SearchDepth = 1, Bf, Bnodes, thinking = 0, bAcceptDraw = FALSE, g_bAnalyze = FALSE;
unsigned char PlayerCOLOR = WHITE;
clock_t starttime, endtime, MoveTm;
float Thinktime = 10, fPanicTime = 1;
unsigned int nodes, nodes2;
int g_MaxDepth, stopthinking;
unsigned char upgrade = 0;
int DRAWSCORE = 0;

#define NOVALUE -19999

unsigned int KillerMoves [MAXDEPTH+1], KillerMoves2 [MAXDEPTH+1];

unsigned long HistoryW [ 128*128 ];
unsigned long HistoryB [ 128*128 ];

Movestruct *MoveStack = (Movestruct *) calloc (MAXDEPTH+1, sizeof(Movestruct));
SBoard *BoardStack  = (SBoard *) calloc (MAXDEPTH+1, sizeof(SBoard));
SBoard SearchBoard;
// Search Info Tracking 

int CurrentBest, CurrentMove = 0, CurrentMoveNum = 0, CurrentEval = 0;
int PV [MAXDEPTH+1][MAXDEPTH+1];
float g_fMoveTm = 0.0f;
int g_nFail = 0, g_SearchColor = WHITE;
unsigned char g_ucAge = 0;

// ==========================
//
//  HASHING 
//
// ==========================
//
// FixMe: Enpassant and castling flags are added in later, since this made an update easier.
// There's nothing wrong with this, except the added chance of confusion.
//
// DATA
struct HashEntry
    {
	 unsigned int checksum;
     short eval;
     unsigned short bestmove;
     unsigned char age;
     signed char mate, depth, failtype;
    };

struct HashPair
	{
	HashEntry Near, Far;
	};

unsigned int g_nHashSize = 0;
unsigned int HashFunction [128*16];
unsigned int HashFunction2 [128*16];
HashPair *TranspositionTable = NULL;

// --------------------------------------------------
// Allocate memory, sizeof (hashentry) * 2 ^ nBits
// --------------------------------------------------
void SetHashSizeBits ( int nBits )
{
 nBits--; // (Since it's a 2 level table, divide by 2)
 g_nHashSize = (2 << (nBits-1) ) - 1;
 if (TranspositionTable != NULL ) free ( TranspositionTable );
 TranspositionTable = (HashPair*) malloc ( (g_nHashSize + 5) * sizeof (HashPair));
 if (TranspositionTable == NULL )
	{
	 MessageBox (mainhwnd, "Error Allocating Transposition Table", "Error", MB_OK);
	 if (nBits > 15) SetHashSizeBits ( 15 );
	 return;
	}
 Clear_Hash ( );
 char buffer[100];
 sprintf (buffer, "# of TT Entries: %d", g_nHashSize * 2);
 DisplayText (buffer);
}

// --------------------------------------------------
// Clear Hash Table by zeroing it out.
// --------------------------------------------------
void Clear_Hash ( void )
{
ZeroMemory (TranspositionTable, sizeof (HashPair) * g_nHashSize);
Clear_PawnHash();
DisplayText ("Hash Table Cleared.");
}

// --------------------------------------------------
//  Compute the Random Numbers of each piece on each square for the Hash Keys & Checksums
// --------------------------------------------------
void Create_HashFunction ( )
    {
	srand (33731);
	int i, x;

	for (i = 0; i <128; i++)
		{
		// xor together some 16-bit rands
		BPHashSq[i] = rand() ^ (rand()<<15) ^ (rand()<<25);
		WPHashSq[i] = rand() ^ (rand()<<15) ^ (rand()<<25);

        for (x = 0; x < 16; x++)
            {
			HashFunction  [i * 16 + x] = rand() ^ (rand()<<15) ^ (rand()<<25);
			HashFunction2 [i * 16 + x] = rand() ^ (rand()<<15) ^ (rand()<<25);
            }
		}

	for (x = 0; x < 5; x++) // Castling flags lost castles to the same
		{
		HashFunction [ x + (4*8) ] = HashFunction [ x+ (3*8) ];
		HashFunction [ x*8+4 ] = HashFunction [ x*8+3 ];
		}

	srand (clock()); 
    }       

// --------------------------------------------------
//  Compute the Hash Value for an entire Board
// --------------------------------------------------
unsigned int Hash_Board (SBoard &Board)
        {
		unsigned int sum = 0;
        int i;

		for (i = 20; i <= 100; i++)
			sum ^= HashFunction [ i * 16 + Board.Sq [i] ];

        if (Board.SideToMove == BLACK) sum ^= HashFunction [8 ];

        return sum;
        }

unsigned int  Hash_Board2 (SBoard &Board)
        {
		unsigned int sum = 0;
        int i;

		for (i = 20; i <= 100; i++)
			sum ^= HashFunction2 [ i * 16 + Board.Sq [i] ];

        if (Board.SideToMove == BLACK) sum ^= HashFunction2 [8 ];

        return sum;
        }

// -------------------------
//  Incrementally update the Hash Value for the board from the last move
// -------------------------
long inline HashUpdate (int u1, int u2,  SBoard &Board)
    {
	 int Sq, oldpiece, newpiece;
        
		 // destination square
         Sq = (u1 & 255);
		 newpiece = Board.Sq [ Sq ];
		 Sq *= 16;
		 oldpiece = (u2 & 255); 
         Board.HashKey  ^= HashFunction  [ Sq + newpiece ] ^ HashFunction  [ Sq + oldpiece ];
         Board.HashCheck ^= HashFunction2 [ Sq + newpiece ] ^ HashFunction2 [ Sq + oldpiece ];
 
		 // source square
         Sq = ((u1>>8)&255);
		 newpiece = Board.Sq [ Sq ];
		 Sq *= 16;
		 oldpiece = ((u2>>8)&255); 
         Board.HashKey  ^= HashFunction  [ Sq + newpiece ] ^ HashFunction  [ Sq + oldpiece ];
         Board.HashCheck ^= HashFunction2 [ Sq + newpiece ] ^ HashFunction2 [ Sq + oldpiece ];

		 // extra castling squares?
        if ( (u1>>16) > 0 )
                { Sq = ((u1>>16)&255); newpiece = Board.Sq [ Sq ]; oldpiece  = ((u2>>16)&255);
				  Sq *= 16;
                  Board.HashKey  ^= HashFunction  [ Sq + newpiece ] ^ HashFunction  [ Sq + oldpiece ];
                  Board.HashCheck ^= HashFunction2 [ Sq + newpiece ] ^ HashFunction2 [ Sq + oldpiece ];
                }
         else if ( (u1>>24) > 0 )
                {Sq = ((u1>>24)&255); newpiece = Board.Sq [ Sq ]; oldpiece = ((u2>>24)&255);
				 Sq *= 16;
                 Board.HashKey ^= HashFunction  [ Sq + newpiece ] ^ HashFunction  [ Sq + oldpiece ];
                 Board.HashCheck ^= HashFunction2 [ Sq + newpiece ] ^ HashFunction2 [ Sq + oldpiece ];
                }
        
		// Switch Side To Move
		Board.HashKey ^= HashFunction   [8];
		Board.HashCheck ^= HashFunction2 [8];

		return Board.HashKey;
    }

//
// For a Null Move
//
// Updates the Board to-play color, and the hashvalues
//
void BoardSwitchColor ( )
{
    if (SearchBoard.SideToMove == WHITE) SearchBoard.SideToMove = BLACK; else SearchBoard.SideToMove = WHITE;
 
    SearchBoard.HashKey   ^= HashFunction  [8];
    SearchBoard.HashCheck ^= HashFunction2 [8];
}

// --------------------------------------------------
//  Look up an entry and retrieve its info for the search
// --------------------------------------------------
//
// Return a Value if possible
//
void inline TT_CutOff ( HashEntry &TEntry, short &value, int &nextbest, short &alpha, short &beta, const int &ahead, const int &depth, const int &bPV)
{
	if (TEntry.depth >= depth && ahead > 1) // Don't grab values at root
		{
		// CheckMate value returned? then adjust the eval since the score depends on ply
		if (TEntry.mate !=0)
			{
			if (TEntry.eval < 0 ) TEntry.eval = -9999 + TEntry.mate + ahead;
 			if (TEntry.eval > 0 ) TEntry.eval =  9999 - TEntry.mate - ahead;
			}
		if (TEntry.failtype == 0) value = TEntry.eval;
        else if (TEntry.failtype == 2 && TEntry.eval >= beta) value = TEntry.eval;
        else if (TEntry.failtype == 1 && TEntry.eval <= alpha) value = TEntry.eval;
		}
}

// Return a Best Move if it looks good. 
void inline TT_BestMove ( HashEntry &TEntry, short &value, int &nextbest, short &alpha, short &beta, const int &ahead, const int &depth, const int &bPV)
{
	// Is the best move from a search to a decent depth
	if ( (TEntry.depth >= (depth-2) || TEntry.mate!=0) ) 
		{
		// Don't take best move from a "fail high" if it was for a window with a significantly lesser bound (so it uses internal iterative deepening instead)
		// This takes care of some degenerate cases where there's a mate in 1 (or more) but a non-mating best move is searched deeply first instead.
		if (TEntry.failtype == 0 || (alpha != (beta-1) && bPV == FALSE)
			|| ( TEntry.failtype == 2 && (SearchBoard.SideToMove == BLACK || (TEntry.eval+140) >= beta))  
			|| ( TEntry.failtype == 1 && (SearchBoard.SideToMove == WHITE || (TEntry.eval-140) <= alpha)) )
			nextbest = TEntry.bestmove;
		}
}


void inline TT_Lookup ( const unsigned int &nHashCheck, HashPair &TEntry, short &value, int &nextbest, short &alpha, short &beta, const int &ahead, const int &depth, const int &bPV)
{
	// hashing off
	if (bHashing == FALSE) return; 

	if (TEntry.Near.checksum == nHashCheck) 
		{
		TT_CutOff ( TEntry.Near, value, nextbest, alpha, beta, ahead, depth, bPV);
		if (value!= NOVALUE ) return;
		if (TEntry.Far.checksum  == nHashCheck) TT_CutOff ( TEntry.Far, value, nextbest, alpha, beta, ahead, depth, bPV);
		if (value!= NOVALUE ) return; // Check if there's a cutoff in the far table for this position
		TT_BestMove ( TEntry.Near, value, nextbest, alpha, beta, ahead, depth, bPV);
		return;
		}
	if (TEntry.Far.checksum  == nHashCheck) 
		{
		TT_CutOff ( TEntry.Far, value, nextbest, alpha, beta, ahead, depth, bPV);
		TT_BestMove ( TEntry.Far, value, nextbest, alpha, beta, ahead, depth, bPV);
		}
}

// --------------------------------------------------
//  Store info from the search in a Hash Entry
// --------------------------------------------------
void inline TT_Write ( HashEntry &TEntry, const unsigned int &nHashCheck, const short &value, const int &nextbest, const short &alpha, const short &beta, const int &ahead, const int &depth)
{
	TEntry.checksum = nHashCheck;
	/*if (value == DRAWSCORE) TEntry.eval = NOVALUE; else*/ TEntry.eval = value;
	TEntry.bestmove = nextbest;
	TEntry.age = g_ucAge;
    TEntry.depth = depth;

	if (value <= alpha) TEntry.failtype = 1;
	else if (value >= beta)  TEntry.failtype = 2;
	else TEntry.failtype = 0;

	if ( abs(value) > 9900)
		{
		TEntry.mate = 9999 - abs(value) - ahead;
		}
	else TEntry.mate = 0;
}

void inline TT_Store ( const unsigned int &nHashCheck, HashPair &TEntry, const short &value, const int &nextbest, const short &alpha, const short &beta, const int &ahead, const int &depth)
{
	if (bHashing == FALSE || (SearchBoard.GameTime - SearchBoard.LastReversible) >= 100 /*|| (depth < 1)*/) return; 

	if (depth >= TEntry.Near.depth || TEntry.Near.age != g_ucAge)
		{
		TT_Write (TEntry.Near, nHashCheck, value, nextbest, alpha, beta, ahead, depth);
		}
	else TT_Write (TEntry.Far, nHashCheck, value, nextbest, alpha, beta, ahead, depth);
                                                            
}

// ==========================
//
//  Various Functions
//
// ==========================

// -----------------
// Copy_Board2 just copies the extra data and not the pieces themselves
// -----------------
const int nBSize = (sizeof ( SBoard ) - 120)/4;

void inline Copy_Board2 (SBoard &DBoard, SBoard &CBoard)
{
//memcpy ( &DBoard.Sq[120], &CBoard.Sq[120], sizeof ( SBoard ) - 120);
	__asm
	{   push edi
        push esi
		push ecx
        mov edi, DBoard
        mov esi, CBoard
        add edi, 120
        add esi, 120
        mov ecx, nBSize
        rep movsd
		pop ecx
        pop edi
        pop esi
	}
}

// -----------------
//  Undo Move
//
// Copy Extra info from Board2 onto Board, and update squares on Board based on undo values u1,u2
// -----------------
void inline Undo_Move (SBoard &Board, SBoard &Board2, int u1, int u2)
        {
        Copy_Board2 (Board, Board2);

        Board.Sq[(u1&255)] = (u2&255);
        Board.Sq[(u1>>8)&255] = (u2>>8)&255;
        Board.Sq[(u1>>16)&255] = (u2>>16)&255;
        Board.Sq[(u1>>24)&255] = (u2>>24)&255;
        }

// ============================================
//
// Returns number of repetitions of the boardkey hashV
//
// ============================================
unsigned int RepNum[ MAX_MOVES ];

// Check for repeated board from last unreversible move
// (only checks every other move, because side to move must be the same)

// with Hashvalue passed
int inline DrawByRepetition ( unsigned int hashV, const SBoard &Board)
        {
		int repeats = 0;
		int i = Board.LastReversible - 1;
		if ((i&1) != (Board.GameTime&1)) i++;

        while (i < Board.GameTime)
			{
			 if (RepNum[i] == hashV)
				{
				 repeats++;
				}
			 i+=2;
			}
                
        return repeats;
        }

// Compute Hashvalue from the board
int DrawByRepetition ( SBoard &Board )
        {
        unsigned int value = Hash_Board (Board) ^ HashFunction[Board.EnPassant] ^ HashFunction[Board.WCastle + (Board.BCastle<<3)];
		return DrawByRepetition ( value, Board );
        }

// Calculate Hash From Board
void AddRepBoard (SBoard &Board )
        {
		if (Board.GameTime < MAX_MOVES)
			RepNum[ Board.GameTime ] = Hash_Board (Board) ^ HashFunction[Board.EnPassant] ^ HashFunction[Board.WCastle + (Board.BCastle<<3)];
        }

void inline AddRepBoard ( const unsigned int hashV, const SBoard &Board )
        {
		if (Board.GameTime < MAX_MOVES)
			RepNum[ Board.GameTime ] = hashV;
        }

// ----------------
// Turn a move from the PV into a string
// ---------------- 
int GetPVMoveString (char* buffer, int i, int format)
{
char mid, check;
char c1, c2, c3, c4;
int j = 0;
	if ( MPiece(PV[0][ i ])> 3) check = '+'; else check = ' '; 

	if ( MSrc(PV [0][ i ]) == 253)
		{
		j+= sprintf (buffer + j, "<tt>");
		}
	else if ( MSrc(PV [0][ i ]) == 255)
		{
		j+= sprintf (buffer + j, "#mate");
		}
	else if ( MSrc(PV [0][ i]) == 254)
		{
		}
	else if ( MSrc(PV [0][ i]) == 252)
		{
		j+= sprintf (buffer + j, "<draw>");
		}
	else if ( MSrc(PV [0][ i ]) == 124 || MSrc(PV [0][ i ]) == 126 )
		{
		j+= sprintf (buffer + j, "o-o%c ", check);
		if (format == 1) j+= sprintf (buffer + j, "   ");
		}
	else if ( MSrc(PV [0][ i ] ) == 125 || MSrc(PV [0][ i ]) == 127 )
		{
		j+= sprintf (buffer + j, "o-o-o%c  ", check);
		}
	else
		{
		c1 = (( MSrc(PV[0][ i ]) - 22) % 10) + 'a';
		c2 = '8' - ((MSrc(PV[0][ i ]) - 22) / 10);
		c3 = ((MDst(PV[0][ i ]) - 22) % 10) + 'a';
		c4 = '8' - ((MDst(PV[0][ i ]) - 22) / 10);

		if ( (MPiece(PV[0][ i ])&1) == 1)
			mid = 'x'; else mid = '-';
		if (format == 1) j += sprintf ( buffer + j, "%c%c%c%c%c%c  ", c1 , c2, mid, c3 , c4, check );
		else 
			{
			j += sprintf ( buffer + j, "%c%c", c1 , c2 );
			if ( mid!= '-' ) j += sprintf ( buffer + j, "x");
			j += sprintf ( buffer + j, "%c%c", c3 , c4 );
			if (check != ' ') j += sprintf ( buffer + j, "+");
			j += sprintf ( buffer + j, "  ");
			}
		}
	if ( format == 1 && (i&1) == 0 ) j += sprintf ( buffer + j, "\n");
return j;
}
// ----------------
// Turn the Entire PV into a string
// ---------------- 
void GetPVString ( char *buffer, int format )
{
int j = 0;
int nMoves = PV[0][0];
buffer[0] = NULL;

for (int i = 1; i <= nMoves; i++)
	{
	j += GetPVMoveString (buffer + j, i, format);
	}
}

// ----------------
// Display Principal Variation in Side Window
// ----------------
void DisplayPV ( void )
{
char buffer [1060];
GetPVString ( buffer, 1 );
DisplayText ( buffer);
}

// ----------------
// Play the PV on the board
// ----------------
void PlayPV ( void )
{
int nMoves = PV[0][0];
char buffer [255];
buffer[0] = NULL;

for (int i = 1; i <= nMoves; i++)
	{
	GetPVMoveString (buffer, i, 0);
	DoMovePGN (buffer, GameBoard, TRUE, NULL);
	}
MoveBox ( GameBoard.GameTime - 2 );
UpdateListTranscript ( FALSE );
UpdateMove( FALSE );
DrawBoard ( GameBoard, GameBoard, 255);
}

// ----------------
// Display TT Entry for current Board Position
// ----------------
void DisplayTTEntry ()
	{
	GameBoard.HashKey = Hash_Board ( GameBoard );
	GameBoard.HashCheck = Hash_Board2 (GameBoard);
	int nHashKey = GameBoard.HashKey ^ HashFunction[GameBoard.EnPassant] ^ HashFunction[GameBoard.WCastle + (GameBoard.BCastle<<3)];
	HashEntry TEntry = TranspositionTable [ nHashKey & g_nHashSize ].Near;
	if (TEntry.checksum != GameBoard.HashCheck) TEntry = TranspositionTable [ nHashKey & g_nHashSize ].Far;
	if (TEntry.checksum == GameBoard.HashCheck)
		{
		char sTemp [ 255 ];
		char sMove [ 30 ];
		char sFail [ 6 ];
		if (TEntry.failtype == 1) strcpy (sFail, "<=");
		else if (TEntry.failtype == 2) strcpy (sFail, ">=");
		else strcpy (sFail, "");
		GetMoveString (sMove, MSrc (TEntry.bestmove), MDst (TEntry.bestmove), GameBoard);
		sprintf (sTemp, "Eval: %s%d  \nMove: %s \nDepth: %d \nAge: %d", sFail, TEntry.eval, sMove, TEntry.depth, TEntry.age );
		DisplayText (sTemp);
		}
	else DisplayText ("Not Found");
	}

// ----------------
//
// Add a Move list to the stored PV for depth ahead
//
// ----------------
void inline AddPV (const int &ahead, const int &i, const int &nSearch, const int &bCheck, const SBoard &Board )
	{
	PV[ahead-1][1] = MoveStack[ ahead ].Moves[i] & 65535;
	
	if ( Board.Sq [ MDst(MoveStack[ ahead ].Moves[i]) ]!= EMPTY ) 
			PV[ahead -1][1] |= (1<<16) ; 
	if (bCheck != 0) PV[ahead-1][1] |= (4 << 16);
	
	if (nSearch == 0) 
		{
		for (int i = 0; i < PV[ ahead ][0]; i++)
			{ PV[ahead-1][i+2] = PV[ahead][i+1]; }
		PV[ahead -1][0] = 1 + PV[ ahead ][0];
		}
	else 
		{
		PV[ahead-1][2] = 254 - nSearch; // HT
		PV[ahead-1][0] = 2;
		}
	}

// ----------------
// History Table
// ----------------
void inline AddHistory (const int &nBestMove, const int &depth, const int &ahead, const char &color)
    {
	int nDst = ((nBestMove >> 8) & 127);
	if (SearchBoard.Sq [nDst] != EMPTY) return; // Captures are already sorted differently, so ignore on history
	KillerMoves[ ahead ] = nBestMove;
	if (depth <= 2) return;
	int sq = (nBestMove&127) + (nDst<<7);
	if (color == BLACK)	HistoryB [ sq ] += depth*depth+1;
		else HistoryW [ sq ] += depth*depth+1;
	}

void ClearHistory ( )
    {
	ZeroMemory ( HistoryW, 128 * 128 * sizeof (int) );
	ZeroMemory ( HistoryB, 128 * 128 * sizeof (int) );
	}

//
//  Sort non Attacks by history heuristic 
// 
void SortByHistory (Movestruct &PMoves, int color)
    {
    int tMove;
	unsigned long iVal;
    int i, d, start = PMoves.numAttackmoves;
    static unsigned long SortVals [200];

	// Fill in Sort Values
	 for (i = start; i < PMoves.nummoves; i++)
		{
		 int nMove = PMoves.Moves[i];
		 if (color == BLACK) SortVals[i] = HistoryB [ MSrc(nMove) + (MDst(nMove)<<7) ];
		 else SortVals[i] = HistoryW [ MSrc(nMove) + (MDst(nMove)<<7) ];
		}

	// Insertion Sort
    for (d = start+1; d < PMoves.nummoves; d++ )
		{
        i = d; 
        iVal = SortVals[i];
		tMove = PMoves.Moves [i];
        while ( i > start && iVal > SortVals[i-1])
			{
			 PMoves.Moves [i] = PMoves.Moves [ i-1 ];
             SortVals[i] = SortVals[i-1];
             i--;
            }
		PMoves.Moves[i] = tMove;
        SortVals[i] = iVal;
        }
    }

// ---------------------------------------------------
// ---------------------------------------------------
int inline GetMoveNum (int BestMove, const Movestruct &PMoves)
        {
        for (int i = 0; i <= PMoves.nummoves; i++)
          if ( (PMoves.Moves[i]&65535) == (BestMove&65535) )
             return i;

        return NO_BEST;
        }

int inline GetMoveNum2 (int BestMove, const Movestruct &PMoves)
        {
		if (BestMove == NO_BEST) return NO_BEST;
        int i;
        for (i = 1; i <= PMoves.numAttackmoves; i++)
          if ((PMoves.Moves[i]&65535) == (BestMove&65535) )
             return i;

        for (i = 51; i <= (50+PMoves.nummoves); i++)
          if ((PMoves.Moves[i]&65535) == (BestMove&65535) )
             return i;

        return NO_BEST;
        }

int inline GetMoveNum3 (int BestMove, const Movestruct &PMoves)
        {
		if (BestMove == NO_BEST) return NO_BEST;
        for (int i = 51; i <= (50+PMoves.nummoves); i++)
          if ((PMoves.Moves[i]&65535) == (BestMove&65535) )
             return i;

        return NO_BEST;
        }

void SortMoveList (int BestMove, Movestruct &PMoves, int KillerMove)
        {
	    int index = 0, i;

		int nBest = GetMoveNum2 ( BestMove, PMoves);
		if (nBest!=NO_BEST)
			{
			PMoves.Moves [index] = PMoves.Moves [nBest];
			index++;
			}

		for (i = 1; i <= PMoves.numAttackmoves; i++)
			{
			if (i!=nBest)
				{
				/*if (index !=i)*/ PMoves.Moves [index] = PMoves.Moves [i];
				index++;
				}
			}

		int nKiller = GetMoveNum3 ( KillerMove, PMoves);
		if (nKiller!=NO_BEST && nKiller != nBest )
			{
			PMoves.Moves [index] = PMoves.Moves [nKiller];
			index++;
			}
		
		PMoves.numAttackmoves = index;
		i= 51;
		if (i==nBest || i==nKiller) i++;
        while ( i <= (50+PMoves.nummoves))
			{
			PMoves.Moves [index] = PMoves.Moves [i];
			index++;
			i++;
			if (i==nBest || i==nKiller) i++;
			}
		PMoves.nummoves = index;
        }

//
// Function to detect if a move is legal
//
void inline AddIfLegal ( Movestruct &PMoves, int color, int nMove, int &index )
{
	int enPassant = 120, bCheck;
    int src = MSrc (nMove); 
    int dst = MDst (nMove); 
	char cPiece = SearchBoard.Sq[dst];
	SearchBoard.Sq[dst] = SearchBoard.Sq[src];
	SearchBoard.Sq[src] = EMPTY;

	if (dst == SearchBoard.EnPassant && (SearchBoard.Sq[dst]&7) == WPAWN)
		{
		if (color == WHITE)  enPassant = dst+10; else enPassant = dst-10;  
		SearchBoard.Sq[ enPassant ] = EMPTY;
		}
	if (MPiece (nMove) == 15) bCheck = IsAttacked ( SearchBoard, dst, color );
		else bCheck = IsCheck (color, SearchBoard);
	if ( bCheck == 0 )
		{
		PMoves.Moves [ index++ ] = nMove;
		}
	SearchBoard.Sq[src] = SearchBoard.Sq[dst];
	SearchBoard.Sq[dst] = cPiece;
	if (enPassant!=120) if (color == WHITE) SearchBoard.Sq[enPassant] = BPAWN; else SearchBoard.Sq[enPassant] = WPAWN;
}

//
// Discard illegal moves while sorting them (used when in check.)
//
void SortLegalMoves (int BestMove, Movestruct &PMoves)
        {
        int index = 0, i;
		int nBest = GetMoveNum2 ( BestMove, PMoves);
		int color = (int)SearchBoard.SideToMove;

		if (nBest!=NO_BEST)
			{
			AddIfLegal ( PMoves,  color, PMoves.Moves[nBest], index );
			}

		for (i = 1; i <= PMoves.numAttackmoves; i++)
			{
			if (i!=nBest) AddIfLegal ( PMoves,  color, PMoves.Moves[i], index );
			}
		PMoves.numAttackmoves = index;
        for (i = 51; i <= (50+PMoves.nummoves); i++)
			{
			if (i!=nBest) AddIfLegal ( PMoves,  color, PMoves.Moves[i], index );
			}
		PMoves.nummoves = index;
        }

//
//  Sort Attacks by Most valuable capture, least valuable attacker
// 
void SortAttacks (Movestruct &PMoves, SBoard &Board)
	{
    int i, d, iVal, temp;
	static int SortVals [48];
	int nMove;

	for (i = 1; i <= PMoves.numAttackmoves; i++)
		{
		nMove = PMoves.Moves[i];
		SortVals[i] = PieceValues[ Board.Sq [ MDst(nMove) ] &7]*8 - PieceValues[ Board.Sq [ MSrc(nMove) ] &7];
		}

	// Insertion Sort
    for (d = 2; d <= PMoves.numAttackmoves ; d++ )
           {
            i = d; 
            iVal = SortVals[i];
			temp = PMoves.Moves[i];
            while ( i >= 2 && iVal > SortVals[i-1])
                    {
					PMoves.Moves[i] = PMoves.Moves [i-1];
                    SortVals[i] = SortVals[i-1];
                    i--;
                    }
 		   PMoves.Moves[i] = temp;
           SortVals[i] = iVal;
           }
	}

// -----------------------------------------------------
// Set time to think for this move from time left and how many moves have been made
// nMoves = nMoves in a time period   nSecs = Increment per move
// nCompTm = Time remaining,
//  ----------------------------------------------------
void SetThinkTime (float nCompTm, int nMoves, int nSecs )
{
	int nMovesPlayed = GameBoard.GameTime/2;

	if (nMoves > 0) 
		{
		nMovesPlayed = nMovesPlayed % nMoves;
		int nMovesLeft = nMoves - nMovesPlayed;
		Thinktime = nCompTm / float( nMovesLeft+1 );
		}
	else
		{
		if (GameBoard.GameTime < 60) Thinktime = nCompTm / (56 - nMovesPlayed);
		else if (GameBoard.GameTime < 134) Thinktime = nCompTm / (88 - nMovesPlayed);
		else if (GameBoard.GameTime < 210) Thinktime = nCompTm / (118 - nMovesPlayed);
		else Thinktime = nCompTm / ((GameBoard.GameTime*2 + 10) - nMovesPlayed);
		Thinktime += (float)nSecs;
		}

	if ((Thinktime+1) > nCompTm) Thinktime = nCompTm/2;
	if (Thinktime < .01f ) Thinktime = .01f;
	fPanicTime = Thinktime * 1.7f;
	if ((Thinktime+fPanicTime) + 5 > nCompTm)  fPanicTime = 0.0f;
}
						
// ----------------
// Set HashValue and info displaying globals
// ----------------
int g_bMating, g_nQueenSide;

// Called before each iteration
void InitIterationGlobals ( SBoard &Board )
{
	 SearchBoard = Board;
	 //
	 if ( SearchBoard.BCastle > 2 ) SearchBoard.BCastle = 4;
	 if ( SearchBoard.WCastle > 2 ) SearchBoard.WCastle = 4;
	 // Calculate initial hash values
	 SearchBoard.HashKey  = Hash_Board (SearchBoard);
     SearchBoard.HashCheck = Hash_Board2 (SearchBoard);
	 SearchBoard.PawnHash = Pawn_HashBoard ( SearchBoard );

	 // Maximum depth, the fullwidth search will kick out after this depth no matter what.
	 g_MaxDepth = SearchDepth + 8;
	 if (SearchDepth > 3) g_MaxDepth = 3*SearchDepth + 4;
	 if (g_MaxDepth >= MAXDEPTH -3) g_MaxDepth = MAXDEPTH-4;
	 // Move Stack
	 MoveStack[ 0 ].extended = 0;
	 MoveStack[ 0 ].recap = 31001;
	 MoveStack[ 1 ].eChecks = 2;

	 if ( IsCheck(Board.SideToMove, SearchBoard) == 1) MoveStack[1].ischeck = 1; // Set Check Flag
				else MoveStack[1].ischeck = 0;

	 // To time how long the first move takes at this depth
	 if (SearchDepth > 4) MoveTm = clock ();
	 g_nFail = 0;
	 CurrentMoveNum = 1;
}

// Called before each search
void InitSearchGlobals ( SBoard &Board )
	{
	 static int LastQS = 0;
	 g_ucAge++;

	 g_nQueenSide = 0;
	 if (Board.WCastle > 2 && SQFILE [ Board.WPiece [15] ] < 5) g_nQueenSide|=1;
	 if (Board.BCastle > 2 && SQFILE [ Board.BPiece [15] ] < 5) g_nQueenSide|=2;

	 g_bMating = FALSE;
	 if (Board.nBPawns == 0 && Board.nWPawns == 0 && (Board.nWP == 0 || Board.nBP==0) ) g_bMating = TRUE;

	 if (LastQS != g_nQueenSide)
		{
		LastQS = g_nQueenSide;
		Clear_PawnHash ();
		}

	 for (int i = 0; i < MAXDEPTH; i++) KillerMoves[i] = NO_BEST;

	 DRAWSCORE = 0;
	 if (Board.GameTime < 60) DRAWSCORE = -20;
	 if (Board.GameTime > 120 && Board.nWP < 3 && Board.nBP < 3) DRAWSCORE = 30;
	 if (Board.SideToMove == BLACK) DRAWSCORE *=-1;
	}


// ------------------------
//  Quiesce Entire Board  
//
//  Cycles through all attacks or pawn promos by color, does the move
//  returns best possible change in material value for color
// ------------------------
short CheckBoard( int color, int alpha, int beta, int ahead)
{
int MMove, u1, u2, bCheck, nSrc, nBP, nWP,  oColor, nMovType;
short Value, oValue;

ahead++;
oValue = EvaluateBoard ( SearchBoard, alpha, beta );

if (ahead >= MAXDEPTH) return oValue;

	// Store no move for PV
	PV[ahead-1][1] = 254;
	PV[ahead-1][0] = 1;

   if (color==WHITE) 
		{
	    if ( oValue >= beta) { return beta;}
		if ( oValue > alpha) alpha = oValue;
		oColor = BLACK;
        FindAttackMovesWhite(SearchBoard, MoveStack[ahead] );
		}	
   else
        {
	     if ( oValue <= alpha) { return alpha;}
		 if ( oValue < beta) beta = oValue;
		 oColor = WHITE;
         FindAttackMovesBlack(SearchBoard, MoveStack[ahead] );
	    }

// remove liquidation/more pieces bonuses
oValue -= MatEval (SearchBoard, SearchBoard.nWP, SearchBoard.nBP, SearchBoard.MatVal );  

// Set Opposite Color and Find possible moves
 SortAttacks ( MoveStack[ahead], SearchBoard);
 Copy_Board2 (BoardStack[ahead], SearchBoard);

 for (short i = 1; i <= MoveStack[ahead].numAttackmoves; i++)
            {
  			 nodes++; 
			 MMove = MoveStack[ahead].Moves[i];
			 nSrc =  MSrc(MMove);
			 Value = -PieceValues[ SearchBoard.Sq [ MDst (MMove) ] ];
			 if ( SearchBoard.Sq[ nSrc ]==WPAWN && SQRANK [nSrc] == 7) Value+= PieceValues [WQUEEN]; //Pawn Promos
			 if ( SearchBoard.Sq[ nSrc ]==BPAWN && SQRANK [nSrc] == 2) Value+= PieceValues [BQUEEN];
			 // liquidation/more pieces added back in (may have changed)
			 nBP =  SearchBoard.nBP; nWP =  SearchBoard.nWP;
			 if (Value < -200) nWP--;
			 if (Value >  200) nBP--;
			 Value = oValue + MatEval ( SearchBoard, nWP, nBP, SearchBoard.MatVal + Value) + Value;

			 // Is (OldEval)-(Material Captured) within 1 pawn of the alpha-beta window?
			 // (and always test after captures to 1 or 0 pieces.)
			 if (  ( color==WHITE && (nBP <2 || Value > (alpha-120) ) ) 
			    || ( color==BLACK && (nWP <2 || Value < (beta+120 ) ) ) )
				{
				nMovType = Do_Move(SearchBoard, MMove , u1, u2, WQUEEN); 
				if (MoveStack[ahead].ischeck!=0 || MPiece(MMove) == 15 ) bCheck = IsCheck(color, SearchBoard);
			 		else bCheck = DiscoveredCheck (color, SearchBoard, MMove);

			    MoveStack[ahead+1].ischeck = CheckingMove (color, SearchBoard, MMove, nMovType );

				if (bCheck == 0) // Not left in check, ie. legal move
					{
					Value = CheckBoard( oColor, alpha, beta, ahead);
					if (color==WHITE) 
						{
						if ( Value >= beta) {Undo_Move (SearchBoard, BoardStack[ahead], u1, u2); return beta;}
						if ( Value > alpha) 
							{alpha = Value;
							 AddPV ( ahead, i, 0, 0, SearchBoard );
							}
						}	
					else
						{
						if ( Value <= alpha) {Undo_Move (SearchBoard, BoardStack[ahead], u1, u2); return alpha;}
						if ( Value < beta) 	
							{beta = Value;
							 AddPV ( ahead, i, 0, 0, SearchBoard );
							}
						}
 					}
				Undo_Move (SearchBoard, BoardStack[ahead], u1, u2);
				}
 			}

  if (color == WHITE) return alpha; else return beta;
}

// ----------------
// Do Fractional Extensions
// ----------------
void inline DoExtensions (const int &depth, const int &ahead, const int &Ext, int &nextDepth, const int &dstSq)
{
	nextDepth = depth-1;
	MoveStack [ahead].extended = MoveStack[ahead-1].extended;

	// Pawn Push or In Check
	if ( Ext >= PAWNPUSH || (MoveStack[ahead+1].ischeck !=0) )
			{
			if (ahead <= (2*SearchDepth)) MoveStack[ahead].extended+=PLY;
				else 
				MoveStack[ahead].extended += HALFPLY + QUARTPLY;
			}
	else // Recapture
		if  (MoveStack[ahead].ischeck ==0 && dstSq == MoveStack[ahead-1].recap) 
			{
			MoveStack[ahead].recap = 31001; // Don't do 2 recaps in a row
			MoveStack[ahead].extended+= HALFPLY + QUARTPLY;
			}
		// Single Move in check
	else if ( MoveStack[ahead].nummoves == 1) 
			{
			 MoveStack[ahead].extended += HALFPLY + QUARTPLY;
			}
	else if (MoveStack[ahead].nummoves == 2) // 2 responses to check
		{
		MoveStack[ahead].extended += QUARTPLY;
		}
	// Increment depth if needed
	if (MoveStack [ahead].extended >= PLY) 
			{
			 MoveStack[ahead].extended -= PLY;
			 nextDepth++;
			}

	// Don't end a search after 2 replies to check, unless we extended past the depth left already
	// (and never stop after 1 reply...)
	if (nextDepth<1) MoveStack[ahead+1].eChecks = 1; else MoveStack[ahead+1].eChecks = 2;
}

// ==========================
// GAME TREE SEARCH
// searches to depth
// returns evaluation of the best move found, (the move itself is returned in bestmove)
// BLACK = MIN, WHITE = MAX
// uses SearchBoard as input Board
//
// ==========================
short ABSearch(int color, int ahead, int depth,  short alpha, short beta,  int &bestmove, int bAfterNull)
{
if (stopthinking > 0 || g_ponder == 3 ) return TIMEOUT;
if ( g_nMaxNodes !=0 && nodes > g_nMaxNodes && SearchDepth > 1) return TIMEOUT;
// ------- Check for time up and display search info every 100,000 nodes ---------
if (nodes > nodes2) 
	{
    endtime = clock ();
    nodes2 = nodes + 100000;
	if (IsPanic () == TRUE) endtime-= (long)(CLOCKS_PER_SEC * (fPanicTime));
    if ( (g_ponder !=1 && (endtime - starttime > CLOCKS_PER_SEC * Thinktime)) )
         return TIMEOUT;  // Search time is used up, abort the search

    DisplaySearchInfo();
    }
// ----------------------

int u1, u2, nHashKeyAndFlags;
int Max, i, smax, Ext = 0;
int nextbest, nextDepth;
int nSearched, bPV = FALSE, MMove, ocolor;
short value;

ahead++;
// ------- Try Null Move ---------
if (bAfterNull != TRUE &&  MoveStack[ahead].ischeck == 0 && depth > 2 && g_bMating == FALSE)
if (   (color == BLACK && (SearchBoard.nBP > 1 || SearchBoard.nBP > SearchBoard.nBMinors) ) 
	|| (color == WHITE && (SearchBoard.nWP > 1 || SearchBoard.nWP > SearchBoard.nWMinors)) )
	{
	// If the static eval isn't crummy, try a null move
	value = EvaluateBoard ( SearchBoard, alpha-250, beta+250 );
	if ( ( (value+200) >= beta && color == WHITE) 
	 || ( (value-200) <= alpha && color == BLACK))
		{
		MoveStack[ahead+1].ischeck = 0;
		nextbest = NO_BEST;
		// ABSearch will search at least 1 ply even if depth is 0 or less
		BoardSwitchColor ( );
		if (color == WHITE ){value =  ABSearch (BLACK, ahead, depth-1 - 2, beta-1, beta, nextbest, TRUE );
							}
		if (color == BLACK ) {value =  ABSearch (WHITE, ahead, depth-1 - 2, alpha, alpha+1, nextbest, TRUE);
							}
		BoardSwitchColor ( );

		if (value == TIMEOUT) return TIMEOUT;
		if (value >= beta && color == WHITE ) return beta;
		if (value <= alpha && color == BLACK) return alpha; 
		}
	}
// --------------------------

// Internal iterative deepening to find a guess for the best move
if (bestmove == NO_BEST && ahead!=1 && depth >= 3) 
        {
		value = ABSearch( color , ahead-1, depth-2, alpha, beta,  bestmove, FALSE );
		if ( value == TIMEOUT) return TIMEOUT;
        }

// Set Opposite Color var and Find possible moves
if (color == WHITE) 
		{ocolor = BLACK;
         smax = 1;
		 if ( (9999-ahead) <= alpha) return alpha-1; // we already know white can mate sooner (value can't be > alpha)
         FindMovesWhite(SearchBoard, MoveStack[ahead]);
        }
     else  {ocolor = WHITE;
            smax = -1;
			if ( (-9999+ahead) >= beta) return beta+1; // (value can't be < beta)
            FindMovesBlack(SearchBoard, MoveStack[ahead]);
           }

SortAttacks ( MoveStack[ahead], SearchBoard );
// In Check, discard illegal moves
if (MoveStack[ahead].ischeck!=0) 
	{
	SortLegalMoves ( bestmove, MoveStack[ahead]);
	if (MoveStack[ahead].nummoves == 0) // 0 moves == CheckMate
		{
		PV[ahead-1][1] = 255; PV[ahead-1][0] = 1; // Add mate to PV
		return (-9999+ahead) * smax; 
		}
	}
else 
	{
	SortMoveList (bestmove, MoveStack[ahead], KillerMoves[ahead]);
	}

// Draw by 50 move rule (here because we have to check for mates first)
if ((SearchBoard.GameTime - SearchBoard.LastReversible) >= 100 ) 
	{PV[ahead-1][1] = 252; PV[ahead-1][0] = 1;return DRAWSCORE;}

Max = short (-19999*smax);

// Save Undo Board & Hash Values
Copy_Board2 (BoardStack[ahead], SearchBoard);

int numMoves = MoveStack[ahead].nummoves;

// Cycle through the moves and evaluate them
    for (i = 0; i < numMoves; i++)
                 {
				 // Sort by History if Bestmove, captures, and killer all failed
				 if ( depth>1 && ahead<11 && i == MoveStack[ahead].numAttackmoves ) SortByHistory (MoveStack[ahead], color);
				 //
				 MMove = MoveStack[ahead].Moves[i];
				 // Is this a legal move? If not skip it
				 if (MoveStack[ahead].ischeck==0)
					{
					 if (MPiece(MMove) == 15) // Moved King
						{if ( IsAttacked ( SearchBoard, MDst(MMove), color) == 1 ) continue;
						}
					 else
						{if (DiscoveredCheck (color, SearchBoard, MMove) == 1 ) continue;
						}
					}

				 // If so do it
                 Ext = Do_Move(SearchBoard, MMove , u1, u2, WQUEEN);
				 MoveStack[ahead].recap = Ext;
                 nodes++;

				 // Set globals for display info, send PV to winboard if it changes
                 if (ahead == 1 ) UpdateDisplayInfo ( i, MMove, Max, bestmove);
                    
				 MoveStack[ahead+1].ischeck = CheckingMove (color, SearchBoard, MMove , Ext );

 				 nSearched = 1;

			     // Update HashValues
                 HashUpdate (u1, u2, SearchBoard );
				 nHashKeyAndFlags = SearchBoard.HashKey ^ HashFunction[SearchBoard.EnPassant] ^ HashFunction[SearchBoard.WCastle + (SearchBoard.BCastle<<3)];
 			     AddRepBoard( nHashKeyAndFlags, SearchBoard);

                 // Draw by repetition
				 int Reps = DrawByRepetition (nHashKeyAndFlags, SearchBoard);
                 if ( (Reps > 1 || (ahead > 2 && Reps > 0) ) 
					 // Draw by Insufficent Material
					 || ( SearchBoard.nBPawns == 0 && SearchBoard.nWPawns == 0 && SearchBoard.nBP <= 1 && SearchBoard.nWP <= 1 && SearchBoard.nBP == SearchBoard.nBMinors && SearchBoard.nWP == SearchBoard.nWMinors ) 
					) 
					{
					 value = DRAWSCORE; nSearched = 2;
					}
                 // If there is no depth left in the search, and the opponent isn't in check, evaluate the board position
                 else if ( ( depth <= 1 && MoveStack[ahead+1].ischeck== 0 && Ext!= PAWNPUSH && numMoves > MoveStack[ahead].eChecks) || (ahead > g_MaxDepth ) )
                       {
						if ( color==WHITE && alpha > 9900 ) value = alpha;
						else if ( color==BLACK && beta < -9900 ) value = beta;
						else value = CheckBoard( ocolor, alpha, beta, ahead);
						nSearched = 0;
						}

                  //If not look ahead to all possible moves
                  else { 
                        value = NOVALUE;
                        nextbest = NO_BEST;

						DoExtensions ( depth, ahead, Ext, nextDepth, MDst (MMove) );
							
						TT_Lookup ( SearchBoard.HashCheck, TranspositionTable [ nHashKeyAndFlags & g_nHashSize ], value, nextbest, alpha, beta, ahead, nextDepth, bPV);

                        if (value == NOVALUE) // No value found in TT
                                {
								if (bPV == TRUE) // Ugly looking PV search code.
									{
									if (smax == 1) 
										{
										value = ABSearch( ocolor , ahead, nextDepth, alpha, alpha+1,  nextbest, FALSE ); 
										if (value > alpha && value!=TIMEOUT )  // Fail High
											{
											if (ahead == 1) { bestmove = (MMove & 65535); CurrentBest = bestmove; g_nFail = 1; AddPV ( ahead, i, 0, MoveStack[ahead+1].ischeck, GameBoard );
															  if (SearchDepth > 3) wbSendThinking (SearchDepth, Max, (100*(clock() - starttime))/( CLOCKS_PER_SEC ), nodes, TRUE, TRUE, FALSE);}
											value = ABSearch( ocolor , ahead, nextDepth, alpha, beta,  nextbest, FALSE ); // Actual recursive call
											}
										}
									else 
										{
										value = ABSearch( ocolor , ahead, nextDepth, beta-1, beta,  nextbest, FALSE ); 
										if (value < beta && value!=TIMEOUT ) 
											{
											if (ahead == 1) {bestmove = (MMove & 65535); CurrentBest = bestmove; g_nFail = 1; AddPV ( ahead, i, 0, MoveStack[ahead+1].ischeck, GameBoard  );
															 if (SearchDepth > 3) wbSendThinking (SearchDepth, Max, (100*(clock() - starttime))/( CLOCKS_PER_SEC ), nodes, TRUE, TRUE, FALSE);}
											value = ABSearch( ocolor , ahead, nextDepth, alpha, beta,  nextbest, FALSE ); // Actual recursive call
											}
										}
									}
								else
									{
									value = ABSearch( ocolor , ahead, nextDepth, alpha, beta,  nextbest, FALSE ); // Actual recursive call
									}
                                if (value == TIMEOUT) return TIMEOUT; // Timelimit reached, bail out
				
								TT_Store ( SearchBoard.HashCheck, TranspositionTable [ nHashKeyAndFlags & g_nHashSize ], value, nextbest, alpha, beta, ahead, nextDepth);
								nSearched = 0;
                               }

						// reduce a 2nd repetition to 7/8ths
						if (Reps == 1 && value != DRAWSCORE) value = (value>>1) + (value>>2) + (value>>3); 
                       }

			 // Restore Board & Hash Values
             Undo_Move (SearchBoard, BoardStack[ahead], u1, u2);

              // Keep Track of Best Move and Alpha-Beta Prune
             if (smax == 1 &&  value > Max) 
					{
                     Max = value;
                     bestmove = (MMove & 65535);
					 AddPV ( ahead, i, nSearched, MoveStack[ahead+1].ischeck, SearchBoard );

					 if (value >= beta || value == (9998-ahead) ) 
						{
						 AddHistory ( bestmove, depth, ahead, WHITE);
						 if (ahead == 1) {CurrentEval = value;}
                         return value;
						}
                     if (value > alpha ) 
						{
						 bPV = TRUE;
						 alpha = value;
						}
					}	
              if (smax == -1 &&  value < Max) 
                    {
                     Max = value;
                     bestmove = (MMove & 65535);
					 AddPV ( ahead, i, nSearched, MoveStack[ahead+1].ischeck, SearchBoard  );
                                                                          
					 if (value <= alpha || value == (-9998+ahead )  )  
							{
							 AddHistory ( bestmove, depth, ahead, BLACK); 
							 if (ahead == 1) {CurrentEval = value;}
							 return value;
							}
                     if (value < beta ) 
						{
						 bPV = TRUE;
						 beta = value;
						 }
				    }
                 }

// ---------- Done going through all the moves (No alpha/beta cuts) ------------
if (ahead==1) {CurrentEval = Max;}
else // StaleMate?
    {
	if ( Max == (-19999*smax) ) return DRAWSCORE;
    }

AddHistory ( bestmove,  depth, ahead, color); 

return Max;
}

               
/* ===============================================================
 *
 * Computer Move
 * have the computer do a search to find its move, then perform the move on Board
 *
 ================================================================ */
int g_lasteval;

int CompMove (SBoard& Board, int nFlags, int MaxAhead)
        {
        unsigned char  Ocolor;
        int Eval, nBestMove, nTime;
		int u1, u2;
        
        nodes = 0;   Bf = 0; Bnodes = 0;
		nodes2 = nodes + 100000;
		g_lasteval = 0; g_fMoveTm = 0.0f;
		if (nFlags == 1) g_bAnalyze = TRUE; else g_bAnalyze = FALSE;
              
        upgrade = WQUEEN; // pawn advancement
        stopthinking = 0; // Changes to 1 stops the search.
        
        SearchBoard = Board;

        starttime = clock();
		Ocolor = Board.SideToMove;
		g_SearchColor = Board.SideToMove;

		SetEvent ( hEngineReady ); // Started thinking, can receive more commands.

		// Check in the Opening book for first 70 half-moves
        if (g_bAnalyze==FALSE && Board.GameTime < 70 && (g_lasteval = pNewBook->DoMove (SearchBoard)) != NOVAL) 
			{
			g_lasteval *= 10;
			SearchDepth = 1;
			PV[ 0 ][1] = 254;
			PV[ 0 ][0] = 1;
			}
        else
           {	 g_lasteval = 0;
                 Eval = 0;
				 SearchDepth = 1;
                 nBestMove = NO_BEST;
               	 ClearHistory ( );
				 InitSearchGlobals ( Board );	

				 // -----  Iterative Deepening -----------------------
                 while ( SearchDepth <= MaxAhead)
                      {
						InitIterationGlobals ( Board );	

                        Eval = ABSearch (Ocolor, 0, SearchDepth, -9998 + SearchDepth, 9998 - SearchDepth, nBestMove, TRUE);

 						if (g_bWinBoard == TRUE && (SearchDepth >3 || abs(Eval) >= (9997 - SearchDepth - 1)) && Eval!=TIMEOUT) 
							{
							nTime = (100*(clock() - starttime))/( CLOCKS_PER_SEC );
							CurrentMove = nBestMove;
							CurrentMoveNum = 1;
							wbSendThinking (SearchDepth, Eval, nTime, nodes, TRUE, FALSE, TRUE);
							}

						SearchDepth++;
                        if (Eval == TIMEOUT) 
							{
							if (CurrentMoveNum==1) SearchDepth--; // Didn't finish any searching this depth
							break;
							}
						g_lasteval = Eval;
						if ( abs(Eval) >= (9997 - SearchDepth) ) break; // CheckMate, don't search further
						if ( MoveStack[1].nummoves == 1) break; // only 1 move, just do it.
					
						if ( SearchDepth >4 && g_ponder == 0)
							{
							// Stop now if very likely won't finish searching first move on next depth anyway
							if ( (Thinktime-float(clock() - starttime)/( CLOCKS_PER_SEC ) ) < (g_fMoveTm * 1.1f)) break;
							}
                       }
				 // ---------------------------------------------------

				if (abs(CurrentEval) < 9999) g_lasteval = CurrentEval;
				if (stopthinking == 3) return 0;
				SearchBoard = Board;
				
				if (nBestMove != NO_BEST && nFlags!=1) // Do the Best Move Found
					{
					int nMove = GetMoveNum ( nBestMove, MoveStack[1]);
					if (abs( g_lasteval) < 9998 && (nFlags == 0 || (nFlags==2 && g_ponder==2)) ) 
						{
						// Delay before moving if requested
						 if (g_bAlwaysWait == TRUE && g_bWinBoard == FALSE )
							{
							 DisplayText ("Waiting...");
							 while ( (clock () - starttime) < (CLOCKS_PER_SEC * Thinktime) && stopthinking <= 0) {}
							}
						 // Add move to transcript then do it.
						 AddTranscript (NULL, MoveStack[1].Moves, nMove,  SearchBoard);
						 Do_Move(SearchBoard, MoveStack[1].Moves[ nMove ], u1, u2, WQUEEN);
						 g_GameResult = NONE;
						}
					}
           }
		endtime = clock();
            
        if ((nFlags == 0 || (nFlags==2 && g_ponder==2))) 
				{
				if (CheckGameOver (Board, FALSE, g_GameResult) == TRUE)
					if (g_GameResult == DRAW) g_lasteval = 10000;

				if (g_bWinBoard == TRUE)
					{
					ResetEvent ( hEngineReady ); // Finish everything related to this move before processing more engine input
					nTime = (100*(clock() - starttime))/( CLOCKS_PER_SEC );
					// Maybe just put this wbSendThinking up near the other one,
					// it's only needed if the search timesout in the middle, or is under depth 4
					wbSendThinking (SearchDepth - 1, g_lasteval, nTime, nodes, TRUE, FALSE, FALSE);
					Board = SearchBoard;
					wbSendMove ( );
					if (g_GameResult == NONE) wbResign ( g_lasteval );
					wbTestSendResult ( Board );
					return g_lasteval;
					}
				 else
					{
					DrawBoard  ( SearchBoard, Board, 255); // Draw Board and Show last move
					DisplayFinalSearchInfo ( g_lasteval, SearchDepth, Ocolor, nBestMove );
					Board = SearchBoard;
					TrimList ();
					UpdateMove( TRUE );
					}
				}
		if (g_bAnalyze == TRUE && g_bWinBoard == FALSE) DisplayFinalSearchInfo ( g_lasteval, SearchDepth, Ocolor, nBestMove );
	
        return g_lasteval;
        }

/* ===============================================================
 *  Eval has dropped almost a pawn or more (nMargin) since last iteration
  ================================================================ */
int IsPanic ( )
{
  if (abs(CurrentEval) < 19000) // This just means we have a value (NOVALUE constant is outside this range.)
	{
	int nMargin = 80;
	if ( abs (g_lasteval) < 200) nMargin = 50;

	if ( GameBoard.SideToMove == WHITE) 
		  if (CurrentEval < (g_lasteval-nMargin) && g_lasteval > -800 && (CurrentEval < 500)) return TRUE;
	if ( GameBoard.SideToMove == BLACK)
		  if (CurrentEval > (g_lasteval+nMargin) && g_lasteval < 800 && (CurrentEval > -500) ) return TRUE;
	}
  return FALSE;
}

// ---------------------------------
//  Resign Lost Games
// ---------------------------------
int wbResign ( int nEval )
{
 static int Resign = 0;
 if (PlayerCOLOR == BLACK)
	 if (nEval < - 720) Resign++; else Resign = 0;
 if (PlayerCOLOR == WHITE)
	 if (nEval > 720)  Resign++; else Resign = 0;
 if (Resign > 3)
	{
	// only resign if we are down pieces already so we don't resign against a human for something he/she may not see
	// also there needs to be at least one major piece (BN mates are hard, NN impossible (except helpmate), so this way we don't resign them)
	if ( (PlayerCOLOR == BLACK && GameBoard.MatVal < -500 && GameBoard.nBP > GameBoard.nBMinors && (GameBoard.nBP > GameBoard.nWP || GameBoard.nBPawns > 0)) 
		 || (PlayerCOLOR == WHITE && GameBoard.MatVal > 500 && GameBoard.nWP > GameBoard.nWMinors && (GameBoard.nWP > GameBoard.nBP || GameBoard.nWPawns > 0) )		) 
		{
		printf("resign\n");
		fflush(stdout);
		return TRUE;
		}
	}

 // Also Offer Draws
 static int Draw = 0;
 if ( (nEval == DRAWSCORE || nEval == 0 || abs(nEval) == 1) && GameBoard.GameTime > 84)
	{
	 Draw++;
	 if (Draw > 4 )
		{
		 Draw = -3;
		 printf("offer draw\n");
		 fflush(stdout);
		 return TRUE;
		}
	}
 else Draw = 0;

 // Will only Accept Draws in the Endgame (this is in other code)
 bAcceptDraw = FALSE;
 static int AcceptDraw = 0;
 if ( (PlayerCOLOR == BLACK && nEval <= DRAWSCORE) || (PlayerCOLOR == WHITE && nEval >= DRAWSCORE) )
	{
	 AcceptDraw++;
	 if (AcceptDraw > 5) bAcceptDraw = TRUE;
	}
 else AcceptDraw = 0;

 return FALSE;
}


/* ===============================================================
// Set Globals used to diplay bestmove, PV, move being searched, etc.
 ================================================================ */
void UpdateDisplayInfo ( int i, int MMove, int Max, int bestmove )
	{
	 g_nFail = 0;
	 CurrentMoveNum = i + 1; 
	 CurrentMove = MMove;
	 if ( (SearchDepth > 3 && abs(CurrentEval) > 19000 && abs(Max) < 19000) || ( SearchDepth > 5 && bestmove!=CurrentBest && abs(Max) < 19000) ) 
		{
		wbSendThinking (SearchDepth, Max, (100*(clock() - starttime))/( CLOCKS_PER_SEC ), nodes, TRUE, FALSE, FALSE);
		}
	 if (SearchDepth > 4 && abs (CurrentEval) > 19000 && abs(Max) < 19000)
	 g_fMoveTm = (float)(clock()-MoveTm)/( CLOCKS_PER_SEC );
     CurrentEval = Max;
     if (bestmove!=NO_BEST) CurrentBest = bestmove;
}

/* ===============================================================
 * Store Eval display in a string
 ================================================================ */
void GetEvalString ( char *sEvalString, int nEval)
{
	int temp = 0;
	int k = 0;

	if (abs(nEval) > 19000) 
		{
		 nEval = g_lasteval; 
		 temp = 1;
		}
	if (abs(nEval) > 9900) 
		{
		if ( nEval < 0) k = sprintf (sEvalString, "Black ");
			else k = sprintf (sEvalString, "White ");
		k+=sprintf (sEvalString + k, "mates in %d ", (9999 - abs(nEval))/2 );
		}
	else 
		{
		if (bEvalWhite == FALSE && g_SearchColor == BLACK) nEval = -nEval;
		k+=sprintf (sEvalString, "%.2f ", (float)nEval / 100.0f );
		}
	if (temp == 1 || g_nFail == 1) 
		{sEvalString [k] ='*'; sEvalString [k+1] = NULL;}
}

/* ===============================================================
 * Store Move in a string
 ================================================================ */
void GetMoveString (char sBuffer[], int src, int dst, SBoard const &Board)
{
	  int i = 0;
	  if (PieceChars [Board.Sq[src]&7] !='p') 
		  sBuffer [i++] = PieceChars [ Board.Sq[src]&7 ];
      sBuffer [i++] = ((src - 22) % 10) + 'a';
      sBuffer [i++] = '8' - ((src - 22) / 10);
      sBuffer [i++] = (Board.Sq [dst] == EMPTY) ? '-':'x';
      sBuffer [i++] = ((dst - 22) % 10) + 'a';
      sBuffer [i++] = '8' - ((dst - 22) / 10);
	  sBuffer [i++] = NULL;

	  if (src == 124 || src == 126) strcpy (sBuffer, "O-O  ");
      if (src == 125 || src == 127) strcpy (sBuffer, "O-O-O");
}

/* ===============================================================
 * Display Search info in Side Window
 ================================================================ */
void DisplayFinalSearchInfo ( int nEval, int SearchDepth, char cOcolor, int bestmove )
	{
     char buffer[512];
     char buffer2[128];
	 int j = 0;
	 float fThinkSec;

		if (bDisplayInfo == FALSE && g_nFlags!=1)
			{
			DisplayText ("Search Info Off.");		
			return;
			}
		if ( DrawByRepetition (GameBoard)  > 1 ) 
			{
			sprintf (buffer2, "Draw by Repetition ");
			}	
		else if ( nEval == 10000 )
			{
			sprintf (buffer2, "Draw");
			}
		else if ( MoveStack[1].nummoves == 1)
			{
			sprintf (buffer2, "Single Move");
			}
		else GetEvalString (buffer2, nEval );

		if (nodes < 10000) j+= sprintf (buffer+j, "Eval: %s\nNodes: %d\nDepth: %d", buffer2, nodes, SearchDepth - 1);
		else j+= sprintf (buffer+j, "Eval: %s\nKNodes: %d\nDepth: %d", buffer2, (nodes/1000), SearchDepth - 1);

		if (CurrentMoveNum!=1 && CurrentMoveNum!=MoveStack[1].nummoves) j+=sprintf (buffer+j, "*", CurrentMoveNum-1, MoveStack[1].nummoves);
			//j+=sprintf (buffer+j, "  [%d/%d]", CurrentMoveNum-1, MoveStack[1].nummoves);

		if (nodes > 1000 && (endtime-starttime)> 2)
            {
		     long NPS = long (float (nodes / 1000)/ ( float(endtime - starttime)/CLOCKS_PER_SEC) );
			 j+= sprintf (buffer + j, "\nKN\\sec: %d", NPS);
            }
		fThinkSec = ( float(endtime - starttime )/CLOCKS_PER_SEC );
		fThinkSec -= .1f;
		if (fThinkSec < 0.0f) fThinkSec = 0.0f;
	   	j+= sprintf (buffer + j, "\nTime: %.1f", fThinkSec  );

     	DisplayText ( buffer );
	}

/* ===============================================================
 * Send Winboard Analyzing Updates
 ================================================================ */
void  wbSendThinking2( int bFinishedDepth )
{
	if (g_bAnalyze == FALSE || thinking == 0) return;
	int tm = (100*(clock() - starttime))/( CLOCKS_PER_SEC );
	int nDepth;
	char sCurMove[20];
	GetMoveString (sCurMove, MSrc (CurrentMove ), MDst ( CurrentMove ), GameBoard );
	// If we finished at this depth, by the time the user sees this we'll be searching the next
	if (bFinishedDepth == TRUE) nDepth = SearchDepth+1; else nDepth = SearchDepth;

	printf("stat01: %d %u %d %d %d %s\n", tm, nodes, nDepth, MoveStack[ 1 ].nummoves-CurrentMoveNum, MoveStack[ 1 ].nummoves, sCurMove);
	fflush(stdout);
}

// ----------------
// Display search Info.
// Display a running tally of info like the Current Evluation, Current Best Move, node count, etc.
// ----------------
void DisplaySearchInfo (void)
	{
	  char buffer[512], buffer2[128];
	  int k = 0;
	  int src, dst, nMoves;

	  if (g_bWinBoard == TRUE) return;
  	  if (bDisplayInfo == FALSE && g_nFlags!=1)
		{
		 if (thinking == 1) SetDlgItemText (sideHWND, 122, "Thinking....");
		 return;
		}
	
	  if (thinking == 2) 
		{
 		 k+= sprintf (buffer + k, "Pondering %c%c-%c%c \n", (g_ponderSrc-22)%10 +'a', '8'-(g_ponderSrc-22)/10,(g_ponderDst-22)%10 +'a', '8'-(g_ponderDst-22)/10);
		}

      GetEvalString (buffer2, CurrentEval );
	  k+= sprintf (buffer+k, "Eval: %s\nKNodes: %d\nDepth: %d", buffer2, (nodes/1000), SearchDepth);

	  src = CurrentBest&255;
      dst = (CurrentBest>>8)&255;
	  GetMoveString (buffer2, src, dst, GameBoard);

	  k+= sprintf (buffer+k, "\nMove: %s", buffer2);

	  nMoves  = MoveStack[ 1 ].nummoves;
	  GetMoveString (buffer2, MSrc(CurrentMove), MDst(CurrentMove), GameBoard);

	  k+= sprintf (buffer+k, "\n(%s) [%d/%d]", buffer2, CurrentMoveNum, nMoves);
	  if (IsPanic () == TRUE) k+= sprintf (buffer+k, "\n[Panic]");

	  if (thinking == 1 && g_ponder == 1)
		{
   		 k+= sprintf (buffer + k, "\n\nanalyzing...");
		}

	  DisplayText ( buffer );
	}

// ----------------------------------
//
// Check For Mate or Stalemate
//
// ----------------------------------
int IsMate ( SBoard &Board, char Color)
{
int u1, u2, i;
Movestruct Moves;
SBoard TempBoard;

if (Color == WHITE)	{FindMovesWhiteCh ( Board, Moves);}
if (Color == BLACK)	{FindMovesBlackCh ( Board, Moves);}

for (i = 1; i<= Moves.nummoves + 50; i++)
	{
	TempBoard = Board;
	if (i == Moves.numAttackmoves + 1) i = 51;
	Do_Move ( TempBoard, Moves.Moves[i], u1, u2, WQUEEN );
	if ( IsCheck (Color, TempBoard) == 0) return 0;
	}

if ( IsCheck (Color, Board) == 0) return 2;
	else return 1;
}

// ----------------------------------
//
// Check For Game Over for 2 Players
//
// ----------------------------------
int CheckGameOver ( SBoard &Board, int bPopUps, int &GameResult )
{
HWND hwnd = mainhwnd;
int mate = IsMate (Board, Board.SideToMove );
if (g_bICC == TRUE || g_bWinBoard == TRUE) bPopUps = FALSE;

			   if (mate == 2) 
					{GameResult = DRAW;
				     if (bPopUps == TRUE) MessageBox( hwnd, " Draw by Stalemate. ", " Game Over! ", MB_OK);
					 return TRUE;
					}
               if (mate == 1) 
					{
					if (Board.SideToMove == BLACK) GameResult = WHITEWINS;
					if (Board.SideToMove == WHITE) GameResult = BLACKWINS;
					if (bPopUps == TRUE) MessageBox( hwnd, " Checkmate ", " Game Over! ", MB_OK);
					return TRUE;
					}
               if ( DrawByRepetition (Board)  > 1 ) 
					{
					 GameResult = DRAW;
					 if (bPopUps == TRUE) MessageBox( hwnd, " Draw by Repetition ", " Game Over! ", MB_OK);
					 return TRUE;
					}
			   if (  (Board.GameTime - Board.LastReversible) >= 100)
					{
					 GameResult = DRAW;
					 if (bPopUps == TRUE) MessageBox( hwnd, " Draw by 50-Move Rule ", " Game Over! ", MB_OK);
					 return TRUE;
					}
				if ((Board.nWPawns == 0 && Board.nBPawns == 0) 
					&& (Board.nBP == 0 || Board.nWP == 0) && (Board.nBP <= 1 && Board.nWP <= 1) 
					&& (Board.nWMinors == Board.nWP && Board.nBMinors == Board.nBP ) )
						{
						GameResult = DRAW;
						if (bPopUps == TRUE) MessageBox( hwnd, " Draw by insuffucient material to mate ", " Game Over! ", MB_OK);
						return TRUE;
						}
	return FALSE;
}
